PRO MAKE_CUT_SUBSET_SIMPL, IMG_UL_X, IMG_UL_Y, IMG_LR_X, IMG_LR_Y, $
                           SUB_UL_X, SUB_UL_Y, SUB_LR_X, SUB_LR_Y, SUB_X_OFFSET, SUB_Y_OFFSET, $
                           X_RES_FL, Y_RES_FL

; ============================================================================
; Purpose: This script is used to overlay an input image onto a subset image. 
;     Both the input is overlayed on the subset image based on it's geographical 
;     coordinates relative to the subset image's geographical coordinates.
;     If the verbose flag is greater than 0, then the script prints the following 
;     information to the standard output:
;     1) The corner pixel coordinates of the input and subset images which overlap 
;        each other. 
;     2) The percentage black pixels in the final output subset image.
;     3) The UL and LR corner map coordinates of the subset image.
;     
;     The script also 'updates' the subset image with the input image, if the 
;     output subset image already exist. 
; 
; Input parameters: 
;    input_img      - The filesystem path to the input image. The input image must 
;        be a PNG image.
;    subset_img     - The filesystem path to the output subset image. The output 
;        subset image can be present. If it is present, it will be updated with 
;        the given input image.
;    img_ul_x       - The input image UL x map coordinate.
;    img_ul_y       - The input image UL y map coordinate.
;    img_lr_x       - The input image LR x map coordinate.
;    img_lr_y       - The input image LR y map coordinate.
;    sub_ul_x       - The subset image UL x map coordinate.
;    sub_ul_y       - The subset image UL y map coordinate.
;    sub_lr_x       - The subset image LR x map coordinate.
;    sub_lr_y       - The subset image LR y map coordinate.
;    sub_x_offset   - The map x refrence point for both the input images.
;    sub_y_offset   - The map y refrence point for both the input images.
;    verbose        - Verbose boolean flag.
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; NOTE: This script regards the Upper Left (LL) part of a raster as the (0, 0) point. The 
; pixel coordinate on the horizontal axis increase to the right, and the vertical pixel 
; coordinate increase towards the bottom.

; =================================================================================================
; Subset image preparation
; =================================================================================================
; Calculate the total number of pixels between the subset UL (x, y) map coordinates and the 
; map refrence point. The following two variables will be in units of pixels and not meters.
sub_ul_x = floor ((sub_ul_x - sub_x_offset) / x_res_fl)
sub_ul_y = ceil ((sub_y_offset - sub_ul_y) / y_res_fl)
; Calculate the total number of pixels between the subset LR (x, y) map coordinates and the 
; map refrence point. The following two variables will be in units of pixels and not meters.
sub_lr_x = floor ((sub_lr_x - sub_x_offset) / x_res_fl)
sub_lr_y = ceil ((sub_y_offset - sub_lr_y) / y_res_fl)

; Calculate the horizontal number of pixels between the UL and LR corners of the subset image. 
; In other words, the number of columns of the output subset image is calculated.
ss_cols_int = (sub_lr_x - sub_ul_x)
; Calculate the vertical number of pixels between the UL and LR corners of the subset image.
; In other words, the number of rows of the output subset image is calculated.
ss_rows_int = (sub_lr_y - sub_ul_y)

; =================================================================================================
; Snapping input image to subset image coordinates
; =================================================================================================
; Calculate the total number of pixels between the input image UL (x, y) map coordinates and the 
; map refrence point. The following two variables will be in the units of pixels and not meters.
img_ul_x = floor ((img_ul_x - sub_x_offset) / x_res_fl)
img_ul_y = ceil ((sub_y_offset - img_ul_y ) / y_res_fl)
; Calculate the total number of pixels between the input image LR (x, y) map coordinates and the 
; map refrence point. The following two variables will be in the units of pixels and not meters.
img_lr_x = floor ((img_lr_x - sub_x_offset) / x_res_fl)
img_lr_y = ceil ((sub_y_offset - img_lr_y ) / y_res_fl)

img_cols_int = (img_lr_x - img_ul_x)
img_rows_int = (img_lr_y - img_ul_y)

; =================================================================================================
; Cutting the input and subset image
; =================================================================================================

; Note: Both the images' rasters now have the same global refrence point. In order words, the two 
;       raster images' grids are implicitly geographically aligned.

; If the input image UL horizontal pixel number is greater than the subset image LR horizontal pixel 
; number, the input image and the subset image do not overlap.
if img_ul_x gt sub_lr_x then begin
    print, "(Warning) Input image and subset image does not overlap horizontally"
    exit
endif

; The input image LR horizontal pixel number is less than the subset image image UL horizontal 
; pixel number, the input image and subset image do not overlap.
if img_lr_x lt sub_ul_x then begin
    print, "(Warning) Input image and subset image does not overlap horizontally"
    exit
endif

help, img_lr_y, sub_ul_y

; The input image LR vertical pixel number is greater than the subset image UL vertical  
; pixel number, the input image and subset image do not overlap.
if img_lr_y lt sub_ul_y then begin
    print, "(Warning) Input image and subset image does not overlap vertically"
    exit
endif

help, img_ul_y, sub_lr_y

; The input image UL vertical pixel number is smaller than the subset image LR vertical  
; pixel number, the input image and subset image do not overlap.
if img_ul_y gt sub_lr_y then begin
    print, "(Warning) Input image and subset image does not overlap vertically"
    exit
endif

; Note: From hence forth, it is assumed that the two images do overlap.

; The following code determines where the input image and the subset image 
; overlap. Where they do overlap, the pixel corner coordinates of the overlap are 
; calculated.

; Case 1: If the input image's UL x corner is to the left of the subset UL x corner, 
;         use the subset UL x corner coordinate as the overlap UL x corner coordinate.
; Else:   If the subset UL x corner coordinate is to the left of the input image UL 
;         x corner, use the input image's UL x corner coordinate as the overlap UL x corner 
;         coordinate.
if img_ul_x lt sub_ul_x then begin
    in_x_b = sub_ul_x
    out_x_b = sub_ul_x
endif else begin
    in_x_b = img_ul_x
    out_x_b = img_ul_x
endelse

; Case 2: If the input image's LR x corner is to the left of the subset LR x corner, 
;         use the image LR x corner coordinate as the overlap LR x corner coordinate.
; Else:   If the subset LR x corner coordinate is to the left of the input image LR 
;         x corner, use the subset image's LR x corner coordinate as the overlap LR x corner 
;         coordinate.
if img_lr_x lt sub_lr_x then begin
    in_x_e = img_lr_x
    out_x_e = img_lr_x
endif else begin
    in_x_e = sub_lr_x
    out_x_e = sub_lr_x
endelse

; Case 3: If the input image's UL y corner is below the subset UL y corner, 
;         use the image LR x corner coordinate as the overlap UL y corner coordinate.
; Else:   If the subset UL y corner coordinate is below the input image UL 
;         y corner, use the subset image's UL y corner coordinate as the overlap LR y corner 
;         coordinate.
if img_ul_y gt sub_ul_y then begin
    in_y_b = img_ul_y
    out_y_b = img_ul_y
endif else begin
    in_y_b = sub_ul_y
    out_y_b = sub_ul_y
endelse

; Case 4: If the subset image's LR y corner is above the image LR y corner, 
;         use the subset image LR y corner coordinate as the overlap LR y corner coordinate.
; Else:   If the image's LR y corner coordinate is above the subset image LR  
;         y corner, use the input image's LR y corner coordinate as the overlap LR y corner 
;         coordinate.
if img_lr_y gt sub_lr_y then begin
    in_y_e = sub_lr_y
    out_y_e = sub_lr_y
endif else begin
    in_y_e = img_lr_y
    out_y_e = img_lr_y
endelse

; For the input image, translate the global corner coordinates of the overlap into the coordinate 
; space of the input image.
r_in_x_b = abs (in_x_b - img_ul_x)
r_in_x_e = abs (in_x_e - img_ul_x)
r_in_y_b = abs (in_y_b - img_ul_y)
r_in_y_e = abs (in_y_e - img_ul_y)

; For the subset image, translate the global corner coordinates of the overlap into the coordinate 
; space of the subset image.
r_out_x_b = abs (out_x_b - sub_ul_x)
r_out_x_e = abs (out_x_e - sub_ul_x)
r_out_y_b = abs (out_y_b - sub_ul_y)
r_out_y_e = abs (out_y_e - sub_ul_y)

; If the verbose flag is greater than 0, print the translated overlap corner coordinates for both 
; the subset and input image.
;if verbose NE 0 then begin
    print, "r_in_x_b: ", r_in_x_b
    print, "r_in_x_e: ", r_in_x_e
    print, "r_in_y_b: ", r_in_y_b
    print, "r_in_y_e: ", r_in_y_e
    print, "img_rows_int: ", img_rows_int
    print, "img_cols_int: ", img_cols_int

    print, "r_out_x_b: ", r_out_x_b
    print, "r_out_x_e: ", r_out_x_e
    print, "r_out_y_b: ", r_out_y_b
    print, "r_out_y_e: ", r_out_y_e
    print, "ss_rows_int: ", ss_rows_int
    print, "ss_cols_int: ", ss_cols_int
;endif

; If the verbose flag is greater than zero, print the following:
; 1) The corner map coordinates, UL and LR, of the subset image.
;if verbose NE 0 then begin
    print, "UL_X:", long (sub_ul_x * x_res_fl + sub_x_offset)
    print, "UL_Y:", long (sub_ul_y * y_res_fl + sub_y_offset)
    print, "LR_X:", long (sub_lr_x * x_res_fl + sub_x_offset)
    print, "LR_Y:", long (sub_lr_y * y_res_fl + sub_y_offset)
;endif

END

