PRO ACS_DESC_FILT_THR, LAT_ARR, LAT_FILL_VAL, SAT_NAME, THR_TRACK_START, DIR_START_BL

; ============================================================================
; Purpose: The script determines at which latitude value, in a given latitude 
;    1-D array, a transition in direction is made. For example, if a satellite 
;    ascends towards to North pole and then descend from the North pole, this 
;    script will find the transition.
; 
; Input parameters: 
;    lat_arr      - The 1-D satellite swath latitude values.
;    lat_fill_val - The fill value that is used in the lat_arr variable.
;    sat_name     - The satellite name. This must all in small letters. One of 
;        the following names must be used: terra, aqua, fy3a, fy3b.
; 
; Output:
;    thr_track_start - The index number at which the latitude values in lat_arr 
;        makes a transition between one direction to another direction.
;    dir_start_bl    - This variable indicates what is the depature point of 
;        the latitude array. If this value is equal to 1, then the start of 
;        the lat_arr array is the depature point. Otherwise it is the end of 
;        the array.
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; ----------------------------------------------------------------------------
; Setup interal script parameters
; ----------------------------------------------------------------------------
; Internal parameter
north_bl = -1

; Set output default values
thr_track_start = -1
dir_start_bl = -1

; The following are indexing parameters. slope_size must < start_stop_val
start_stop_val = 10
slope_size = 10 ; Must be an even number!
; Calculate the half size of 3 * slope_size
slope_hlf_size = slope_size + FIX (slope_size / 2)
; Determine last index until when the array can be indexed
lat_arr_len = N_ELEMENTS (lat_arr)
; Calculate the number of 'slope_size's can fit in the latitude array
factor = (FIX (lat_arr_len) - start_stop_val) / FIX (slope_size * 3)

; Determine what region the latitude data is in, North or South.
if lat_arr [FIX (lat_arr_len / 2)] GT 0 then begin
    north_bl = 1
endif else begin
    north_bl = 0
endelse

; Any sign changes to the slope will be recorded in this array.
slp_change_arr = BYTARR (N_ELEMENTS (lat_arr))

; The start indexing number
idx_start = start_stop_val + slope_hlf_size
; The last indexing number
idx_stop = start_stop_val + slope_hlf_size * (factor - 1)

; ----------------------------------------------------------------------------
; Determine if there are any transition, by calculating the rate of change 
; (the slope) of three equally spaced areas over a sliding window. The sliding 
; window size is equal to the three equally space areas. If there is a sign 
; difference between the first and last area slopes, then a local maxima or 
; minima was reached.
; ----------------------------------------------------------------------------
; Increment index value is slope_size * 2. The indexing can be 
; visualized as the following:
; ---+*+===
;       ---+*+===
;             ---+*+===
;                   ---+*+===
; The slopes are calculated over over '---' and '===' regions. If the signs of 
; the two slopes differ, the slope change is recorded at the '*' index.
; From one '*' to the next '*', is equal to slope_size * 2. The size of '---' 
; and '===' is equal to slope_size.
for cnr_idx = idx_start, idx_stop, (slope_size * 2) do begin
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ; Calculate pre slope
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    idx_b = cnr_idx - slope_hlf_size
    idx_e = idx_b + slope_size - 1
    idx_1b = idx_b
    
    tmp_val = lat_arr [idx_b:idx_e]
    
    ; If there is any fill values, skip
    if TOTAl (tmp_val EQ lat_fill_val) GT 0 then begin
        continue
    endif
    
    ; First order differentiate
    tmp_diff_val = tmp_val - shift (tmp_val, 1)
    tmp_diff_val = tmp_diff_val [1:(slope_size - 1)]
    ; Caculate mean valaue of differentiated values
    mean_pre_slpe = MEAN (tmp_diff_val)

    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ; Calculate post slope
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    idx_b = cnr_idx + FIX (slope_size / 2)
    idx_e = idx_b + slope_size - 1
    idx_2b = idx_b

    tmp_val = lat_arr [idx_b:idx_e]
    
    ; If there is any fill values, skip
    if TOTAl (tmp_val EQ lat_fill_val) GT 0 then begin
        continue
    endif
    
    tmp_diff_val = tmp_val - shift (tmp_val, 1)
    tmp_diff_val = tmp_diff_val [1:(slope_size - 1)]
    mean_pst_slpe = MEAN (tmp_diff_val)

    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ; Calculate middle slope
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    idx_b = cnr_idx - FIX (slope_size / 2)
    idx_e = idx_b + slope_size - 1
    idx_3b = idx_b

    tmp_val = lat_arr [idx_b:idx_e]
    
    ; If there is any fill values, skip
    if TOTAl (tmp_val EQ lat_fill_val) GT 0 then begin
        continue
    endif
    
    tmp_diff_val = tmp_val - shift (tmp_val, 1)
    tmp_diff_val = tmp_diff_val [1:(slope_size - 1)]
    mean_mdl_slpe = MEAN (tmp_diff_val)
    
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ; Record all the cases where there are slope changes
    ; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ 
    test_val = mean_pre_slpe * mean_pst_slpe
    if test_val LT 0 then begin
        slp_change_arr [cnr_idx] = 1
    endif
endfor

; ----------------------------------------------------------------------------
; Find all the recorded slope changes. If there are no slope changes, 
; determine if the data is ascending or descending.
; ----------------------------------------------------------------------------
idx_slp_change = where (slp_change_arr EQ 1)

; There should be only one slope change. If there are none, 
; determine if the data are ascending or descending
idx_slp_change_bl = 1
if N_ELEMENTS (idx_slp_change) EQ 1 then begin
    if idx_slp_change EQ -1 then idx_slp_change_bl = 0
endif else begin
    ; If there are more than one slope change, something could be wrong. Print 
    ; a warning
    print, "(WARNING) There are more than one slope change."
endelse

; Ascending data boolean. If the data are ascending, set to 1. Otherwise 0
asc_data_bl = 1
diff_data = lat_arr - shift (lat_arr, 1)
data_mdl_idx = FIX (lat_arr_len / 2)
mdl_mean_slope = mean (diff_data [(data_mdl_idx - 10):(data_mdl_idx + 10)])
if mdl_mean_slope LT 0 then begin
    asc_data_bl = 0
endif 

; ----------------------------------------------------------------------------
; Based on the satellite name and the region (North or South), set the output 
; variables.
; ----------------------------------------------------------------------------
if idx_slp_change_bl EQ 1 then begin
    if (sat_name EQ 'terra') OR (sat_name EQ 'fy3a') then begin
        if north_bl EQ 1 then begin
            ; The data start with ascending order, and this is 'night time' data. 
            ; Thus start to remove data from last latitude direction change of 
            ; the swath towards the start of the swath
            dir_start_bl = 1
            thr_track_start = idx_slp_change [N_ELEMENTS (idx_slp_change) - 1]
        endif else begin
            ; The data start with descending order, and this is 'day time' data. 
            ; Thus start to remove data from first latitude direction change of 
            ; the swath towards the end of the swath
            dir_start_bl = 0
            thr_track_start = idx_slp_change [0]
        endelse
    endif
    
    if (sat_name EQ 'aqua') OR (sat_name EQ 'fy3b') OR (sat_name EQ 'npp') then begin
        if north_bl EQ 1 then begin
            ; The data start with ascending order, and this is 'day time' data. 
            ; Thus start to remove data from first latitude direction change of 
            ; the swath towards the end of the swath
            dir_start_bl = 0
            thr_track_start = idx_slp_change [0]
        endif else begin
            ; The data start with descending order, and this is 'night time' data. 
            ; Thus start to remove data from last latitude direction change of 
            ; the swath towards the start of the swath
            dir_start_bl = 1
            thr_track_start = idx_slp_change [N_ELEMENTS (idx_slp_change) - 1]
        endelse
    endif
endif else begin
    if (sat_name EQ 'terra') OR (sat_name EQ 'fy3a') then begin
        ; If the data is ascending, remove all the data
        if asc_data_bl EQ 1 then begin
            dir_start_bl = 1
            thr_track_start = lat_arr_len - 1
        endif
    endif
    
    if (sat_name EQ 'aqua') OR (sat_name EQ 'fy3b') OR (sat_name EQ 'npp') then begin
        ; If the data is descending, remove all the data
        if asc_data_bl EQ 0 then begin
            dir_start_bl = 1
            thr_track_start = lat_arr_len - 1
        endif
    endif
endelse

END

