PRO ASC_DESC_FILT_BINFILE, GEO_FILE, BIN_FILE, OUT_FILL_VAL, SAT_NAME, WIDTH, LENGTH, TYPE

; ============================================================================
; Purpose: Morning equator crossing satellites (Terra, FY3A), for example, do 
;     have day light data at the polar region when in ascending mode. Normally 
;     only day light data are in descending mode. The day light data that are 
;     in the ascending mode is of low quality. This script replaces these data 
;     with a given fill value (out_fill_val). The same is done for afternoon 
;     equator crossing satellites (Aqua, FY3B).
; 
; Input parameters:
;     geo_file     - The L1B HDF4 geolocation file of the sensor.
;     bin_file     - The binary data file which will be filtered.
;     out_fill_val - The value that will be used to replace 'low quality' 
;         solar data in the file 'bin_file'
;     sat_name - The name of the satellite of the data. One of the following 
;         names must be used: terra, aqua, fy3a, fy3b.
;     width        - The number of columns in the raster file.
;     length       - The number of rows in the raster file.
;     type         - The data type in the raster file.
; 
; Output parameters:
;     bin_file - This is the same file from which the data are read. The content 
;         in this file is replaced by the filtered data.
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; ----------------------------------------------------------------------------
; Get the latitude data from the geolocation file
; ----------------------------------------------------------------------------
hdfid = hdf_sd_start (geo_file)

hdf_sd_varread, hdfid, 'Latitude', lat_arr
index_sds = hdf_sd_nametoindex (hdfid, 'Latitude')
sds_varid = hdf_sd_select (hdfid, index_sds)

; Get the fill value of the latitude array
attindex = hdf_sd_attrfind (sds_varid, '_FillValue')
hdf_sd_attrinfo, sds_varid, attindex, data = sds_fillvalue
if N_ELEMENTS (sds_fillvalue GT 1) then begin
    sds_fillvalue = sds_fillvalue [0]
endif

hdf_sd_endaccess, sds_varid
hdf_sd_end, hdfid

; ----------------------------------------------------------------------------
; Get the data from the file bin_file
; ----------------------------------------------------------------------------
openr, lun, bin_file, /GET_LUN
data_arr = make_array (width, length, TYPE=type)
readu, lun, data_arr
close, lun

; ----------------------------------------------------------------------------
; Filter the data based on the latitude data
; ----------------------------------------------------------------------------
acs_desc_filt_data, lat_arr, sds_fillvalue, sat_name, data_arr, out_fill_val

; ----------------------------------------------------------------------------
; Write the filted data to the file bin_file
; ----------------------------------------------------------------------------
openw, lun, bin_file, /GET_LUN
writeu, lun, data_arr
close, lun

END

