PRO FILTER_HIGH_SENSOR_ZANGLE_BIN, CREFL_F, SEN_ZEN_F, C_COLS_N, C_ROWS_N, S_COLS_N, S_ROWS_N, $
                                   CREFL_DTYPE, SEN_ZEN_DTYPE, CREFL_FILL_VAL, SEN_ZEN_FILL_VAL, $
                                   MAX_SENS_ZANGL, SCALE_VALUE

; ============================================================================
; Purpose: This script is used to filter high sensor view zenith angle 
;     pixels. Pixels that are of low quality have high view zenith angles. 
;     These pixels values are replaced by fill values. The swath width is 
;     therefore decreased.
; 
; Input parameters: 
;    crefl_f      - An array of CREFL binary file names. All the CREFL binary 
;        files that must be filtered, must be in this array of strings. 
;    sen_zen_f        - The sensor zenith view angle binary file.
;    c_cols_n         - The number of columns in the CREFL binary files.
;    c_rows_n         - The number of rows in the CREFL binary files.
;    s_cols_n         - The number of columns in the sensor zenith view angle 
;        binary file.
;    s_rows_n         - The number of rows in the sensor zenith view angle 
;        binary file.
;    crefl_dtype      - The CREFL binary file data type.
;    sen_zen_dtype    - The sensor zenith view angle binary file data type.
;    crefl_fill_val   - The fill value that is used in the CREFL binary files.
;    sen_zen_fill_val - The fill value that is used in the sensor zenith 
;        view angle binary file.
;    max_sens_zangl   - The threshold sensor view zenith angle value.
; 
; Output:
;    data - The files indicated in the array crefl_f will be modified.
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; Assume that CREFL data will be of higher resolution

openr, sen_zen_lun, sen_zen_f, /GET_LUN
sen_zen_arr = make_array (s_cols_n, s_rows_n, TYPE=sen_zen_dtype)
readu, sen_zen_lun, sen_zen_arr
free_lun, sen_zen_lun

sen_zen_arr = regrid (temporary (sen_zen_arr), c_cols_n, c_rows_n)

; Create an array index where the SensorZenith angle greater than
; MAX_SENS_ZANGL. This array index will be used to set the longitude and
; latitude values, which are associated with high SensorZenith angles, to fill
; values.
filter_index = where (sen_zen_arr GT (max_sens_zangl * scale_value))

if N_ELEMENTS (filter_index) EQ 1 then begin
    if filter_index EQ -1 then begin
        print, "(Warning) There are no sensor zenith angle values greater than ", max_sens_zangl
        free_lun, sen_zen_lun
        exit
    endif
endif

openr, crefl_lun, crefl_f, /GET_LUN
crefl_arr = make_array (c_cols_n, c_rows_n, TYPE=crefl_dtype)
readu, crefl_lun, crefl_arr
close, crefl_lun

openw, crefl_lun, crefl_f, /GET_LUN
crefl_arr [filter_index] = crefl_fill_val
writeu, crefl_lun, crefl_arr
flush, crefl_lun
close, crefl_lun

END

