PRO HDF_TO_BINARY, HDF_FILE_STR, SDS_NAME_STR, SCALE, OUT_PREFIX_STR

; ============================================================================
; Purpose: This script is used to convert a SDS array in a HDF4 file into a 
;     raster binary file. If geolocation data in a HDF4 file is converted into 
;     a raster binary file, this script can also be used to scale the geolocation 
;     data.
; 
; Input parameters: 
;     1. The L1B HDF4 file name.
;     2. The SDS array name in the L1B HDF4 file that is going to be converted 
;        into a raster binary file.
;     3. If the geolocation data need to be scaled, a scale value of either 1, 
;        2 or 4 can be given. The scale value indicates by how many multiples 
;        the geolocation data resolution must be increased.
;     4. A prefix string that will be added to the output file name.
; Output:
;     The output will be a raster binary file. The file name will include 
;     the specifications of the binary file. There are four fields, delimited  
;     by the '_' character. The fields are are follows:
;     <output prefix string>_<nr of columns>_<nr of rows>_<IDL data type>.dat
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; Convert a SDS in a HDF file to a binary file. The output binary file will start with the 
; given prefix string (OUT_PREFIX_STR), the dimensions will be placed in the file name and the 
; data type of the SDS array. 
; The output file name format will be:
;   out_prefix_str_<nr of columns>_<nr of rows>_<IDL data type nr>.dat

; Open the HDF file and read in the SDS data
hdfid = hdf_sd_start (hdf_file_str)
hdf_sd_varread, hdfid, sds_name_str, sds_arr
hdf_sd_end, hdfid

dims = size (sds_arr, /dimensions)

if scale EQ 4 then $
    sds_arr_out = modis_geo_interp_250 (sds_arr)
if scale EQ 2 then $
    sds_arr_out = modis_geo_interp_500 (sds_arr)
if scale EQ 1 then $
    sds_arr_out = sds_arr

; Get the dimensions of the SDS array
dimm_arr = size (sds_arr_out, /dimensions)
; Get the data type
data_type_int = size (sds_arr_out, /type)
; Construct the output file name
out_file_name_str = out_prefix_str + "_" + $
                    strtrim (string (dimm_arr [0]), 1) + "_" + $
                    strtrim (string (dimm_arr [1]), 1) + "_" + $
                    strtrim (string (data_type_int), 1) + ".dat"

; Write data to output binary file
openw, lun, out_file_name_str, /get_lun
writeu, lun, sds_arr_out
free_lun, lun

END
