PRO IMAGE_SZANGLE_COMPOSITE, IMG_ARRAY, SZENANGLE_ARRAY, OUTFILE, MISSING=MISSING

; ============================================================================
; Purpose: Create a composite image from a series of image tiles. Each tile's 
;     corresponding sensor zenith angle data are used to choose the best pixels
;     that should be placed in the output composite image file. If only one
;     input image is available, the output image is identical to the input
;     image.
;     
;     The IDL procedurial script 'overlay_szangle_image.pro' is used to to 
;     overlay the given image tiles. This script makes a composite of the 
;     the image tiles and the sensor zenith angle data.
; 
; Input parameters:
;     img_array - An array of strings. Each string is a filesystem path of 
;         a image tile.
;     szenangle_array - An array of strings. Each string is a filesystem 
;         path to a sensor zenith angle binary file, which are associated 
;         with the given image tiles.
;     outfile - The filesystem path of the output file.
;     missing - A three element array. Each element represent the missing 
;         values for the red, green and blue bands.
; 
; Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; Date: 07/22/2011
; 
; Change log:
; 10/25/2011 - The sensor zenith angle data were always written out to binary files, 
;              and the typical size of the files were 23MB. To decrease the size of 
;              the files, the files are now written in a PNG format. 
; ============================================================================

; Check arguments
if (n_elements (img_array) eq 0) then message, 'Argument IMG_ARRAY is undefined'
if (n_elements (szenangle_array) eq 0) then message, 'Argument SZENANGLE_ARRAY is undefined'
if (n_elements (outfile) eq 0) then message, 'Argument OUTFILE is undefined'
if (n_elements (missing) eq 0) then missing = [0B, 0B, 0B]

; Read first input image
image_out = read_png (img_array [0])
; Get the dimensions of the input image
image_dim_arr = size (image_out, /DIMENSIONS)
; Given that the input image has three layers (red, green and blue bands), the 
; other two array elements in the image_dim_arr array represent the column and 
; row values of the input image.
szenangle_dim_arr = [image_dim_arr [1], image_dim_arr [2]]

; If more than one input file was supplied, overlay the images in turn.
if (n_elements (img_array) gt 1) then begin
    ; Create an array in which composite sensor zenith angle value will be 
    ; stored.
    image_out_szenangle = make_array (szenangle_dim_arr, TYPE = 2)
    ; Create an array in which the input image's sensor zenith angle 
    ; value will be stored.
    szenangle_tmp_arr = make_array (szenangle_dim_arr, TYPE = 2)
    
    ; Get the first image tile's sensor zenith angle data.
    ;openr, lun, szenangle_array [0], /GET_LUN
    ;readu, lun, image_out_szenangle
    ;close, lun
    ;image_out_szenangle = reverse (temporary (image_out_szenangle), 2)
    ;image_out_szenangle = read_png (szenangle_array [0])
    openr, lun, szenangle_array [0], /GET_LUN, /COMPRESS
    readu, lun, image_out_szenangle
    close, lun
    free_lun, lun
    
    image_out_szenangle = reverse (temporary (image_out_szenangle), 2)
    
    ; Overlay the rest of the image tiles over the first image tile.
    for index = 1, n_elements (img_array) - 1 do begin
        ;openr, lun, szenangle_array [index], /GET_LUN
        ;readu, lun, szenangle_tmp_arr
        ;close, lun
        ;szenangle_tmp_arr = reverse (temporary (szenangle_tmp_arr), 2)
        ;szenangle_tmp_arr = read_png (szenangle_array [index])
        openr, lun, szenangle_array [index], /GET_LUN, /COMPRESS
        readu, lun, szenangle_tmp_arr
        close, lun
        free_lun, lun
        
        szenangle_tmp_arr = reverse (temporary (szenangle_tmp_arr), 2)
        
        overlay_szangle_image, image_out, read_png (img_array [index]), $
            szenangle_tmp_arr, image_out_szenangle, missing=missing
    endfor
endif

; Write output file.
write_png, outfile, image_out

END
