PRO MAKE_CUT_SUBSET, INPUT_IMG, SUBSET_IMG, IMG_UL_X, IMG_UL_Y, IMG_LR_X, IMG_LR_Y, $
                     SUB_UL_X, SUB_UL_Y, SUB_LR_X, SUB_LR_Y, SUB_X_OFFSET, SUB_Y_OFFSET, $
                     X_RES_FL, Y_RES_FL, VERBOSE, OUTPUT_FRMT_STR

; ============================================================================
; Purpose: This script is used to overlay an input image onto a subset image. 
;     Both the input is overlayed on the subset image based on it's geographical 
;     coordinates relative to the subset image's geographical coordinates.
;     If the verbose flag is greater than 0, then the script prints the following 
;     information to the standard output:
;     1) The corner pixel coordinates of the input and subset images which overlap 
;        each other. 
;     2) The percentage black pixels in the final output subset image.
;     3) The UL and LR corner map coordinates of the subset image.
;     
;     The script also 'updates' the subset image with the input image, if the 
;     output subset image already exist. 
; 
; Input parameters: 
;    input_img      - The filesystem path to the input image. The input image must 
;        be a PNG image.
;    subset_img     - The filesystem path to the output subset image. The output 
;        subset image can be present. If it is present, it will be updated with 
;        the given input image.
;    img_ul_x       - The input image UL x map coordinate.
;    img_ul_y       - The input image UL y map coordinate.
;    img_lr_x       - The input image LR x map coordinate.
;    img_lr_y       - The input image LR y map coordinate.
;    sub_ul_x       - The subset image UL x map coordinate.
;    sub_ul_y       - The subset image UL y map coordinate.
;    sub_lr_x       - The subset image LR x map coordinate.
;    sub_lr_y       - The subset image LR y map coordinate.
;    sub_x_offset   - The map x refrence point for both the input images.
;    sub_y_offset   - The map y refrence point for both the input images.
;    verbose        - Verbose boolean flag.
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; NOTE: This script regards the Upper Left (LL) part of a raster as the (0, 0) point. The 
; pixel coordinate on the horizontal axis increase to the right, and the vertical pixel 
; coordinate increase towards the bottom.

; Set the pixel resolution (meters) of the x and y axises of the grid.
;x_res_fl=250.0
;y_res_fl=250.0

; =================================================================================================
; Subset image preparation
; =================================================================================================
; Calculate the total number of pixels between the subset UL (x, y) map coordinates and the 
; map refrence point. The following two variables will be in units of pixels and not meters.
sub_ul_x = floor ((sub_ul_x - sub_x_offset) / x_res_fl)
sub_ul_y = floor ((sub_ul_y - sub_y_offset) / y_res_fl)
; Calculate the total number of pixels between the subset LR (x, y) map coordinates and the 
; map refrence point. The following two variables will be in units of pixels and not meters.
sub_lr_x = floor ((sub_lr_x - sub_x_offset) / x_res_fl)
sub_lr_y = floor ((sub_lr_y - sub_y_offset) / y_res_fl)

adjust_map_xy_coord, sub_ul_x, sub_ul_y, sub_lr_x, sub_lr_y, $
    sub_x_offset, sub_y_offset, x_res_fl, y_res_fl, $
    adj_ul_x, adj_ul_y, adj_lr_x, adj_lr_y

; Calculate the horizontal number of pixels between the UL and LR corners of the subset image. 
; In other words, the number of columns of the output subset image is calculated.
ss_cols_int = (sub_lr_x - sub_ul_x)
; Calculate the vertical number of pixels between the UL and LR corners of the subset image.
; In other words, the number of rows of the output subset image is calculated.
ss_rows_int = (sub_ul_y - sub_lr_y)

; Create the subset image array.
ss_img_arr = bytarr (3, ss_cols_int, ss_rows_int)

; If a previous version of the subset image already exist, read the subset image and copy the 
; image data into the array ss_img_arr.
result = file_test (subset_img)
if result EQ 1 then begin
    ; First check if the given subset image is corrupt. If it is not corrupt, read it. 
    if output_frmt_str EQ "tif" then begin
        result = query_tiff (subset_img)
        
        if result EQ 1 then $
            ss_img_arr [*, *, *] = read_tiff (subset_img)
    endif else begin
        result = query_png (subset_img)
        
        if result EQ 1 then begin
            ss_img_arr [*, *, *] = read_png (subset_img)
            ss_img_arr = reverse (temporary (ss_img_arr), 3)
        endif
    endelse
endif

; =================================================================================================
; Input image preparation
; =================================================================================================
; Read the input image into an array
if output_frmt_str EQ "tif" then begin
    in_img_arr = read_tiff (input_img)
endif else begin
    in_img_arr = read_png (input_img)
    ; The refrence point for PNGs differ from raster images in IDL. The refrence point for an image raster 
    ; in IDL is the LL corner. For images it is normally the UL corner.
    in_img_arr = reverse (temporary (in_img_arr), 3)
endelse

; Get the dimension sizes of the input image.
img_nr_col_l = (size (in_img_arr, /DIMENSIONS))[1]
img_nr_row_l = (size (in_img_arr, /DIMENSIONS))[2]

; =================================================================================================
; Snapping input image to subset image coordinates
; =================================================================================================
; Calculate the total number of pixels between the input image UL (x, y) map coordinates and the 
; map refrence point. The following two variables will be in the units of pixels and not meters.
img_ul_x = floor ((img_ul_x - sub_x_offset) / x_res_fl)
img_ul_y = floor ((img_ul_y - sub_y_offset) / y_res_fl)
; Calculate the total number of pixels between the input image LR (x, y) map coordinates and the 
; map refrence point. The following two variables will be in the units of pixels and not meters.
img_lr_x = floor ((img_lr_x - sub_x_offset) / x_res_fl)
img_lr_y = floor ((img_lr_y - sub_y_offset) / y_res_fl)

; =================================================================================================
; Cutting the input and subset image
; =================================================================================================

; Note: Both the images' rasters now have the same global refrence point. In order words, the two 
;       raster images' grids are implicitly geographically aligned.

; If the input image UL horizontal pixel number is greater than the subset image LR horizontal pixel 
; number, the input image and the subset image do not overlap.
if img_ul_x gt sub_lr_x then begin
    print, "(Warning) Input image and subset image does not overlap horizontally"
    exit
endif

; The input image LR horizontal pixel number is less than the subset image image UL horizontal 
; pixel number, the input image and subset image do not overlap.
if img_lr_x lt sub_ul_x then begin
    print, "(Warning) Input image and subset image does not overlap horizontally"
    exit
endif

; The input image LR vertical pixel number is greater than the subset image UL vertical  
; pixel number, the input image and subset image do not overlap.
if img_lr_y gt sub_ul_y then begin
    print, "(Warning) Input image and subset image does not overlap vertically"
    exit
endif

; The input image UL vertical pixel number is smaller than the subset image LR vertical  
; pixel number, the input image and subset image do not overlap.
if img_ul_y lt sub_lr_y then begin
    print, "(Warning) Input image and subset image does not overlap vertically"
    exit
endif

; Note: From hence forth, it is assumed that the two images do overlap.

; The following code determines where the input image and the subset image 
; overlap. Where they do overlap, the pixel corner coordinates of the overlap are 
; calculated.

; Case 1: If the input image's UL x corner is to the left of the subset UL x corner, 
;         use the subset UL x corner coordinate as the overlap UL x corner coordinate.
; Else:   If the subset UL x corner coordinate is to the left of the input image UL 
;         x corner, use the input image's UL x corner coordinate as the overlap UL x corner 
;         coordinate.
if img_ul_x lt sub_ul_x then begin
    in_x_b = sub_ul_x
    out_x_b = sub_ul_x
endif else begin
    in_x_b = img_ul_x
    out_x_b = img_ul_x
endelse

; Case 2: If the input image's LR x corner is to the left of the subset LR x corner, 
;         use the image LR x corner coordinate as the overlap LR x corner coordinate.
; Else:   If the subset LR x corner coordinate is to the left of the input image LR 
;         x corner, use the subset image's LR x corner coordinate as the overlap LR x corner 
;         coordinate.
if img_lr_x lt sub_lr_x then begin
    in_x_e = img_lr_x - 1
    out_x_e = img_lr_x - 1
endif else begin
    in_x_e = sub_lr_x - 1
    out_x_e = sub_lr_x - 1
endelse

; Case 3: If the input image's UL y corner is below the subset UL y corner, 
;         use the image LR x corner coordinate as the overlap UL y corner coordinate.
; Else:   If the subset UL y corner coordinate is below the input image UL 
;         y corner, use the subset image's UL y corner coordinate as the overlap LR y corner 
;         coordinate.
if img_ul_y lt sub_ul_y then begin
    in_y_b = img_ul_y
    out_y_b = img_ul_y
endif else begin
    in_y_b = sub_ul_y
    out_y_b = sub_ul_y
endelse

; Case 4: If the subset image's LR y corner is above the image LR y corner, 
;         use the subset image LR y corner coordinate as the overlap LR y corner coordinate.
; Else:   If the image's LR y corner coordinate is above the subset image LR  
;         y corner, use the input image's LR y corner coordinate as the overlap LR y corner 
;         coordinate.
if img_lr_y lt sub_lr_y then begin
    in_y_e = sub_lr_y + 1
    out_y_e = sub_lr_y + 1
endif else begin
    in_y_e = img_lr_y + 1
    out_y_e = img_lr_y + 1
endelse

; For the input image, translate the global corner coordinates of the overlap into the coordinate 
; space of the input image.
r_in_x_b = abs (in_x_b - img_ul_x)
r_in_x_e = abs (in_x_e - img_ul_x)
r_in_y_b = abs (in_y_b - img_ul_y)
r_in_y_e = abs (in_y_e - img_ul_y)

; For the subset image, translate the global corner coordinates of the overlap into the coordinate 
; space of the subset image.
r_out_x_b = abs (out_x_b - sub_ul_x)
r_out_x_e = abs (out_x_e - sub_ul_x)
r_out_y_b = abs (out_y_b - sub_ul_y)
r_out_y_e = abs (out_y_e - sub_ul_y)

; If the verbose flag is greater than 0, print the translated overlap corner coordinates for both 
; the subset and input image.
if verbose NE 0 then begin
    print, "r_in_x_b: ", r_in_x_b
    print, "r_in_x_e: ", r_in_x_e
    print, "r_in_y_b: ", r_in_y_b
    print, "r_in_y_e: ", r_in_y_e

    print, "r_out_x_b: ", r_out_x_b
    print, "r_out_x_e: ", r_out_x_e
    print, "r_out_y_b: ", r_out_y_b
    print, "r_out_y_e: ", r_out_y_e
endif

; Where the input image overlap the subset image, copy the input image data onto the subset image.
ss_img_arr [*, r_out_x_b:r_out_x_e, r_out_y_b:r_out_y_e] = in_img_arr [*, r_in_x_b:r_in_x_e, r_in_y_b:r_in_y_e]

if output_frmt_str EQ "tif" then begin
    ; For some odd reason, the 'bottom' of a tif is the top of an image. So to have the correct orientation, the 
    ; image rows has to be reversed.
    write_tiff, subset_img, ss_img_arr, compression=1
endif else begin
    ss_img_arr = reverse (temporary (ss_img_arr), 3)
    write_png, subset_img, ss_img_arr
endelse

; If the verbose flag is greater than zero, print the following:
; 1) The corner map coordinates, UL and LR, of the subset image.
; 2) The percentage of black pixels in the subset image.
if verbose NE 0 then begin
    total_black_pix = float (TOTAL (ss_img_arr EQ 0))
    total_pixels = float (N_ELEMENTS (ss_img_arr))
    per_total_black_pix = LONG ((total_black_pix / total_pixels) * 100.0)
    
    sub_mdl_x = sub_ul_x + (sub_lr_x - sub_ul_x) / 2.0
    sub_mdl_y = sub_lr_y + (sub_ul_y - sub_lr_y) / 2.0   
    
    print, "Percentage black pixels:", per_total_black_pix
    ;print, "UL_X:", long (sub_ul_x * x_res_fl + sub_x_offset)
    ;print, "UL_Y:", long (sub_ul_y * y_res_fl + sub_y_offset)
    ;print, "LR_X:", long (sub_lr_x * x_res_fl + sub_x_offset)
    ;print, "LR_Y:", long (sub_lr_y * y_res_fl + sub_y_offset)
    print, "UL_X:", adj_ul_x
    print, "UL_Y:", adj_ul_y
    print, "LR_X:", adj_lr_x
    print, "LR_Y:", adj_lr_y
    print, "MD_X:", long (sub_mdl_x * x_res_fl + sub_x_offset)
    print, "MD_Y:", long (sub_mdl_y * y_res_fl + sub_y_offset)
endif

END

