pro make_global_kml,sensor_name,kml_base_path
;---------------------------------------------------------------------
; Program to make a single .kml file that points to the top level
; super overlay images/kml for each of the tiles in the global
; MODIS imagery.  The global tiles are numbered from 36 through 611 
; (576 in total).  This program must be executed each time any tile is
; updated to check for new links, missing data, etc.
;
; INPUT:
;  sensor_name: This is the name of the new MODIS image, only needed to
;   determine the year/jday being considered will be made.
;
; OUTPUT:
;  A kml file that will link to all of the top level super overlay
;  files and imagery.  Naming convention will be date + satellite,
;  e.g., 2008213.fy3b.kml or 2008213.fy3a.kml.
;
; HARDWIRED:
;  Path to the superoverlay tiles (e.g., '../tiles/2008213/')
;
; AUTHOR: Amato Evan - CIMSS/UW-Madison amatoe@ssec.wisc.edu
;         Apapted by Willem.Marais@ssec.wisc.edu
; CHANGES:
;  30 December 2010 - When the script was suppose to write tile000 at 
;                     the href tags, it had written tile0. This has 
;                     been corrected
;---------------------------------------------------------------------

;---------------------------------------------------------------------
; Define the path to the super overlay folders for this day
;---------------------------------------------------------------------
satellite_id = strmid(sensor_name,8,4)
if(satellite_id eq 'terr')then satellite_id = 'terra'
if(satellite_id eq 'npp.')then satellite_id = 'npp'

;date = strmid(sensor_name,strpos(sensor_name,'.jpg')-22,8)
;band_comb = strmid(sensor_name,strpos(sensor_name,'.jpg')-13,11)
sat_name_str_len = strlen (satellite_id)
sat_name_str_pos = strpos (sensor_name, satellite_id)
date = strmid (sensor_name, sat_name_str_pos + sat_name_str_len + 1, 8)
band_comb = strmid (sensor_name, sat_name_str_pos + sat_name_str_len + 1 + 8 + 1, 14)
;kml_control_path = '../kml/'+date+'/'+band_comb+'/'
kml_control_path = kml_base_path + '/'+date+'/'+band_comb+'/'
kml_name = date+'.'+band_comb+'.'+satellite_id+'.kml'

;---------------------------------------------------------------------
; Determine which of the global tiles have imagery to be linked to
;---------------------------------------------------------------------
;global_tiles = file_search(kml_control_path,'*'+satellite_id)
spawn, 'find '+kml_control_path+' -name "tile*'+satellite_id+'" -exec basename {} \; | sort', global_tiles
n_tiles = n_elements(global_tiles)
id_tiles = intarr(n_tiles)
for i = 0, n_tiles-1 do id_tiles[i] = 1*strmid(global_tiles[i],4,3)

;---------------------------------------------------------------------
; Define the MODIS tiles lat/lon limits
;---------------------------------------------------------------------
region_coords = dblarr(n_tiles,4)
gridindex_to_latlon,id_tiles,10,lat,lon
region_coords[*,0] = lat
region_coords[*,1] = lon
region_coords[*,2] = temporary(lat)+10.
region_coords[*,3] = temporary(lon)+10.

;---------------------------------------------------------------------
; Make the kml file that will point to all the top level super overlay
; images.
;---------------------------------------------------------------------
openw,u,kml_control_path+kml_name,/get_lun

;write the opening lines
printf,u,'<?xml version="1.0" encoding="UTF-8"?>'
printf,u,'<kml xmlns="http://earth.google.com/kml/2.2">'
printf,u,'  <Document>'
printf,u,'    <Region>'
printf,u,'      <Lod>'
printf,u,'        <minLodPixels>128</minLodPixels><maxLodPixels>-1</maxLodPixels>'
printf,u,'      </Lod>'
printf,u,'      <LatLonAltBox>'
printf,u,'        <north>'+strcompress(max(region_coords[*,2]))+'</north>'
printf,u,'        <south>'+strcompress(min(region_coords[*,0]))+'</south>'
printf,u,'        <east>'+strcompress(max(region_coords[*,3]))+'</east>'
printf,u,'        <west>'+strcompress(min(region_coords[*,1]))+'</west>'
printf,u,'      </LatLonAltBox>'
printf,u,'    </Region>'

;---------------------------------------------------------------------
; loop through and link to sub-directories
;---------------------------------------------------------------------
for i = 0, n_tiles-1 do begin

  ;-------------------------------------------------------------------
  ; Determine the "uppermost" kml super overlay tile by doing a search
  ;-------------------------------------------------------------------
  tile_name = strcompress(id_tiles[i],/remove_all)
  if(id_tiles[i] lt 10)then tile_name = '00'+ $
                                  strcompress(id_tiles[i],/remove_all)
  if(id_tiles[i] ge 10 and id_tiles[i] lt 100)then $
                  tile_name = '0'+strcompress(id_tiles[i],/remove_all)
  full_path = kml_control_path+'tile'+tile_name+'_'+satellite_id+'/'

  tmp = file_search(full_path,'*.kml')
  tiles = intarr(n_elements(tmp))
  for s = 1, n_elements(tmp)-1 do begin
    ;tnum = strmid(tmp[s],strlen(tmp[s])-7,3)
    ;if(strmid(tnum,0,1) eq '/' or strmid(tnum,0,1) eq 'a')then tnum = strmid(tnum,1,2)
    ;if(strmid(tnum,0,1) eq '/')then tnum = strmid(tnum,1,1)
    last_pos_path = strpos (tmp [s], '/', /REVERSE_SEARCH)
    last_pos_perr = strpos (tmp [s], '.', /REVERSE_SEARCH)

    tnum = strmid (tmp [s], last_pos_path + 1, last_pos_perr - last_pos_path - 1)
    
    tiles[s] = tnum*1
  endfor

  upper_tile = strcompress(max(tiles),/remove_all)
  if(max(tiles) lt 10)then upper_tile = '00'+ $
                                   strcompress(max(tiles),/remove_all)
  if(max(tiles) ge 10 and max(tiles) lt 100)then $ 
                  upper_tile = '0'+strcompress(max(tiles),/remove_all)
  ;-------------------------------------------------------------------

  printf,u,'    <NetworkLink>'
  printf,u,'      <name>'+upper_tile+'</name>'
  printf,u,'      <Region>'
  printf,u,'        <Lod>'
  printf,u,'          <minLodPixels>128</minLodPixels><maxLodPixels>-1</maxLodPixels>'
  printf,u,'        </Lod>'
  printf,u,'        <LatLonAltBox>'
  printf,u,'          <north>'+strcompress(region_coords[i,2])+'</north>'
  printf,u,'          <south>'+strcompress(region_coords[i,0])+'</south>'
  printf,u,'          <east>'+strcompress(region_coords[i,3])+'</east>'
  printf,u,'          <west>'+strcompress(region_coords[i,1])+'</west>'
  printf,u,'        </LatLonAltBox>'
  printf,u,'      </Region>'
  printf,u,'      <Link>'
  
  if (id_tiles[i] lt 10) then $
    printf,u,'        <href>tile00'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.kml</href>'
  if (id_tiles[i] ge 10 and id_tiles[i] lt 100) then $
    printf,u,'        <href>tile0'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.kml</href>'
  if (id_tiles[i] ge 100) then $
    printf,u,'        <href>tile'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.kml</href>'
  
  printf,u,'        <viewRefreshMode>onRegion</viewRefreshMode>'
  printf,u,'      </Link>'
  printf,u,'    </NetworkLink>'

  printf,u,'        <GroundOverlay>'
  printf,u,'          <drawOrder>0</drawOrder>'
  printf,u,'          <Icon>'
  
  if (id_tiles[i] lt 10) then $
    printf,u,'            <href>tile00'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.jpg</href>'
  if (id_tiles[i] ge 10 and id_tiles[i] lt 100) then $
    printf,u,'            <href>tile0'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.jpg</href>'
  if (id_tiles[i] ge 100) then $
    printf,u,'            <href>tile'+strcompress(id_tiles[i],/remove_all)+'_'+satellite_id+'/'+upper_tile+'.jpg</href>'
  
  printf,u,'          </Icon>'
  printf,u,'          <LatLonBox>'
  printf,u,'            <north>'+strcompress(region_coords[i,2])+'</north>'
  printf,u,'            <south>'+strcompress(region_coords[i,0])+'</south>'
  printf,u,'            <east>'+strcompress(region_coords[i,3])+'</east>'
  printf,u,'            <west>'+strcompress(region_coords[i,1])+'</west>'
  printf,u,'          </LatLonBox>'
  printf,u,'        </GroundOverlay>'

endfor

;close the file up
printf,u,'      </Document>'
printf,u,'    </kml>'

close,u
free_lun,u

end
;-------------------------------------------------------------
