PRO MODIS_SATURATION_FIX, FILE, DATA_TYPE, NSCANS

; ============================================================================
; Purpose: Fix saturation in MODIS bands 1-7 (HKM or QKM L1B files)
; 
; Input parameters: 
;   file - The MODIS L1B input file. This will be either the HKM or QKM 
;          L1B file.
;   data_type - If the L1B data is a 250m resolution L1B, this variable 
;               must be equal to 'QKM'. Otherwise, it should be 'HKM.
;   nscans - The number of scans in the L1B file.
; 
; Author: Liam.Gumley@ssec.wisc.edu
; ============================================================================

COMPILE_OPT IDL2

;- Get scale factor for HKM or QKM images
case data_type of
  'QKM' : scale = 4
  'HKM' : scale = 2
endcase

;- Set number of columns and rows in this file
ncol = scale * 1354
nrow = scale * 10 * nscans

;- Set bands to check
if (data_type eq 'QKM') then begin
  band_nr_start = 1
  band_nr_stop = 2
endif else begin
  band_nr_start = 1
  band_nr_stop = 7
endelse

;- Open the file
hdfid = hdf_sd_start(file, /rdwr)

;- Loop over bands
for band = band_nr_start, band_nr_stop do begin
  ;- Read this band
  read_modis_radiance, hdfid, band, data_type, ncol, nrow, image, /raw

  ;- Get valid minimum and maximum scaled integer (SI)
  loc = where((image ge 0) and (image le 32767))
  ;- It could be that there is no minimum value. If there is no 
  ;- minimum value, skip the processing of this band
  if N_ELEMENTS (loc) EQ 1 then begin
      if loc EQ -1 then begin
          continue
      endif
  endif
  min_value = min(image[loc], max=max_value)

  ;- Fix over and under saturation
  if (data_type eq 'QKM') then begin

    ;- QKM data
    ;- Fix over saturation (SI = 65533)
    loc = where(image eq 65533, count)
    if (count gt 0) then image[loc] = max_value

    ;- Fix under saturation (SI = 65532)
    loc = where(image eq 65532, count)
    if (count gt 0) then image[loc] = min_value

  endif else begin

    ;- HKM data

    if (band le 2) then begin

      ;- Bands 1, 2
      ;- Fix over saturation (SI = 65528)
      loc = where(image eq 65528, count)
      if (count gt 0) then image[loc] = max_value

    endif else begin
    
      ;- Bands 3, 4, 5, 6, 7
      ;- Fix over saturation (SI = 65533)
      loc = where(image eq 65533, count)
      if (count gt 0) then image[loc] = max_value

      ;- Fix under saturation (SI = 65530 or 65532)
      loc = where((image eq 65530) or (image eq 65532), count)
      if (count gt 0) then image[loc] = min_value
    endelse
    
  endelse

  ;- Write this band
  write_modis_radiance, hdfid, band, data_type, ncol, nrow, image, /raw
endfor

;- Close the file
hdf_sd_end, hdfid

END
