PRO OVERLAY_SZANGLE_IMAGE, IMAGE_OUT, IMAGE_IN, IMAGE_IN_SZENANGLE, IMAGE_OUT_SZENANGLE, MISSING=MISSING

; ============================================================================
; Purpose: This script creates overlay two given images over each other, 
;     using each image's sensor zenith angle data. Each pixel has a sensor 
;     zenith angle value associated with it. When the script overlay two 
;     images over each other, the pixels with the lowest sensor zenith angle 
;     values are chosen. 
;     
;     The script performs the following steps to create an output composite 
;     file:
;     1. A mask is created which indicates the area of overlap between the 
;        the input images.
;     2. The minimum sensor zenith angle values are identified in the 
;        overlapping area.
;     3. A mask is created where the output image's best pixels are 
;        chosen.
;     4. A mask is created for the process where data from the array 
;        'image_in' will be copied to the output array 'image_out'.
;     5. The pixels in the input mask which have high sensorn zenith 
;        angle values, are set to zero.
;     6. All the best pixels of the input image are copied over to the 
;        output image by using the previously created mask.
;     7. A composite sensor zenith angle array is created.
;     
; Input parameters:
;     image_out - An image array onto which another image will be overlayed 
;         upon. 
;     image_in - An image array which will be overlayed onto the image array 
;         'image_out'.
;     image_in_szenangle - The sensor zenith angle array data which is 
;         associated with the image array image_in.
;     image_out_szenangle - The sensor zenith angle array data which is 
;         associated with the image array image_out.
;     missing - A three element array. Each element represent the missing 
;         values for the red, green and blue bands.
; 
; Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; Date: 07/22/2011
; ============================================================================

; Check arguments
if (n_elements(image_out) eq 0) then message, 'Argument IMAGE_ONE is undefined'
if (n_elements(image_in) eq 0) then message, 'Argument IMAGE_IN is undefined'
if (n_elements(image_in_szenangle) eq 0) then message, 'Argument IMAGE_IN_SZENANGLE is undefined'
if (n_elements(image_out_szenangle) eq 0) then message, 'Argument IMAGE_OUT_SZENANGLE is undefined'
if (n_elements(missing) eq 0) then missing = [255B, 255B, 255B]

; ----------------------------------------------------------------------------
; CREATE OVERLAP MASK ARRAY
; ----------------------------------------------------------------------------
; Create masks of the images. The masks are used to determine where there 
; are valid data.
in_img_mask = $
    (reform(image_in[0, *, *]) ne missing[0]) AND $
    (reform(image_in[1, *, *]) ne missing[1]) AND $
    (reform(image_in[2, *, *]) ne missing[2])

out_img_mask = $
    (reform(image_out[0, *, *]) ne missing[0]) AND $
    (reform(image_out[1, *, *]) ne missing[1]) AND $
    (reform(image_out[2, *, *]) ne missing[2])

; Create a mask where the two images overlap.
overlap_img_mask = in_img_mask AND out_img_mask

; ----------------------------------------------------------------------------
; WHERE THERE ARE MISSING VALUES IN IMAGE_OUT AND IMAGE_IN, FILL VALUES ARE 
; SET IN THE SENSOR ZENITH ANGLE ARRAYS.
; ----------------------------------------------------------------------------
; The sensor zenith angle data do not have the same no-data areas as 
; with the image. The following lines of code synchronise the no-data 
; areas of the image with the sensor zenith angle data.
tmp_idx = where (in_img_mask EQ 0)
tmp_idx_valid = 1
if N_ELEMENTS (tmp_idx) EQ 1 then begin
    if tmp_idx EQ -1 then begin
        tmp_idx_valid = 0
    endif
endif

if tmp_idx_valid EQ 1 then $
    image_in_szenangle [tmp_idx] = 18000

tmp_idx = where (out_img_mask EQ 0)
tmp_idx_valid = 1
if N_ELEMENTS (tmp_idx) EQ 1 then begin
    if tmp_idx EQ -1 then begin
        tmp_idx_valid = 0
    endif
endif

if tmp_idx_valid EQ 1 then $
    image_out_szenangle [tmp_idx] = 18000

; ----------------------------------------------------------------------------
; FIND MINIMUM SENSOR ZENITH ANGLE VALUES, AND CREATE MASK FOR OUTPUT ARRAY
; ----------------------------------------------------------------------------
; The pixels with the minimum sensor zenith angles are preserved in 
; the output image. Therefore look for pixels in the output image which 
; have the minimum sensor zenith angle values.

;min_szenangle = min ([[[image_in_szenangle]], [[image_out_szenangle]]], DIMENSION = 3)
;min_out_mask = (image_out_szenangle EQ min_szenangle) AND overlap_img_mask
;min_out_idx = where (min_out_mask NE 0)
;
;min_out_idx_valid = 1
;if N_ELEMENTS (min_out_idx) EQ 1 then begin
;    if min_out_idx EQ -1 then min_out_idx_valid = 0
;endif

; Any sensor zenith angle data that are out of bounds, clamp the data
tmp_idx = where ((image_in_szenangle GT 18000) OR (image_in_szenangle LT 0))
tmp_idx_valid = 1
if N_ELEMENTS (tmp_idx) EQ 1 then begin
    if tmp_idx EQ -1 then $
        tmp_idx_valid = 0
endif

if tmp_idx_valid EQ 1 then $
    image_in_szenangle [tmp_idx] = 18000

tmp_idx = where ((image_out_szenangle GT 18000) OR (image_out_szenangle LT 0))
tmp_idx_valid = 1
if N_ELEMENTS (tmp_idx) EQ 1 then begin
    if tmp_idx EQ -1 then $
        tmp_idx_valid = 0
endif

if tmp_idx_valid EQ 1 then $
    image_out_szenangle [tmp_idx] = 18000

mask_szenangle_in_min = (image_in_szenangle LT image_out_szenangle) AND overlap_img_mask

; ----------------------------------------------------------------------------
; CREATE MASK FOR INPUT DATA
; ----------------------------------------------------------------------------
; The mask is used to pixels from the input image to the output image. Where 
; the output image's sensor zenith angles values are lower than the input 
; image's sensor zenith angle value, erase it from the input mask.

;mask = in_img_mask 
;if min_out_idx_valid EQ 1 then $
;    mask [min_out_idx] = 0

mask = ((in_img_mask AND overlap_img_mask) AND mask_szenangle_in_min) OR (in_img_mask AND (1 - overlap_img_mask))

; ----------------------------------------------------------------------------
; OVERLAY INPUT IMAGE OVER OUTPUT IMAGE
; ----------------------------------------------------------------------------
; Overlay the good pixels from image_in onto image_out.
for dim = 0, 2 do begin
    image_out[dim, *, *] = $
        reform(image_in[dim, *, *]) * (mask) + $
        reform(image_out[dim, *, *]) * (1B - mask)
endfor 

; ----------------------------------------------------------------------------
; CREATE COMPOSITE SENSOR ZENITH ANGLE ARRAY
; ----------------------------------------------------------------------------
; Transfer the good sensor zenith angles values to the output sensor zenith
; angle array.
;help, image_out_szenangle
;print, max (image_out_szenangle)
;print, min (image_out_szenangle)
;print, min (image_in_szenangle)
image_out_szenangle = (image_in_szenangle * LONG (mask)) + $
    temporary (image_out_szenangle) * LONG (1B - mask)
;help, image_out_szenangle
;print, max (image_out_szenangle)
;print, min (image_out_szenangle)
END

