PRO WRITE_GLOBAL_METADATA, LAT_BIN_FILE, LON_BIN_FILE, BIN_FILE_WIDTH, BIN_FILE_LENGTH, BIN_FILE_TYPE, GEO_FILL_VALUE, SENSOR, SAT, DATE, TIME, RES, BAND_COMB_ARR, DIR, TEXCLUDE_FILE

; ============================================================================
; Purpose: Create metadata and event files for the given L1B 
;   geolocation file. Metadata and event files are created 
;   per band combination given by the array band_comb_arr.
; 
; Input parameters: 
;   geo_file - The L1B geolocation file.
;   sensor - The sensor for which the files will be created.
;   sat - The satellite name for which the files will be created.
;   date - The yeardate value of the L1B geolocation file.
;   time - The UTC time value of the L1B geolocation file.
;   res - The output tile resolution. 
;   band_comb_arr - An array of the band combinations. Each element in 
;       the array must be a four element array. The last 
;       four element array item must have only zero values. 
;       The first number in the four element array is the band 
;       band number for red, the second for green, the third 
;       for blue, and the last the band number which is going 
;       to be used for band sharpening. If no band sharpening 
;       is going to be used for the band combination, the 
;       value should be set to zero.
;   dir - The tile output directory.
;   texclude_file - The file which list the tile number for which tile 
;                   metadata and event files should not be created.
;   
; Output:
;   For all the possible tiles which are overlapped by the given L1B, a tile number 
;   directory is created. In each tile directory, for each band combination both a 
;   metadata and event file are created. The file numbers range from 0 to 647, and 
;   tile directories are from tile000 to tile647.
;   The file format for the metadata files are: 
;       <tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.dat
;   The file format for the event files are:
;       <tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.event
; 
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

; Check arguments
if (n_params() ne 14) then message, 'Usage: WRITE_GLOBAL_METADATA, LAT_BIN_FILE, BIN_FILE_WIDTH, BIN_FILE_LENGTH, BIN_FILE_TYPE, LAT_FILL_VALUE, SENSOR, SAT, DATE, TIME, RES, BAND_COMB_ARR, DIR, TEXCLUDE_FILE'

; ----------------------------------------------------------------------------
; Get the list of tile number which should be ignored
; ----------------------------------------------------------------------------
openr, lun, texclude_file, /GET_LUN
line_str = ''
texclude_arr = [0]
while not EOF (lun) do begin
    readf, lun, line_str
    check_comment = strpos (line_str, '#')
    if check_comment EQ 0 then continue
    texclude_arr = [texclude_arr, long (line_str)]
endwhile
free_lun, lun

; ----------------------------------------------------------------------------
; Get lat/lon data from geo file and also get the for the lat/long fill values
; ----------------------------------------------------------------------------
openr, lat_bin_lun, lat_bin_file, /GET_LUN
lat = make_array (bin_file_width, bin_file_length, TYPE=bin_file_type)
readu, lat_bin_lun, lat
close, lat_bin_lun

openr, lon_bin_lun, lon_bin_file, /GET_LUN
lon = make_array (bin_file_width, bin_file_length, TYPE=bin_file_type)
readu, lon_bin_lun, lon
close, lon_bin_lun
; ----------------------------------------------------------------------------
; Calculate the tile numbers from the latitude and longitude values
; ----------------------------------------------------------------------------
; Get lat/lon at middle of geo file
dims = size(lat, /dimensions)
lat_mid = lat[dims[0] / 2, dims[1] / 2]
lon_mid = lon[dims[0] / 2, dims[1] / 2]

; Get locations of good lat/lon values
loc = where (((lat ge -90.0)  and (lat ne FLOAT (geo_fill_value))) and $
             ((lon ge -180.0) and (lon ne FLOAT (geo_fill_value))))

; Get grid indices
latlon_to_gridindex, lat[loc], lon[loc], res, tile_index

; ----------------------------------------------------------------------------
; Calculate the tile lower left coordinates
; ----------------------------------------------------------------------------
; Get unique grid indices
tile_index = tile_index[unique(tile_index)]

; Convert indices to tile LL corner lat/lons
gridindex_to_latlon, tile_index, res, lat_cnr, lon_cnr

; ----------------------------------------------------------------------------
; Create tile number directories, and create both metadata files and event files
; ----------------------------------------------------------------------------
; Switch to output directory
cd, dir

; Loop over each tile and write metadata file
for tile = 0, n_elements(tile_index) - 1 do begin
  ; If the 'tile' number is in the variable texclude_arr, skip it
  tmp_bool = TOTAL (texclude_arr EQ tile_index [tile])
  if tmp_bool GT 0 then begin
      continue
  endif

  ; Set metadata directory name for this tile (e.g., tile412)
  meta_dir = string(tile_index[tile], format='("tile", i3.3)')

  ; If directory does not exist, create it
  if (file_test(meta_dir, /directory) eq 0) then spawn, 'mkdir -p ' + meta_dir

  ; For each band combination, creat the metadata and event files. The last element 
  ; in the band combination array is zero elements, therefore skip it.
  for idx = 0, (size (band_comb_arr, /dim))[1] - 2 do begin
      comb_arr = band_comb_arr [*, idx]
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
; Construct the metadata output file names. 
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
      ; Set metadata file name for this tile (e.g., tile412/tile412.terra.20080607.1415.01_04_03_01.dat)
      if N_ELEMENTS (comb_arr) EQ 3 then begin
        meta_file = meta_dir + '/' + string(tile_index[tile], sat, date, time, comb_arr [0], comb_arr [1], comb_arr [2], $
                    format='("tile", i3.3, ".", a, ".", i8.8, ".", i4.4, ".", i2.2, "_", i2.2, "_", i2.2, ".dat")')
      endif else begin
        meta_file = meta_dir + '/' + string(tile_index[tile], sat, date, time, comb_arr [0], comb_arr [1], comb_arr [2], $
                    comb_arr [3], comb_arr [4], $
                    format='("tile", i3.3, ".", a, ".", i8.8, ".", i4.4, ".", i2.2, "_", i2.2, "_", i2.2, "_", i2.2, "_", i2.2, ".dat")')
      endelse
      
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
; Open the metadata output file and calculate the tile corner coordinates
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
      ; Create metadata file
      openw, lun, meta_file, /get_lun
      
      ; Compute UL and LR corner lat/lons
      ul_lat = lat_cnr[tile] + res
      ul_lon = lon_cnr[tile]
      lr_lat = lat_cnr[tile]
      lr_lon = lon_cnr[tile] + res

; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
; Print the metadata information to the metadata file, and close the file
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
      ; Write metadata
      printf, lun, sat, format='("SAT  = ", a)'   
      printf, lun, sensor, format='("SENSOR  = ", a)'   
      printf, lun, date, format='("DATE = ", i8)'  
      printf, lun, tile_index[tile], format='("TILE = ", i4)'
      printf, lun, ul_lon, ul_lat, format='("OUL  = ", f9.3, ",", f9.3)'
      printf, lun, lr_lon, lr_lat, format='("OLR  = ", f9.3, ",", f9.3)'
      if N_ELEMENTS (comb_arr) EQ 3 then begin
          band_comb_str = string (comb_arr [0], comb_arr [1], comb_arr [2], $
              format = '(i2.2, "_", i2.2, "_", i2.2)')
      endif else begin
          band_comb_str = string (comb_arr [0], comb_arr [1], comb_arr [2], comb_arr [3], comb_arr [4], $
              format = '(i2.2, "_", i2.2, "_", i2.2, "_", i2.2, "_", i2.2)')
      endelse
      printf, lun, band_comb_str, format = '("BAND_COMB = ", a)'

      ; Write granule file name
      printf, lun, "@GRAN_NAME@", format='("GRAN_NAME = ", a)'
      
      ; Write granule time and middle lat/lon
      printf, lun, time, lon_mid, lat_mid, format='(i4.4, " UTC: ", f9.3, ",", f9.3)'
      
      ; Close metadata file
      free_lun, lun
      
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
; Construct the event file name and create the event file
; ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
      ; Create event file name for this tile (e.g., tile412/tile412.terra.20080607.1415.4_3_1.event)
      if N_ELEMENTS (comb_arr) EQ 3 then begin
          event_file = meta_dir + '/' + string(tile_index[tile], sat, date, time, comb_arr [0], comb_arr [1], comb_arr [2], $
              format='("tile", i3.3, ".", a, ".", i8.8, ".", i4.4, ".", i2.2, "_", i2.2, "_", i2.2, ".event")')
      endif else begin
          event_file = meta_dir + '/' + string(tile_index[tile], sat, date, time, comb_arr [0], comb_arr [1], comb_arr [2], $
              comb_arr [3], comb_arr [4], $
              format='("tile", i3.3, ".", a, ".", i8.8, ".", i4.4, ".", i2.2, "_", i2.2, "_", i2.2, "_", i2.2, "_", i2.2, ".event")')
      endelse
      
      ; Create event file
      openw, lun, event_file, /get_lun
      printf, lun, ""
      free_lun, lun
  endfor
endfor

END

