PRO WRITE_POLAR_METADATA, LAT_BIN_FILE, LON_BIN_FILE, BIN_FILE_WIDTH, BIN_FILE_LENGTH, BIN_FILE_TYPE, GEO_FILL_VALUE, SENSOR, SAT, DATE, TIME, RES, BAND_COMB_ARR, POLAR_REGION_BL, TILE_DIR

; ============================================================================
; Purpose: Create metadata and event files for polar image tiles. The files 
;   are created for all the band combinations given by band_comb_arr.
; 
; Input parameters: 
;   geo_file - The L1B geolocation file.
;   sensor - The sensor for which the files will be created.
;   sat - The satellite name for which the files will be created.
;   date - The yeardate value of the L1B geolocation file.
;   time - The UTC time value of the L1B geolocation file.
;   res - The output tile resolution. 
;   band_comb_arr - An array of the band combinations. Each element in 
;       the array must be a four element array. The last 
;       four element array item must have only zero values. 
;       The first number in the four element array is the band 
;       band number for red, the second for green, the third 
;       for blue, and the last the band number which is going 
;       to be used for band sharpening. If no band sharpening 
;       is going to be used for the band combination, the 
;       value should be set to zero.
;   polar_region_bl - This is a boolean value. If the value is equal to 1, then 
;       the Northen polar region's metadata and event files will be created. 
;       Otherwise the Southern polar region's metadata and event files will be 
;       created.
;   tile_dir - The directory filesystem path where the polar tile number directories 
;         will be created, and in which the metadata and event files will be 
;         created.
; 
; Output:
;   For all the possible polar tiles which are overlapped by the given L1B, a polar tile number 
;   directory is created. In each polar tile directory, for each band combination both a 
;   metadata and event file are created. The file numbers range from 0 to 63, and 
;   tile directories are from ptile00 to ptile63.
;   The file format for the metadata files are: 
;       <polar tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.dat
;   The file format for the event files are:
;       <polar tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.event
;
; Author: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
; ============================================================================

;- Check arguments
if (n_params() ne 14) then message, 'Usage: WRITE_POLAR_METADATA, LAT_BIN_FILE, LON_BIN_FILE, BIN_FILE_WIDTH, BIN_FILE_LENGTH, BIN_FILE_TYPE, GEO_FILL_VALUE, SENSOR, SAT, DATE, TIME, RES, BAND_COMB_ARR, POLAR_REGION_BL, TILE_DIR'

; ----------------------------------------------------------------------------
; Get lat/lon data from geo file and also get the for the lat/long fill values
; ----------------------------------------------------------------------------
openr, lat_bin_lun, lat_bin_file, /GET_LUN
lat_arr = make_array (bin_file_width, bin_file_length, TYPE=bin_file_type)
readu, lat_bin_lun, lat_arr
close, lat_bin_lun

openr, lon_bin_lun, lon_bin_file, /GET_LUN
lon_arr = make_array (bin_file_width, bin_file_length, TYPE=bin_file_type)
readu, lon_bin_lun, lon_arr
close, lon_bin_lun

; ----------------------------------------------------------------------------
; Filter the latitude and longtitude based on the ascending and descending 
; modes of the satellites. If Terra and FY3A are ascending during a daytime 
; pass, the data are filtered out. If Aqua and FY3B are descending during a 
; daytime pass, the data are filtered out.
; ----------------------------------------------------------------------------
old_lat_arr = lat_arr
acs_desc_filt_data, old_lat_arr, geo_fill_value, sat, lat_arr, geo_fill_value
acs_desc_filt_data, old_lat_arr, geo_fill_value, sat, lon_arr, geo_fill_value

; ----------------------------------------------------------------------------
; Filter the latitude and longitude data based on out of range data, and 
; fill value.
; ----------------------------------------------------------------------------
loc = where (((lat_arr ge -90.0)  and (lat_arr ne FLOAT (geo_fill_value))) and $
             ((lon_arr ge -180.0) and (lon_arr ne FLOAT (geo_fill_value))))

; TODO: These values are hard coded. Make it configurable.
tile_width = 1024.0 * 1000.0
nr_rows = 8
nr_cols = 8

; ----------------------------------------------------------------------------
; Write the metadata and event files out to the output tile directory
; ----------------------------------------------------------------------------
; Make sure that there are valid latitude and longtitude data
loc_valid_bl = 1
if N_ELEMENTS (loc) EQ 1 then begin
    if loc EQ -1 then loc_valid_bl = 0
endif

if loc_valid_bl EQ 1 then begin
    ; Generate the tileIDs for the North Pole
    if polar_region_bl GT 0 then begin
        ; Convert the latitude and longitude values into tile numbers
        latlon_to_polar_tiles, lon_arr [loc], lat_arr [loc], tile_width, nr_rows, nr_cols, 1, tilenr_arr
        if tilenr_arr [0] NE -1 then begin
            ; If there are valid tile numbers, convert the tile numbers into polar tile coordinates
            tilenr_arr = tilenr_arr [unique (tilenr_arr)]
            polar_tiles_to_latlon, tilenr_arr, tile_width, nr_rows, nr_cols, 1, tile_long_arr, tile_lat_arr
            
            ; Write the polar tile coordinates to a metadata file, and create an event file
            write_polar_metadata_file, sensor, sat, date, time, res, band_comb_arr, tile_dir, tile_width, $
                'North', tilenr_arr, tile_long_arr, tile_lat_arr
        endif
    endif
    
    ; Generate the tileIDs for the South Pole
    if polar_region_bl EQ 0 then begin
        ; Convert the latitude and longitude values into tile numbers
        latlon_to_polar_tiles, lon_arr [loc], lat_arr [loc], tile_width, nr_rows, nr_cols, 0, tilenr_arr
        if tilenr_arr [0] NE -1 then begin
            ; If there are valid tile numbers, convert the tile numbers into polar tile coordinates
            tilenr_arr = tilenr_arr [unique (tilenr_arr)]
            polar_tiles_to_latlon, tilenr_arr, tile_width, nr_rows, nr_cols, 0, tile_long_arr, tile_lat_arr
            
            ; Write the polar tile coordinates to a metadata file, and create an event file
            write_polar_metadata_file, sensor, sat, date, time, res, band_comb_arr, tile_dir, tile_width, $
                'South', tilenr_arr, tile_long_arr, tile_lat_arr
        endif
    endif
endif

END
