#!/bin/bash

# ============================================================================
# Purpose: This script is used to create either a global mosaic image, or a 
#     polar mosaic image. The script first determines the upper left (UL) and
#     lower right (LR) corner coordinates of an output image mosaic, given a
#     specific number of column and row tiles. Then the tile numbers and the
#     spatial formation of it are determined using the script
#     get_tileid_range.bash. A list of all the available image composites is
#     then generated based upon the given input arguments. The tile number
#     table is used to create the output mosaic image. For every tile number
#     row, for every tile number in every row, an empty composite tile image is
#     created if a corrosponding composite tile does not exist. If a composite
#     image does exist, it is appended to the right of a previous image of the
#     previous tile number in the row. If all the image composites of a row are
#     attached to each other, the resulting image is attached to the previous
#     row image. This continues until all the tile numbers have been iterated
#     through in the tile number table.  The script then creates a GeoTIFF
#     metadata file which is then used to create the final output GeoTIFF
#     image.
# 
# Input parameters:
#     1. The tile directory. This will normally be either MIPS_TILE_DIR, 
#        MIPS_POLR_TILE_DIR/north or MIPS_POLR_TILE_DIR/south.
#     2. The type of mosaic that will be created. This value must be either 
#        polar-north, polar-south or global.
#     3. The date in the format <year><month><day>.
#     4. The satellite name, this must be either aqua, terra, fy3a or fy3b.
#     5. The band combination of the output mosiac image. There must be 
#        composite images that are available in this given band combination. 
#     6. The output image resolution in meters.
#     7. The output file path.
#
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 7 ]; then
    echo "Usage: "
    echo "  "$(basename $0)" <Tile directory> <Mosaic type: polar-north | polar-south | global> <Date: yyyymmdd> <Sat name: aqua | terra | fy3a | fy3b | npp> <Band combination> <Resolution in meters> <Output file>"
    
    exit 1
fi

tile_base_dir=$1
mosaic_type=$2
data_date=$3
sat_name=$4
band_comb=$5
resol_m=$6
out_file=$7

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# ----------------------------------------------------------------------------
# GET THE LATITUDE AND LONGITUDE BOUNDS OF THE OUTPUT MOSAIC IMAGE
# ----------------------------------------------------------------------------
mosaic_bounds_file='mosaic_bounds_file.txt'
case $mosaic_type in 
    "polar-north" | "polar-south" )
        nr_rows=8
        nr_cols=8
        
        polar_value=$(echo $mosaic_type | cut -d"-" -f 2)
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo "[DEBUG] Executing the following:"
            echo "    "/bin/bash $MIPS_HOME/scripts/auxiliary/get_polar_bounds.bash $nr_rows $nr_cols $polar_value $mosaic_bounds_file
        fi
        /bin/bash $MIPS_HOME/scripts/auxiliary/get_polar_bounds.bash $nr_rows $nr_cols $polar_value $mosaic_bounds_file
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        event_prefix_str="polarmosaic"
    ;;

    "global" )
        nr_rows=18
        nr_cols=36
         
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo "[DEBUG] Executing the following:"
            echo "    "/bin/bash $MIPS_HOME/scripts/auxiliary/get_geo_bounds.bash $nr_rows $nr_cols $mosaic_bounds_file
        fi
        /bin/bash $MIPS_HOME/scripts/auxiliary/get_geo_bounds.bash $nr_rows $nr_cols $mosaic_bounds_file
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        event_prefix_str="globalmosaic"
    ;;

    * )
        echo "(ERROR)[E62] The wrong mosaic type was given."
        exit 1
    ;;
esac

ul_lon=$(grep 'UL_LON' $mosaic_bounds_file | cut -d"=" -f 2)
ul_lat=$(grep 'UL_LAT' $mosaic_bounds_file | cut -d"=" -f 2)
lr_lon=$(grep 'LR_LON' $mosaic_bounds_file | cut -d"=" -f 2)
lr_lat=$(grep 'LR_LAT' $mosaic_bounds_file | cut -d"=" -f 2)
map_ul_x=$(grep 'UL_MAP_X' $mosaic_bounds_file | cut -d"=" -f 2)
map_ul_y=$(grep 'UL_MAP_Y' $mosaic_bounds_file | cut -d"=" -f 2)
map_lr_x=$(grep 'LR_MAP_X' $mosaic_bounds_file | cut -d"=" -f 2)
map_lr_y=$(grep 'LR_MAP_Y' $mosaic_bounds_file | cut -d"=" -f 2)

# ----------------------------------------------------------------------------
# FOR POLAR MOSAIC IMAGES, SET THE MAXIMUM LATITUDE VALUE BASED ON THE CONFIG 
# FILE
# ----------------------------------------------------------------------------
update_map_ul_bl="False"
case $mosaic_type in 
    "polar-north" )
        bl_result=$(echo "$MIPS_NORTH_POLARM_LAT > $ul_lat" | bc)
        if [ $bl_result -eq 1 ]; then
            ul_lat=$MIPS_NORTH_POLARM_LAT
            lr_lat=$MIPS_NORTH_POLARM_LAT
            ctr_lat_val="90.0"

            update_map_ul_bl="True"
        fi
    ;;

    "polar-south" )
        bl_result=$(echo "$MIPS_SOUTH_POLARM_LAT < $ul_lat" | bc)
        if [ $bl_result -eq 1 ]; then
            ul_lat=$MIPS_SOUTH_POLARM_LAT
            lr_lat=$MIPS_SOUTH_POLARM_LAT
            ctr_lat_val="-90.0"
            
            update_map_ul_bl="True"
        fi
    ;;
esac

if [ "$update_map_ul_bl" == "True" ]; then
    tmp_out_file="new_map_ul_lr_output.txt"
    /bin/bash $MIPS_HOME/scripts/auxiliary/stereo_graphic_proj.bash $ctr_lat_val $ul_lat $ul_lon $tmp_out_file
    
    map_ul_x=$(grep 'MAP_X' $tmp_out_file | cut -d"=" -f 2 | awk '{print $1}')
    map_ul_y=$(grep 'MAP_Y' $tmp_out_file | cut -d"=" -f 2 | awk '{print $1}')
    
    /bin/bash $MIPS_HOME/scripts/auxiliary/stereo_graphic_proj.bash $ctr_lat_val $lr_lat $lr_lon $tmp_out_file
    
    map_lr_x=$(grep 'MAP_X' $tmp_out_file | cut -d"=" -f 2 | awk '{print $1}')
    map_lr_y=$(grep 'MAP_Y' $tmp_out_file | cut -d"=" -f 2 | awk '{print $1}')
    
    rm -f $tmp_out_file
fi

# ----------------------------------------------------------------------------
# CALCULATE THE WIDTH AND THE LENGTH OF THE OUTPUT IMAGE
# ----------------------------------------------------------------------------
out_img_width=$(echo "$map_lr_x - $map_ul_x" | bc)
out_img_length=$(echo "$map_ul_y - $map_lr_y" | bc)
scale_val=$(echo "scale=4;($resol_m / $MIPS_PC_PIXEL_RESOL)" | bc)
scale_val_per=$(echo "scale=4;(100.0 / $scale_val)" | bc)

scl_out_img_width=$(echo "$out_img_width / $resol_m" | bc)
scl_out_img_length=$(echo "$out_img_length / $resol_m" | bc)

# ----------------------------------------------------------------------------
# CREATE THE BASE IMAGE, IF NECESSARY
# ----------------------------------------------------------------------------
# If previous version of the output image exist, copy it to the current directory. 
# Otherwise create a blank base image.
working_out_file="working."$(basename $out_file)
if [ -e $out_file ]; then 
    cp $out_file $working_out_file
else
    echo  convert -size $scl_out_img_width"x"$scl_out_img_length xc:black $working_out_file
    convert -depth 8 -colorspace RGB -size $scl_out_img_width"x"$scl_out_img_length xc:black $working_out_file
fi

# ----------------------------------------------------------------------------
# GET ALL THE AVAILABLE IMAGE COMPOSITES MOSAIC EVENT FILES
# ----------------------------------------------------------------------------
# Get a list of all the tile event files
tile_img_file_lst="tile_img_file_lst.txt"
# TODO: Look for global mosaic event files
find $tile_base_dir -name $event_prefix_str'.'$tile_prefix_str'*.'$sat_name'.'$data_date'.'$band_comb'.*.event' > $tile_img_file_lst

# Check if the event files were found. Otherwise exit.
if [ ! -s $tile_img_file_lst ]; then
    exit 0
fi

# Create a list of all the unique tiles in the given tile image event file list
tile_nr_lst="tile_nr_lst.txt"
unique_tile_nr_lst="unique_tile_nr_lst.txt"
filt_img_file_lst="filt_img_file_lst.txt"
# Remove the above files in case the files are present
rm -f $tile_nr_lst $unique_tile_nr_lst $filt_img_file_lst

OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for line_str in $(cat $tile_img_file_lst); do
    part_str=$(basename $line_str | cut -d "." -f 2)
    echo $part_str >> $tile_nr_lst
done
IFS=$OLD_IFS
uniq $tile_nr_lst > $unique_tile_nr_lst

# ----------------------------------------------------------------------------
# MAKE LIST OF ALL THE MOST RECENT EVENT FILES
# ----------------------------------------------------------------------------
tmp_unique_file_lst="tmp_unique_tile_nr_lst.txt"
tmp_filt_img_file_lst="tmp_filt_img_file_lst.txt"
for tile_nr_str in $(cat $unique_tile_nr_lst); do
    rm -f $tmp_unique_file_lst $tmp_filt_img_file_lst
    grep $tile_nr_str $tile_img_file_lst > $tmp_unique_file_lst
    /bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $tmp_unique_file_lst $tmp_filt_img_file_lst 6
    tail -n 1 $tmp_filt_img_file_lst >> $filt_img_file_lst
done

# ----------------------------------------------------------------------------
# PROCESS THE EVENT FILES
# ----------------------------------------------------------------------------
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for tile_event_file in $(cat $filt_img_file_lst); do
    # Get the image and proj file names
    comp_dirname_str=$(dirname $tile_event_file)
    comp_bs_filename_str=$(basename $tile_event_file | cut -d "." -f 2-6)
    comp_img_file_str=$comp_dirname_str"/"$comp_bs_filename_str".png"
    comp_proj_file_str=$comp_dirname_str"/"$comp_bs_filename_str".proj"
    
    # Get the tile projected corner coordinates
    tile_ul_map_x=$(grep UL_XY $comp_proj_file_str | cut -d"=" -f 2 | cut -d"," -f 1)
    tile_ul_map_y=$(grep UL_XY $comp_proj_file_str | cut -d"=" -f 2 | cut -d"," -f 2)
    tile_lr_map_x=$(grep LR_XY $comp_proj_file_str | cut -d"=" -f 2 | cut -d"," -f 1)
    tile_lr_map_y=$(grep LR_XY $comp_proj_file_str | cut -d"=" -f 2 | cut -d"," -f 2)

    # Offset the coordinates with the LL map coordinates
    tile_ul_map_x=$(echo "$tile_ul_map_x - $map_ul_x" | bc)
    tile_ul_map_y=$(echo "$tile_ul_map_y - $map_lr_y" | bc)
    tile_lr_map_x=$(echo "$tile_lr_map_x - $map_ul_x" | bc)
    tile_lr_map_y=$(echo "$tile_lr_map_y - $map_lr_y" | bc)

    # Scale the coordinates
    scl_tile_ul_map_x=$(echo "$tile_ul_map_x / $resol_m" | bc)
    scl_tile_ul_map_y=$(echo "$tile_ul_map_y / $resol_m" | bc)
    scl_tile_lr_map_x=$(echo "$tile_lr_map_x / $resol_m" | bc)
    scl_tile_lr_map_y=$(echo "$tile_lr_map_y / $resol_m" | bc)

    echo "Event file: "$tile_event_file
    echo scl_tile_ul_map_x: $scl_tile_ul_map_x
    echo scl_tile_ul_map_y: $scl_tile_ul_map_y
    echo scl_tile_lr_map_x: $scl_tile_lr_map_x
    echo scl_tile_lr_map_y: $scl_tile_lr_map_y
    
    # Make a scaled version of the composite image, and sharpen it
    convert -resize $scale_val_per"%" $comp_img_file_str local_img.png
    
    convert local_img.png -geometry +$scl_tile_ul_map_x+$scl_tile_lr_map_y -composite $working_out_file
    
    exit 1
done

exit 1

# TODO: NEW CODE FROM HENCE FORTH
# BEGIN
# END

row_img_file="row_image.tif"
col_img_file="col_image.tif"
target_image_file="temp_image.tif"
target_image_scaled_file="temp_image_scaled.tif"

# ----------------------------------------------------------------------------
# FOR EVERY ROW OF TILE NUMBERS
# ----------------------------------------------------------------------------
rm -f $row_img_file
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for line_str in $(cat $tileid_range_file); do
    rm -f $col_img_file
    
    line_str=$(echo $line_str | tr -s ' ' ',')
    # Count number of ','s
    nr_tiles=$(echo $line_str | grep -o ',' | wc -l)
    # ----------------------------------------------------------------------------
    # FOR TILE NUMBER IN A ROW OF TILE NUMBERS
    # ---------------------------------------------------------------------------- 
    for idx in $(seq 1 $nr_tiles); do
        rm -f $target_image_file $target_image_scaled_file
        
        tile_val=$(echo $line_str | cut -d"," -f $idx)
        echo "    (Processing "$tile_val")"
        tile_img_lst=$(grep $tile_val $tile_img_file_lst)
        # ----------------------------------------------------------------------------
        # CREATE AN EMPTY TILE IF A COMPOSITE IMAGE DOES NOT EXIST
        # ---------------------------------------------------------------------------- 
        if [ $? -ne 0 ]; then
            # File does not exist. Make an empty file
            tile_nr=$(echo $tile_val | cut -c6-)
            
            if [ "$MIPS_DEBUG" == "YES" ]; then
                echo "    (Creating empty polar tiff file)"
            fi
            
            if [ "$tile_prefix_str" == "tile" ]; then
                /bin/bash $MIPS_HOME/scripts/auxiliary/create_normal_tile_tiff.bash $tile_nr $target_image_file
            else
                /bin/bash $MIPS_HOME/scripts/auxiliary/create_polartile_tiff.bash $tile_nr $target_image_file
            fi
            # If an error occured, exit immediately.
            if [ $? -ne 0 ]; then
                exit 1
            fi
        # ----------------------------------------------------------------------------
        # GET A MOST RECENT COMPOSITE IMAGE OF THE GIVEN TILE NUMBER
        # ---------------------------------------------------------------------------- 
        else
            tile_img_tmp_filelst=$(mktemp)
            
            grep $tile_val $tile_img_file_lst > $tile_img_tmp_filelst
            /bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $tile_img_tmp_filelst $tile_img_tmp_filelst 5
            if [ $? -ne 0 ]; then
                echo "(ERROR)[E77] The script 'sort_comp_img_filelst.bash' failed."
                exit 1
            fi

            tile_img=$(tail -n 1 $tile_img_tmp_filelst)
            rm -f $tile_img_tmp_filelst
            
            if [ "$MIPS_DEBUG" == "YES" ]; then
                echo "    (Using file "$(basename $tile_img)")"
            fi 
            
            eval convert $MIPS_IMG_SHARPEN_CNF $tile_img $target_image_file
        fi
        
        # ----------------------------------------------------------------------------
        # APPEND THE COMPOSITE IMAGE TO THE RIGHT OF A PREVIOUS COMPOSITE IMAGE
        # ---------------------------------------------------------------------------- 
        if [ -e $col_img_file ]; then
            # If the column of images already exist, append to it.
            convert -compress None $target_image_file -scale $scale_val% -depth 8 -colorspace RGB $target_image_scaled_file
            if [ $? -ne 0 ]; then
                echo "(ERROR)[E78] The ImageMagick command 'convert' failed to scale image "$target_image_scaled_file
                exit 1
            fi
            
            eval convert -compress None $col_img_file $target_image_scaled_file \
                    +append -depth 8 -colorspace RGB \
                    $col_img_file 
            if [ $? -ne 0 ]; then
                echo "(ERROR)[E79] The ImageMagick command 'convert' failed to append image "$col_img_file" to "$target_image_scaled_file
                exit 1
            fi
        else
            # If the column of images do not exist, create a column image.
            eval convert -compress None $target_image_file \
                    -scale $scale_val% -depth 8 -colorspace RGB \
                    $target_image_scaled_file
            if [ $? -ne 0 ]; then
                echo "(ERROR)[E80] The ImageMagick command 'convert' failed to scale image "$target_image_scaled_file
                exit 1
            fi
            
            cp $target_image_scaled_file $col_img_file
        fi
    done
    
    # ----------------------------------------------------------------------------
    # APPEND THE COLUMN OF IMAGES TO THE BOTTOM OF THE PREVIOUS ROW
    # ----------------------------------------------------------------------------
    if [ -e $row_img_file ]; then
        convert -compress None -depth 8 -colorspace RGB $row_img_file \
                -depth 8 -colorspace RGB $col_img_file \
                -append -depth 8 -colorspace RGB \
                $row_img_file
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E81] The ImageMagick command 'convert' failed to append image "$row_img_file" to "$col_img_file
            exit 1
        fi
    else
        cp $col_img_file $row_img_file
    fi
done
IFS=$OLD_IFS

#-------------------------------------------------------------------------------
# CREATE GEOTIFF IMAGE
#-------------------------------------------------------------------------------
# Create a geotiff version of the tiff file. First create the geotiff template file
geotiff_templ_file="geotiff_template.txt"
if [ "$tile_prefix_str" == "tile" ]; then
    source_geotiff_templ_file=$MIPS_HOME/templates/geotiff_metadata.txt
else
    source_geotiff_templ_file=$MIPS_HOME/templates/geotiff_metadata_polar.txt
fi

cat $source_geotiff_templ_file > $geotiff_templ_file
if [ ! -e "$geotiff_templ_file" ]; then
    echo "(ERROR)[E82] The "$source_geotiff_templ_file" could not be dublicated."
    exit 1
fi

# Calculate the pixel resolution of the data
pixel_resol=$(echo "250.0 * (100 / $scale_val)" | bc)

# Set the value in the geotiff template file
sed -i "s/@UL_X@/$map_ul_x/g" $geotiff_templ_file
sed -i "s/@UL_Y@/$map_ul_y/g" $geotiff_templ_file
sed -i "s/@RESOL@/$pixel_resol/g" $geotiff_templ_file
if [ "$tile_prefix_str" == "ptile" ]; then
    sed -i "s/@POLAR_LAT_VAL@/$ctr_lat_val/g" $geotiff_templ_file
fi

# Create a copy of the geotiff image
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    "$MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_templ_file $row_img_file $out_file
fi
$MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_templ_file $row_img_file $out_file
if [ $? -ne 0 ]; then
    echo "(ERROR)[E83] The binary geotifcp failed to create the GeoTIFF file "$(basename $out_file)
    exit 1
fi

#-------------------------------------------------------------------------------
# REMOVE TEMPORARY FILES
#-------------------------------------------------------------------------------
rm -f $tile_img_file_lst $mosaic_bounds_file $tileid_range_file $row_img_file \
      $row_img_file $col_img_file $target_image_file $target_image_scaled_file \
      idlsave.pro idl_output.txt runidl $geotiff_templ_file flsave.pro

