#!/bin/bash
#
# ============================================================================
# MIPS subset image processor BASH script
# 
# This script creates subsets images based on the subset configuration files. 
# The script search for all available composite image tiles, and create 
# a subset image for each configured subset images. The steps that are followed 
# to create the subset images, are as follows:
# 1. For each subset configuration, configuration parameters are loaded
# 2. All the composite image tiles which overlap the geographical region of 
#    the loaded subset configuration, is located
# 3. The subset image is created
# 4. The metadata of the subset image is created
# 5. A geotiff version of the subset image is created
# 6. A JPEG version of the subset image is created
# 
# If more than 40% of the subset image is black pixels (missing data), the 
# the subset image will be in an intermediate directory. When less then 40% 
# of the image is black pixels, the subset image will be copied in the final 
# subset output directory.
# 
# Note that the script does not take any input parameters.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Check for MIPS_CONF environment variable
if [ -z "$MIPS_CONF" ]; then
    echo "Environment variable MIPS_CONF is not set. Source the dbge_env.bash file."
    exit 1
fi

# See if the subsets.conf file exists
if [ ! -e $MIPS_SUBSET_CONF ]; then
    echo "The configuration file \"$MIPS_SUBSET_CONF\" is not present."
    echo "Processing cannot continue without this file."
    exit 1
fi

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

if [ $# -ne 4 ]; then
    echo "Usage: "
    echo "    "$(basename $0)" <date> <satellite: fy3a | fy3b | terra | aqua> <band combination> <:complete:incomplete:>"
    
    exit 1
fi

yeardate=$1
satname=$2
band_comb=$3
process_arg_str=$4

# Determine if all the tiles must be present in order to create a subset, or can it be 
# incomplete.
proccess_flag_str=$(echo $process_arg_str | cut -d":" -f 2)

# First remove composite event files which are older than two days. This is to prevent 
# possible stale composite event files which could disrupt the processing
if [ "$MIPS_DEBUG" != "YES" ]; then
    find $MIPS_TILE_DIR -name "subset.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
fi

# Get a list of all the available composite tiff files.
comp_file_lst=$(mktemp)
find $MIPS_TILE_DIR -type f -name "subset.tile*.$satname.$yeardate.$band_comb.*.event" -exec ls {} \; > $comp_file_lst
#-------------------------------------------------------------------------------
# LOOP THROUGH ALL THE DEFINED SUBSETS
#-------------------------------------------------------------------------------
if [ ! -e $MIPS_SUBSET_CONF ]; then
    echo "(ERROR)[E45] The subset configuration file "$MIPS_SUBSET_CONF" is missing"
    exit 1
fi

if [ ! -e $MIPS_RO_SUBSET_CONF ]; then
    echo "(ERROR)[E87] The subset configuration file "$MIPS_SUBSET_CONF" is missing"
    exit 1
fi

subset_comb_file="subset_combine_file.txt"
cat $MIPS_RO_SUBSET_CONF > $subset_comb_file
cat $MIPS_SUBSET_CONF   >> $subset_comb_file

# The configuration files has spaces in the lines, thus manipulate the BASH separator
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for conf_line_str in $(cat $subset_comb_file | grep -v "#"); do
    # Get the subset configuration parameters
    subset_name_str=$(echo $conf_line_str | cut -d";" -f 1)
    coord_bounds_str=$(echo $conf_line_str | cut -d";" -f 2)
    output_dir=$MIPS_SUBSET_DIR
    
    echo
    echo "(Creating subset $subset_name_str)"
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "subset_name_str: $subset_name_str"
        echo "coord_bounds_str: $coord_bounds_str"
        echo "output_dir: $output_dir"
    fi
    
    intermediate_dir=$output_dir"/intermediate"
    # Make sure that the intermediate subset directory exists
    if [ ! -e $intermediate_dir ]; then
        mkdir -p $intermediate_dir
    fi
    
    # Make the subset image file name, and create the file paths to the 
    # final output and the intermediate output.
    subset_output_name=$subset_name_str.$satname.$yeardate.$band_comb.tif
    subset_final_output_path=$output_dir/$subset_name_str.$satname.$yeardate.$band_comb.tif
    subset_inter_output_path=$intermediate_dir/$subset_name_str.$satname.$yeardate.$band_comb.tif
    subset_working_output_path="working."$subset_name_str.$satname.$yeardate.$band_comb.tif
    # Make the subset metadata file name for the subset image, and create the 
    # file paths to the final output and the intermediate output.
    subset_mdata_name=$subset_name_str.$satname.$yeardate.$band_comb.dat
    subset_final_mdata_path=$output_dir/$subset_mdata_name
    subset_inter_mdata_path=$intermediate_dir/$subset_mdata_name
    # Make the subset projection file name for the subset image
    subset_proj_name="working."$subset_name_str.$satname.$yeardate.$band_comb.proj
    subset_inter_proj_path=$intermediate_dir/$subset_proj_name
    
#-------------------------------------------------------------------------------
# GET THE TILE NUMBERS WHICH OVERLAPS THE CONFIGURED SUBSET GEOGRAPHICAL AREA
#-------------------------------------------------------------------------------
    ss_ul_lon=$(echo $coord_bounds_str | awk '{print $1}')
    ss_ul_lat=$(echo $coord_bounds_str | awk '{print $2}')
    ss_lr_lon=$(echo $coord_bounds_str | awk '{print $3}')
    ss_lr_lat=$(echo $coord_bounds_str | awk '{print $4}')
    
    tile_idr_out=tile_id_range_output.txt
    tile_tbl_out=tile_id_table_output.txt
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_tileid_range.bash 'Normal' \
        $ss_ul_lon $ss_ul_lat $ss_lr_lon $ss_lr_lat $tile_idr_out
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_tileid_range.bash 'Normal-table' \
        $ss_ul_lon $ss_ul_lat $ss_lr_lon $ss_lr_lat $tile_tbl_out
    
    # If an error occured, exit immediately.
    if [ $? -eq 1 ]; then
        exit 1
    fi
    
    # If the output of the script get_tileid_range.bash is empty, there are then 
    # no tile numbers which cover the subset geographical area.
    tilename_lst=$(sort $tile_idr_out)
    if [ -z "$tilename_lst" ]; then
        echo "(WARNING)[W2] The subset configuration of '$subset_name_str' does not cover any tiles"
        continue
    fi

#-------------------------------------------------------------------------------
# CHECK IF THERE ARE COMPOSITE TILES AVAILABLE FOR THIS SUBSET
#-------------------------------------------------------------------------------   
    if [ $MIPS_DEBUG == "YES" ]; then
        echo "[DEBUG]: Making a list of the available composite images."
    fi
    
    filt_tile_img_lst=$(mktemp) # This file will list all the event files that will be used to make the composite image
    count_nr_tiles=0
    for tilename_str in $(cat $tile_idr_out); do
        tmp_filt_tile_img_lst=$(mktemp)
        grep $tilename_str $comp_file_lst > $tmp_filt_tile_img_lst
        
        sort_tile_img_lst=$(mktemp)
        /bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $tmp_filt_tile_img_lst $sort_tile_img_lst 6
        
        # The last file has the most satellite passes, therefore it is the most recent composite image
        tail -n 1 $sort_tile_img_lst >> $filt_tile_img_lst
        count_nr_tiles=$(echo "$count_nr_tiles + 1" | bc)
        
        rm -f $tmp_filt_tile_img_lst $sort_tile_img_lst
    done
    
    nr_lines_filt_img=$(wc -l $filt_tile_img_lst | awk '{print $1}')
    
    if [ "$proccess_flag_str" == "complete" ]; then
        if [ $nr_lines_filt_img -ne $count_nr_tiles ]; then
            if [ "$MIPS_DEBUG" == "YES" ]; then
                echo "        [DEBUG]: For subset "$subset_name_str" not all the tile composite images are available."
            fi
            
            rm -f $filt_tile_img_lst $comp_file_lst
            continue
        fi
    fi
    
#-------------------------------------------------------------------------------
# GET THE PROJECTED CORNER COORDINATES OF THE MOSAIC IMAGE
#-------------------------------------------------------------------------------   
    # Get the first tile number, and the last tile number
    first_tilenr_str=$(head -n 1 $tile_idr_out | cut -c5-)
    last_tilenr_str=$(tail -n 1 $tile_idr_out | cut -c5-)
    tmp_cnr_coord_file=$(mktemp)
    
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_normal_tile_ul_lr_coord.bash $first_tilenr_str 0 $tmp_cnr_coord_file
    mosaic_img_ul_map_x=$(grep "UL_MAP_X" $tmp_cnr_coord_file | cut -d"=" -f 2)
    mosaic_img_ul_map_y=$(grep "UL_MAP_Y" $tmp_cnr_coord_file | cut -d"=" -f 2)
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "Mosaic UL and LR:"
        echo "    UL lon:"$(grep UL_LON $tmp_cnr_coord_file | cut -d"=" -f 2)
        echo "    UL lat:"$(grep UL_LAT $tmp_cnr_coord_file | cut -d"=" -f 2)
    fi
    
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_normal_tile_ul_lr_coord.bash $last_tilenr_str 0 $tmp_cnr_coord_file
    mosaic_img_lr_map_x=$(grep "LR_MAP_X" $tmp_cnr_coord_file | cut -d"=" -f 2)
    mosaic_img_lr_map_y=$(grep "LR_MAP_Y" $tmp_cnr_coord_file | cut -d"=" -f 2)
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "    LR lon:"$(grep LR_LON $tmp_cnr_coord_file | cut -d"=" -f 2)
        echo "    LR lat:"$(grep LR_LAT $tmp_cnr_coord_file | cut -d"=" -f 2)
    fi
    
    rm -f $tmp_cnr_coord_file
    
#-------------------------------------------------------------------------------
# LOOP THROUGH THE TABLE OF TILES AND MOSAIC ALL THE TILES
#-------------------------------------------------------------------------------   
    row_img_file="row_image.tif"
    col_img_file="col_image.tif"
    target_image_file="temp_image.tif"
    
    if [ $MIPS_DEBUG == "YES" ]; then
        echo "[DEBUG]: Mosaic processing about to start. The following will be processed:"
        echo "==========================================================================="
        cat $tile_tbl_out
        echo "==========================================================================="
        echo
    fi

    rm -f $row_img_file
    for tile_line_str in $(cat $tile_tbl_out); do
        rm -f $col_img_file
        
        tile_line_str=$(echo $tile_line_str | tr -s ' ' ',')
        # Count number of ','s
        nr_tiles=$(echo $tile_line_str | grep -o ',' | wc -l)
        for idx in $(seq 1 $nr_tiles); do
            rm -f $target_image_file $target_image_scaled_file
            
            tile_val_str=$(echo $tile_line_str | cut -d"," -f $idx)
            if [ $MIPS_DEBUG == "YES" ]; then
                echo -n $tile_val_str
            fi
            
            # Get the tile event file which has the value of tile_val_str
            target_tile_event_file=$(grep $tile_val_str $filt_tile_img_lst)

            if [ $? -ne 0 ]; then
                tile_nr_str=$(echo $tile_val_str | cut -c5-)
                /bin/bash $MIPS_HOME/scripts/auxiliary/create_normal_tile_tiff.bash $tile_nr_str $target_image_file > /dev/null
                
                if [ $MIPS_DEBUG == "YES" ]; then
                    echo -n '_'
                fi
            else
                # Derive the image file path
                tile_dir_name_str=$(dirname $target_tile_event_file)
                tile_bsn_name_str=$(basename $target_tile_event_file)
                comp_bsn_part_str=$(echo $tile_bsn_name_str | cut -d"." -f 2-6)
                comp_img_file_path=$tile_dir_name_str/$comp_bsn_part_str".png"
                comp_dat_file_path=$tile_dir_name_str/$comp_bsn_part_str".dat"

                # Make double sure the file exists
                if [ ! -e $comp_img_file_path ]; then
                    echo "(ERROR)[Exx] The composite image file "$comp_img_file_path" does not exist"
                    exit 1
                fi

                # Create a sharpened version of the composite image
                eval convert $MIPS_IMG_SHARPEN_CNF $comp_img_file_path $target_image_file
                
                if [ $MIPS_DEBUG == "YES" ]; then
                    echo -n '.'
                fi
            fi
            
            # ----------------------------------------------------------------------------
            # APPEND THE COMPOSITE IMAGE TO THE RIGHT OF A PREVIOUS COMPOSITE IMAGE
            # ---------------------------------------------------------------------------- 
            if [ -e $col_img_file ]; then
                
                eval convert -compress None $col_img_file $target_image_file \
                        +append -depth 8 -colorspace RGB $col_img_file 
                if [ $? -ne 0 ]; then
                    echo "(ERROR)[Exx] The ImageMagick command 'convert' failed to append image "$col_img_file" to "$target_image_file
                    exit 1
                fi
                
                if [ $MIPS_DEBUG == "YES" ]; then
                    echo -n '.'
                fi
            else
                # If the column of images do not exist, create a column image.
                eval convert -compress None $target_image_file \
                        -depth 8 -colorspace RGB $target_image_file
                if [ $? -ne 0 ]; then
                    echo "(ERROR)[Exx] The ImageMagick command 'convert' failed to scale image "$target_image_file
                    exit 1
                fi
                
                cp $target_image_file $col_img_file
                
                if [ $MIPS_DEBUG == "YES" ]; then
                    echo -n '_'
                fi
            fi
        done
        
        if [ $MIPS_DEBUG == "YES" ]; then
            echo ""
        fi
        
        # ----------------------------------------------------------------------------
        # APPEND THE COLUMN OF IMAGES TO THE BOTTOM OF THE PREVIOUS ROW
        # ----------------------------------------------------------------------------
        if [ -e $row_img_file ]; then
            convert -compress None -depth 8 -colorspace RGB $row_img_file \
                    -depth 8 -colorspace RGB $col_img_file \
                    -append -depth 8 -colorspace RGB $row_img_file
            if [ $? -ne 0 ]; then
                echo "(ERROR)[Exx] The ImageMagick command 'convert' failed to append image "$row_img_file" to "$col_img_file
                exit 1
            fi
        else
            cp $col_img_file $row_img_file
        fi
        
        cp $row_img_file $subset_working_output_path
    done
    
    rm -f $filt_tile_img_lst
    
#-------------------------------------------------------------------------------
# CREATE SUBSET IMAGE
#-------------------------------------------------------------------------------
    # If the image is all black, skip the rest of the processing
    return_str=$(identify -format '%[colorspace]' $subset_working_output_path)
    if [ $return_str == "Gray" ]; then
        continue
    fi
    
    # Create a projection file
    echo "UL_XY="$mosaic_img_ul_map_x,$mosaic_img_ul_map_y  > $subset_proj_name
    echo "LR_XY="$mosaic_img_lr_map_x,$mosaic_img_lr_map_y >> $subset_proj_name
    /bin/bash $MIPS_HOME/scripts/auxiliary/make_cut_subset.bash $subset_working_output_path \
                                                                $ss_ul_lon $ss_ul_lat \
                                                                $ss_lr_lon $ss_lr_lat \
                                                                $subset_output_name
    
    # If the idl_output.txt file do not exist, the make_cut_subset.bash 
    # script failed.
    if [ ! -e idl_output.txt ]; then
        echo "(ERROR)[Exx] The BASH script failed to produce a text file 'idl_output.txt'."
        exit 1
    fi
    
    img_ss_ul_x=$(grep UL_X idl_output.txt | awk '{print $2}')
    img_ss_ul_y=$(grep UL_Y idl_output.txt | awk '{print $2}')
    img_ss_lr_x=$(grep LR_X idl_output.txt | awk '{print $2}')
    img_ss_lr_y=$(grep LR_Y idl_output.txt | awk '{print $2}') 

    if [ -e $subset_inter_output_path ]; then
        if [ $MIPS_DEBUG == "YES" ]; then
            echo "[DEBUG]: An intermediate version of the subset image "$subset_output_name" exist."
        fi
        
        cp $subset_inter_output_path $subset_working_output_path
        
        echo "overlay_subset_image, '$subset_output_name', '$subset_working_output_path', 'tif'" > runidl
        echo "exit" >> runidl
        
        if [ "$MIPS_DEBUG" == "YES" ]; then
            cat runidl
            $IDL_CMD runidl &> idl_output.txt
            idl_rtn=$?
            cat idl_output.txt
        else
            $IDL_CMD runidl &> idl_output.txt
            idl_rtn=$?
        fi
        
        grep -i "Error" idl_output.txt &> /dev/null
        if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
            echo "(ERROR)[Exx] IDL script overlay_subset_image.pro failed"
            echo "========= IDL script ========="
            cat runidl
            echo "========= IDL output ========="
            cat idl_output.txt
            
            exit 1
        fi
        
        rm -f $subset_output_name
        mv $subset_working_output_path $subset_output_name
    fi
    rm -f $subset_proj_name
    
#-------------------------------------------------------------------------------
# CREATE GEOTIFF IMAGE
#-------------------------------------------------------------------------------
    cp $subset_output_name $subset_inter_output_path
    # If more than 40% of the image is full of black pixels, do not 
    # copy it to the final output directory. Otherwise, the subset images will 
    # reside in the intermediate directory.
    pblack_pixels=$(grep "Percentage black pixels:" idl_output.txt | awk '{print $4}')
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG]: Percentage of black pixels in subset image: $pblack_pixels"
    fi
    
    if [ "$pblack_pixels" -gt 40 ]; then
        continue
    fi
    
    enh_subset_output_name="enhnc_"$subset_output_name
    eval convert -compress None $MIPS_IMG_SHARPEN_STR $subset_output_name $enh_subset_output_name
    
    geotiff_subset_output_name="geotiff_"$subset_output_name
    # Get a template of the geotiff metadata file from the MIPS installation.
    cat $MIPS_HOME/templates/geotiff_metadata.txt > geotiff_metadata.txt 
    # Add the upper left map coordinates to the duplicate geotiff metadata file.
    sed -i "s/@UL_X@/$img_ss_ul_x/g" geotiff_metadata.txt
    sed -i "s/@UL_Y@/$img_ss_ul_y/g" geotiff_metadata.txt
    sed -i "s/@RESOL@/$MIPS_PC_PIXEL_RESOL/g" geotiff_metadata.txt
    # Copy the new geotiff metadata into the existing TIFF image, thus a 
    # GeoTIFF image is created.
    $MIPS_HOME/bin/geotifcp -c lzw -g geotiff_metadata.txt $enh_subset_output_name $subset_final_output_path
    if [ $? -ne 0 ]; then
        echo "(ERROR)[Exx] The binary geotifcp failed to create the GeoTIFF file "$(basename $subset_inter_output_path)
        exit 1
    fi
    
#-------------------------------------------------------------------------------
# CREATE JPEG IMAGE AND COPY TO FINAL OUTPUT DIRECTORY
#-------------------------------------------------------------------------------
    # If code executes from this point onwards, it assumed that the subset image 
    # can be copied to the final output directory.
    if [ -e $subset_final_output_path ]; then
        # Create the JPEG subset image from the subset TIFF image. First 
        # construct the file name and filesystem path of the JPEG subset image. 
        jpg_subset_output_path=$output_dir/$subset_name_str.$satname.$yeardate.$band_comb.jpg
        
        convert -quality 95 $subset_final_output_path $jpg_subset_output_path &> /dev/null
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E86] The ImageMagick command 'convert' failed to convert the TIFF file "$(basename $subset_inter_output_path)" into a JPEG file."
            exit 1
        fi
        
        ## Copy GeoTIFF file
        #cp $geotiff_subset_output_name $subset_final_output_path
        # Copy metadata
        #cp $subset_inter_mdata_path $subset_final_mdata_path
    fi
done   
IFS=$OLD_IFS

# Remove all the composite image event files if debug mode is disabled.
if [ "$MIPS_DEBUG" != "YES" ]; then 
    for file_path in $(cat $comp_file_lst); do
        rm -f $file_path
    done
fi

# Remove temporary files if debug mode is disabled.
if [ "$MIPS_DEBUG" != "YES" ]; then
    rm -f flsave.pro geotiff_metadata.txt idl_output.txt runidl $subset_comb_file $tile_idr_out $geotiff_subset_output_name
    rm -f $enh_subset_output_name $subset_output_name
fi

echo 
echo "(Ended: "$(date)")"

