#!/bin/bash

# ============================================================================
# Purpose: Morning equator crossing satellites (Terra, FY3A), for example, do 
#     have day light data at the polar region when in ascending mode. Normally 
#     only day light data are in descending mode. The day light data that are 
#     in the ascending mode is of low quality. This script replaces these data 
#     with a given fill value (out_fill_val). The same is done for afternoon 
#     equator crossing satellites (Aqua, FY3B).
# 
# Input parameters:
#     1. The L1B geolocation HDF4 file of the sensor
#     2. The binary data file which will be filtered.
#     3. The name of the satellite of the data. One of the following 
#        names must be used: terra, aqua, fy3a, fy3b.
# 
# Output: The given binary file will be modified by this script. 'Low quality' 
#     solar data will be replaced by the fill value 32767.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 4 ]; then
    echo "Usage: "
    echo "    "$(basename $0)" <latitude bin file> <latitude fillvalue> <bin file> <sat name>"

    exit 1
fi

lat_bin_file=$1
lat_fillvalue=$2
bin_file=$3
sat_name=$4

lat_bin_file_basename=$(basename $lat_bin_file ".dat")

lat_nr_col=$(echo $lat_bin_file_basename | cut -d"_" -f 3)
lat_nr_row=$(echo $lat_bin_file_basename | cut -d"_" -f 4)
lat_type_val=$(echo $lat_bin_file_basename | cut -d"_" -f 5)

bin_file_basename=$(basename $bin_file ".dat")

# Get the column, row and data type values
nr_col=$(echo $bin_file_basename | cut -d"_" -f 2)
nr_row=$(echo $bin_file_basename | cut -d"_" -f 3)
type_val=$(echo $bin_file_basename | cut -d"_" -f 4)

cat > runidl << EOF
asc_desc_filt_bin_binfile, '$lat_bin_file', $lat_nr_col, $lat_nr_row, $lat_type_val, $lat_fillvalue, '$bin_file', 32767, '$sat_name', $nr_col, $nr_row, $type_val
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep --ignore-case "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E2] IDL script 'asc_desc_filt_bin_binfile.pro' could not filter file $bin_file."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

grep --ignore-case "Warning" idl_output.txt &> /dev/null
if [ $? -eq 0 ]; then
    warning_str=$(grep --ignore-case "Warning" idl_output.txt)
    echo "(WARNING)[W3] IDL script asc_desc_filt_bin_binfile: "
    echo "    "$warning_str
fi

