#!/bin/bash

# ============================================================================
# Purpose: The sensor band numbers are sometimes available in different 
#
#     resolutions - the data are available in different resolution CREFL files.
#     If all the bands are not all available in the same maximum resolution,
#     band sharpening must be done - this is done by the IDL script
#     reproject_binary.pro. This script determines if all the bands have the
#     same maximum resolution. It is assumed there is three possible
#     resolutionns. If the common maximum resolution is the highest resolution,
#     a string 'HRES' will be printed to an output file. Otherwise if it is an
#     intermediate resolution, the output file will have a string 'MRES'. And
#     if it is lowest resolution, the output file will have a string 'LRES'.
#
#     
#     If none of the bands have a common resolution, the string 'MIX' will be 
#     printed to the output file. This indicates that band sharpening must be 
#     done. 
#     
#     If all the above rules fail, the script will print 'LRES' to the output 
#     file.
# 
# Input parameters:
#     1. The red color band number. It must be a two digit number. 
#     2. The green color band number. It must be a two digit number. 
#     3. The blue color band number. It must be a two digit number. 
#     4. The the sharpen band number. It must be a two digit number. 
#     5. The filesystem path of the file to which the putput will be written to.
# 
# Output: The output will be a text file. Read the 'Purpose' clause for more 
#     information about the output.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 6 ]; then
    echo "Usage:"
    echo "    "$(basename $0)" <red band> <green band> <blue band> <lower res sharp band> <higher res sharp band> <output file>"
    
    exit 1
fi

# The heuristics: 
# 1. If R, G and B are all HRES, return HRES
# 2. If R, G and B all differ in resolution, but S is equal to one of the three, return MIX
# 3. If R, G and B are all MRES, return MRES
# 4. If R, G, and B are all LRES, return LRES

band_r=$1
band_g=$2
band_b=$3
band_ls=$4
band_hs=$5
out_fil=$6

# Test for highest resolution
hres_count=0
grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_r | grep "HRES" > /dev/null
if [ $? -eq 0 ]; then
    hres_count=$(echo "$hres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_g | grep "HRES" > /dev/null
if [ $? -eq 0 ]; then
    hres_count=$(echo "$hres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_b | grep "HRES" > /dev/null
if [ $? -eq 0 ]; then
    hres_count=$(echo "$hres_count + 1" | bc)
fi

# Test for intermediate resolution
mres_count=0
grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_r | grep "MRES" > /dev/null
if [ $? -eq 0 ]; then
    mres_count=$(echo "$mres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_g | grep "MRES" > /dev/null
if [ $? -eq 0 ]; then
    mres_count=$(echo "$mres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_b | grep "MRES" > /dev/null
if [ $? -eq 0 ]; then
    mres_count=$(echo "$mres_count + 1" | bc)
fi

# Test for lowest resolution
lres_count=0
grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_r | grep "LRES" > /dev/null
if [ $? -eq 0 ]; then
    lres_count=$(echo "$lres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_g | grep "LRES" > /dev/null
if [ $? -eq 0 ]; then
    lres_count=$(echo "$lres_count + 1" | bc)
fi

grep -v "#" $MIPS_BAND_SPEC_CONF | grep $band_b | grep "LRES" > /dev/null
if [ $? -eq 0 ]; then
    lres_count=$(echo "$lres_count + 1" | bc)
fi

## Check if the sharpen band corresponds with any of the bands
#sband_match=0
#echo "$band_r $band_g $band_b" | grep "$band_s" &> /dev/null
#if [ $? -eq 0 ]; then
#    sband_match=1
#fi
if [ ! -z $band_ls ] && [ ! -z $band_hs ]; then 
    sband_match=1
fi

grep -v '#' $MIPS_BAND_SPEC_CONF | grep $band_hs | grep "HRES" > /dev/null
if [ $? -eq 0 ]; then
    hres_count=1
fi

# Heuristic one:
if [ $hres_count -eq 3 ]; then
    echo 'HRES' > $out_fil
    exit 0
fi

if [ $sband_match -eq 1 ] && [ $hres_count -gt 0 ]; then
    echo 'MIX' > $out_fil
    exit 0
fi

if [ $mres_count -eq 3 ]; then
    echo 'MRES' > $out_fil
    exit 0
fi

if [ $sband_match -eq 1 ] && [ $mres_count -gt 0 ]; then
    echo 'MIX' > $out_fil
    exit 0
fi

echo 'LRES' > $out_fil

exit 0

