#!/bin/bash

# calc_grid_size.bash: Calculate the grid size of the given corner coordinates.
# Input: The corner coordinates, the projection that is going to be used, the resolution
# Output: <UL longitude> <UL latitude> <LR longitude> <LR latitude> <nr columns> <nr rows> <UL x> <UL y> <LR x> <LR y>
# Output: The grid size will be printed to standard output (columns, rows), then the new corner coordinates
#
# Warning: The results are printed at the output, therefore other text messages should NOT be printed to the 
#          output.

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ -z "$IDL_CMD" ]; then
  echo "Environment variable IDL_CMD is not set"
  exit 1
fi

usage_str=$(basename $0)" <OUL> <OLR> <Projection type: Normal, Polar-North or Polar-South> <Resolution> <Output file>"
if [ $# -ne 5 ]; then
    echo "(ERROR) Usage: "$usage_str
    exit 1
fi

oul=$1
olr=$2
proj_name_str=$3
resolution=$4
grid_info_txt=$5

# Define the proj4 projection strings
proj4_from_str="+proj=latlong +datum=WGS84"
case $proj_name_str in 
    "Normal" )
        # Projection script when used for ms2gt0.5
        #proj4_to_str="+proj=eqc +datum=WGS84"
        # Projection script when used for ms2gt0.22
        proj4_to_str="+proj=eqc +a=6371007.181 +b=6371007.181"
        xy_offset_coord="-180.0 -90.0"
    ;;
    
    "Polar-North" )
        proj4_to_str="+proj=stere +lat_0=90  +a=6371007.181 +b=6371007.181"
        xy_offset_coord="225.0 +0.0"
    ;;

    "Polar-South" )
        proj4_to_str="+proj=stere +lat_0=-90 +a=6371007.181 +b=6371007.181"
        xy_offset_coord="-225.0 +0.0"
    ;;
    
    * )
        echo "(ERROR) Unknow projection type is given. It should be either 'Normal' or 'Polar-South, Polar-North'"
        exit 1
    ;;
esac

# Prepare the project the latitude and longitude values
ul_lon=$(echo $oul | cut -d"," -f 1 | awk '{print $1}')
ul_lat=$(echo $oul | cut -d"," -f 2 | awk '{print $1}')
lr_lon=$(echo $olr | cut -d"," -f 1 | awk '{print $1}')
lr_lat=$(echo $olr | cut -d"," -f 2 | awk '{print $1}')

cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj4_from_str +to $proj4_to_str"
xy_oul_str=$(echo "$ul_lon $ul_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
xy_olr_str=$(echo "$lr_lon $lr_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
xy_offset_str=$(echo "$xy_offset_coord" | $cs2cs_exe_str | awk '{print $1 ", " $2}')

# Calulate the grid size and the corresponding data
cat > runidl << EOF
calc_grid_size, $xy_oul_str, $xy_olr_str, $xy_offset_str, $resolution, '$grid_info_txt'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep --ignore-case "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E17] IDL script calc_grid_size.pro failed"
    cat idl_output.txt
    exit 1
fi

# Collect the projected adjusted coordinates and the the calculate grid size
adj_ul_x=$(grep ADJ_UL_X $grid_info_txt | cut -d"=" -f 2)
adj_lr_x=$(grep ADJ_LR_X $grid_info_txt | cut -d"=" -f 2)
adj_ul_y=$(grep ADJ_UL_Y $grid_info_txt | cut -d"=" -f 2)
adj_lr_y=$(grep ADJ_LR_Y $grid_info_txt | cut -d"=" -f 2)
columns=$(grep COLS $grid_info_txt | cut -d"=" -f 2)
rows=$(grep ROWS $grid_info_txt | cut -d"=" -f 2)

# Inverse project the adjusted coordinates to get the longitude and latitude values
cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj4_to_str +to $proj4_from_str -f %.6f"
adj_lon_lat_oul_str=$(echo "$adj_ul_x $adj_ul_y" | $cs2cs_exe_str | awk '{print $1" "$2}')
adj_lon_lat_olr_str=$(echo "$adj_lr_x $adj_lr_y" | $cs2cs_exe_str | awk '{print $1" "$2}')

echo "UL_LON = "$(echo $adj_lon_lat_oul_str | cut -d" " -f 1) >> $grid_info_txt
echo "UL_LAT = "$(echo $adj_lon_lat_oul_str | cut -d" " -f 2) >> $grid_info_txt
echo "LR_LON = "$(echo $adj_lon_lat_olr_str | cut -d" " -f 1) >> $grid_info_txt
echo "LR_LAT = "$(echo $adj_lon_lat_olr_str | cut -d" " -f 2) >> $grid_info_txt

#rm -f runidl idl_output.txt

