#!/bin/bash

# ============================================================================
# Purpose: This script is used to filter high sensor view zenith angle 
#     pixels. Pixels that are of low quality have high view zenith angles. 
#     These pixels values are replaced by fill values. The swath width is 
#     therefore decreased.
# 
# Input parameters:
#     1. The CREFL binary file that will be used for the red band in the 
#         RGB composite.
#     2. The CREFL binary file that will be used for the green band in the 
#         RGB composite.
#     3. The CREFL binary file that will be used for the blue band in the 
#         RGB composite.
#     4. The sensor view zenith angle binary file. 
#     5. The threshold sensor view zenith angle value in degrees.
# 
# Output: The CREFL files will be modified.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 4 ]; then
    echo "Usage:"
    echo "    "$(basename $0)" <CREFL file> <Sensor zenith angle file> <Max sensor zenith angle> <Angle scale value>"
    
    exit 1
fi

crefl_file=$1
sen_zen_agl=$2
max_sen_zen_agl=$3
scale_value=$4

crefl_file_bsname=$(basename $crefl_file ".dat")
sen_zen_angl_bsname=$(basename $sen_zen_agl ".dat")

# Get the CREFL files' number of columns and rows
crfl_nr_cols=$(echo $crefl_file_bsname | cut -d"_" -f 2)
crfl_nr_rows=$(echo $crefl_file_bsname | cut -d"_" -f 3)
crfl_data_type=$(echo $crefl_file_bsname | cut -d"_" -f 4)

# Get the sensor zenith file's number of columns and rows
sen_zen_nr_cols=$(echo $sen_zen_angl_bsname | cut -d"_" -f 2)
sen_zen_nr_rows=$(echo $sen_zen_angl_bsname | cut -d"_" -f 3)
sen_zen_data_type=$(echo $sen_zen_angl_bsname | cut -d"_" -f 4)

cat > runidl << EOF
crefl_fill = 32767
senzen_fill = 180 * $scale_value
filter_high_sensor_zangle_bin, '$crefl_file', '$sen_zen_agl', $crfl_nr_cols, $crfl_nr_rows, $
                               $sen_zen_nr_cols, $sen_zen_nr_rows, $crfl_data_type, $sen_zen_data_type, $
                               crefl_fill, senzen_fill, $max_sen_zen_agl, $scale_value
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E27] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt

    exit 1
fi

