#!/bin/bash

# ============================================================================
# Purpose: This script calculates the corner coordinates of a normal projection, 
#     where output are a two dimensional set of tiles. The number of row tiles
#     and the number of column tiles are given as parameters. Each tile's width
#     and length are 10 degrees each. The upper left map coordinate is also
#     calculated.
# 
# Input parameters:
# number of rows - The number of row tiles. This MUST be an even number.
# number of columns - The number of column tiles. This MUST be a even number.
# out file - The filesystem path to the output text file.
# 
# Output:
# The output file contains the latitude and longtitude values of the four
# corners of the output polar projection file. The following shows an example
# of a text output file, where the fields are described.
# UL_LON=<Upper Left Longtitude floating point value>
# UL_LAT=<Upper Left Latitude floating point value>
# LR_LON=<Lower Right Longtitude floating point value>
# LR_LAT=<Lower Right Latitude floating point value>
# UR_LON=<Upper Right Longtitude floating point value>
# UR_LAT=<Upper Right Latitude floating point value>
# LL_LON=<Lower Left Longtitude floating point value>
# LL_LAT=<Lower Left Latitude floating point value>
# UL_MAP_X=<Upper left map x coordinate in meters - float>
# UL_MAP_Y=<Upper left map y coordinate in meters - float>
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 3 ]; then
    echo "Usage:"
    echo "    "$(basename $0)" <number of rows> <number of columns> <output file>"
    
    exit 1
fi

nr_rows=$1
nr_cols=$2
out_file=$3

# The number of row tiles and the number of column tiles are specified. The starting 
# tile number is at the bottom left of the output tile grid. The last tile number is 
# at the upper right of the tile grid. The corner tile numbers are now calculated.
tilenr_ll=0
tilenr_lr=$(echo "$nr_cols - 1" | bc)
tilenr_ur=$(echo "$nr_cols * $nr_rows - 1" | bc)
tilenr_ul=$(echo "$tilenr_ur - $nr_cols + 1" | bc)

# Construct an IDL procedure script calculates the tile grid corner coordinates. Each 
# tile's corner coordinates are calculated. Next the corner coordinates of the tile grid 
# are written to the output file.
cat > runidl << EOF
tile_res=10.0
gridindex_to_latlon, [$tilenr_ll, $tilenr_lr, $tilenr_ur, $tilenr_ul], $
    tile_res, lat_cnr, lon_cnr

ll_coord_lon=lon_cnr [0]
ll_coord_lat=lat_cnr [0]

lr_coord_lon=lon_cnr [1] + tile_res
lr_coord_lat=lat_cnr [1]

ur_coord_lon=lon_cnr [2] + tile_res
ur_coord_lat=lat_cnr [2] + tile_res

ul_coord_lon=lon_cnr [3]
ul_coord_lat=lat_cnr [3] + tile_res

openw, lun, '$out_file', /get_lun
printf, lun, "UL_LON=", double (ul_coord_lon)
printf, lun, "UL_LAT=", double (ul_coord_lat)
printf, lun, "LR_LON=", double (lr_coord_lon)
printf, lun, "LR_LAT=", double (lr_coord_lat)
printf, lun, "UR_LON=", double (ur_coord_lon)
printf, lun, "UR_LAT=", double (ur_coord_lat)
printf, lun, "LL_LON=", double (ll_coord_lon)
printf, lun, "LL_LAT=", double (ll_coord_lat)
free_lun, lun

exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E34] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt

    exit 1
fi

# Get the upper-left projection coordinates
ul_lon=$(grep UL_LON $out_file | cut -d"=" -f 2)
ul_lat=$(grep UL_LAT $out_file | cut -d"=" -f 2)

lr_lon=$(grep LR_LON $out_file | cut -d"=" -f 2)
lr_lat=$(grep LR_LAT $out_file | cut -d"=" -f 2)

proj4_from_str="+proj=latlong +datum=WGS84"
# Projection script when used for ms2gt0.5
#proj4_to_str="+proj=eqc +datum=WGS84"
# Projection script when used for ms2gt0.22
proj4_to_str="+proj=eqc +a=6371007.181 +b=6371007.181"
cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj4_from_str +to $proj4_to_str"

x_oul_str=$(echo "$ul_lon $ul_lat" | $cs2cs_exe_str | awk '{print $1}')
y_oul_str=$(echo "$ul_lon $ul_lat" | $cs2cs_exe_str | awk '{print $2}')

x_olr_str=$(echo "$lr_lon $lr_lat" | $cs2cs_exe_str | awk '{print $1}')
y_olr_str=$(echo "$lr_lon $lr_lat" | $cs2cs_exe_str | awk '{print $2}')

echo "UL_MAP_X="$x_oul_str >> $out_file
echo "UL_MAP_Y="$y_oul_str >> $out_file
echo "LR_MAP_X="$x_olr_str >> $out_file
echo "LR_MAP_Y="$y_olr_str >> $out_file

