#!/bin/bash

# ============================================================================
# Purpose: This script calculates the upper left and lower right corner 
#     coordinates of a specific non-polar tile. An offset value can be used 
#     to contract inwardly the corner coordinates.
# 
# Input parameters:
# tile number - The non-polar tile number who's upper left and lower right 
#     coordinates will be calculated.
# offset - A value in degrees to contract the corner coordinates inwardly.
# output txt file - The filesystem path to the output text file.
# 
# Output:
# The output file contains the latitude and longtitude values of the upper left 
# and lower right corners. The following shows an example of a text output file, 
# where the fields are described.
# UL_LON=<Upper Left Longtitude floating point value>
# UL_LAT=<Upper Left Latitude floating point value>
# LR_LON=<Lower Right Longtitude floating point value>
# LR_LAT=<Lower Right Latitude floating point value>
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 3 ]; then
    echo "Usage: "
    echo "    "$(basename $0)" <tile nr> <offset> <output txt file>"
    
    exit 1
fi

tilenr=$1
offset=$2
output_file=$3

cat > runidl << EOF
tile_res=10.0
gridindex_to_latlon, $tilenr, tile_res, lat_ll, lon_ll

lat_ul = double (lat_ll) + tile_res - $offset
lon_ul = double (lon_ll) + $offset
lat_lr = double (lat_ll) + $offset
lon_lr = double (lon_ll) + tile_res - $offset

print, 'UL_LAT=', lat_ul
print, 'UL_LON=', lon_ul
print, 'LR_LAT=', lat_lr
print, 'LR_LON=', lon_lr

exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E56] The IDL script in script "$(basename $0)" failed."
    cat idl_output.txt
    exit 1
fi

ul_lat=$(grep 'UL_LAT' idl_output.txt | cut -d"=" -f 2 | awk '{print $1}')
ul_lon=$(grep 'UL_LON' idl_output.txt | cut -d"=" -f 2 | awk '{print $1}')
lr_lat=$(grep 'LR_LAT' idl_output.txt | cut -d"=" -f 2 | awk '{print $1}')
lr_lon=$(grep 'LR_LON' idl_output.txt | cut -d"=" -f 2 | awk '{print $1}')

echo "UL_LAT=$ul_lat" >  $output_file
echo "UL_LON=$ul_lon" >> $output_file
echo "LR_LAT=$lr_lat" >> $output_file
echo "LR_LON=$lr_lon" >> $output_file

rm -f idl_output.txt runidl

proj4_from_str="+proj=latlong +datum=WGS84"
proj4_to_str="+proj=eqc +a=6371007.181 +b=6371007.181"
cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj4_from_str +to $proj4_to_str"

xy_oul_str=$(echo "$ul_lon $ul_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
xy_olr_str=$(echo "$lr_lon $lr_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')

ul_map_x=$(echo $xy_oul_str | cut -d"," -f 1)
ul_map_y=$(echo $xy_oul_str | cut -d"," -f 2)
lr_map_x=$(echo $xy_olr_str | cut -d"," -f 1)
lr_map_y=$(echo $xy_olr_str | cut -d"," -f 2)

echo "UL_MAP_X="$ul_map_x >> $output_file
echo "UL_MAP_Y="$ul_map_y >> $output_file
echo "LR_MAP_X="$lr_map_x >> $output_file
echo "LR_MAP_Y="$lr_map_y >> $output_file

