#!/bin/bash

# ============================================================================
# Purpose: This script calculates the corner coordinates of a polar projection, 
#     where output are a two dimensional set of tiles. The number of row tiles 
#     and the number of column tiles are given as parameters. Each tile's 
#     width and length are 1024 km each. The upper left map coordinate is 
#     also calculated.
# 
# Input parameters:
# number of rows - The number of row tiles. This MUST be an even number.
# number of columns - The number of column tiles. This MUST be a even number.
# polar region - This MUST be either north or south.
# out file - The filesystem path to the output text file.
# 
# Output:
# The output file contains the latitude and longtitude values of the four 
# corners of the output polar projection file. The following shows an example 
# of a text output file, where the fields are described.
# UL_LON=<Upper Left Longtitude floating point value>
# UL_LAT=<Upper Left Latitude floating point value>
# LR_LON=<Lower Right Longtitude floating point value>
# LR_LAT=<Lower Right Latitude floating point value>
# UR_LON=<Upper Right Longtitude floating point value>
# UR_LAT=<Upper Right Latitude floating point value>
# LL_LON=<Lower Left Longtitude floating point value>
# LL_LAT=<Lower Left Latitude floating point value>
# UL_MAP_X=<Upper left map x coordinate in meters - float>
# UL_MAP_Y=<Upper left map y coordinate in meters - float>
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 4 ]; then
    echo "(ERROR) Usage:"
    echo "    "$(basename $0)" <number or rows> <number of columns> <north | south> <out file>"
    
    exit 1
fi

nr_rows=$1
nr_cols=$2
region=$3
out_file=$4

# The number of row tiles and the number of column tiles are specified. The starting 
# tile number is at the bottom left of the output tile grid. The last tile number is 
# at the upper right of the tile grid. The corner tile numbers are now calculated.
tilenr_ll=0
tilenr_lr=$(echo "$nr_cols - 1" | bc)
tilenr_ur=$(echo "$nr_cols * $nr_rows - 1" | bc)
tilenr_ul=$(echo "$tilenr_ur - $nr_cols + 1" | bc)

# Make sure that the correct polar region parameter was given to the script.
polar_region_bl=0
ctr_deg_lat="0.0"
case $region in 
    "north" ) 
        polar_region_bl=1
        ctr_deg_lat="90.0"
    ;;

    "south" )
        polar_region_bl=0
        ctr_deg_lat="-90.0"
    ;;

    * )
        echo "(ERROR)[E57] The region (argument number 3) must be either north or south."
        exit 1
    ;;
esac

# Construct an IDL procedure script calculates the tile grid corner coordinates. Each 
# tile's corner coordinates are calculated. Next the corner coordinates of the tile grid 
# are written to the output file.
cat > runidl << EOF
nr_rows=$nr_rows
nr_cols=$nr_cols
tile_width = 1024.0 * 1000.0
polar_tiles_to_latlon, [$tilenr_ll, $tilenr_lr, $tilenr_ur, $tilenr_ul], $
    tile_width, $nr_rows, $nr_cols, $polar_region_bl, long_deg_arr, lat_deg_arr

ll_coord_lon=long_deg_arr [3, 0]
ll_coord_lat=lat_deg_arr [3, 0]
lr_coord_lon=long_deg_arr [2, 1]
lr_coord_lat=lat_deg_arr [2, 1]
ur_coord_lon=long_deg_arr [1, 2]
ur_coord_lat=lat_deg_arr [1, 2]
ul_coord_lon=long_deg_arr [0, 3]
ul_coord_lat=lat_deg_arr [0, 3]

stereo_graphic_proj, ul_coord_lon, ul_coord_lat, $ctr_deg_lat, 0.0,  6371007.181, ul_map_x, ul_map_y
stereo_graphic_proj, lr_coord_lon, lr_coord_lat, $ctr_deg_lat, 0.0,  6371007.181, lr_map_x, lr_map_y

openw, lun, '$out_file', /get_lun
printf, lun, "UL_LON=", double (ul_coord_lon)
printf, lun, "UL_LAT=", double (ul_coord_lat)
printf, lun, "LR_LON=", double (lr_coord_lon)
printf, lun, "LR_LAT=", double (lr_coord_lat)
printf, lun, "UR_LON=", double (ur_coord_lon)
printf, lun, "UR_LAT=", double (ur_coord_lat)
printf, lun, "LL_LON=", double (ll_coord_lon)
printf, lun, "LL_LAT=", double (ll_coord_lat)
printf, lun, "UL_MAP_X=", double (ul_map_x)
printf, lun, "UL_MAP_Y=", double (ul_map_y)
printf, lun, "LR_MAP_X=", double (lr_map_x)
printf, lun, "LR_MAP_Y=", double (lr_map_y)
free_lun, lun

exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E35] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt

    exit 1
fi

