#!/bin/bash

# ============================================================================
# Purpose: This script determines the the tile numbers which cover a specified 
#     geographical area. The type of tiles is specified by the user, which can 
#     be either non-polar or polar tiles. For non-polar tiles, the tile number 
#     range can be printed in a list format or in a table format. The table format 
#     shows the spatial arrangement of the tile numbers. With polar tiles, the 
#     tile numbers are always printed in a table format. 
# 
# Input parameters:
#     1. The projection type. The projection type can be one of the following 
#         four values:
#         a. Normal - Print the non-polar tile numbers in a list format.
#         b. Normal-table - Print the non-polar tile numbers in a table format.
#         c. Polar-North - Print the North polar tile numbers in a table format.
#         d. Polar-South - Print the South polar tile numbers in a table format.
#     2. The geographical upper left longitude value.
#     3. The geographical upper left latitude value.
#     4. The geographical lower right longitude value.
#     5. The geographical lower right latitude value.
#     6. The output text file which has all the tile number range values.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ -z "$IDL_CMD" ]; then
    echo "The environment variable IDL_CMD does not exist. Source the dbge_env.bash script."
    exit 1
fi

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

usage_str=$(basename $0)" <Projection> <UL_longitude> <UL_latitude> <LR_longitude> <LR_latitude> <output file>"
if [ $# -ne 6 ]; then
    echo "(ERROR) Usage:"
    echo "  "$usage_str
    exit 1
fi

proj_name_str=$1
ul_lon=$2
ul_lat=$3
lr_lon=$4
lr_lat=$5
out_file=$6

case $proj_name_str in 
    "Normal" )
        resolution=10
        nr_rows=18
        nr_cols=36
        print_type="PRINT_LST"
        idl_proj_name="Normal"

        proj_frm_str="+proj=latlong +datum=WGS84"
        proj_to_str="+proj=eqc +a=6371007.181 +b=6371007.181"
        
        offset_lon="-180.0"
        offset_lat="-90.0"
    ;;

    "Normal-table" )
        resolution=10
        nr_rows=18
        nr_cols=36
        print_type="PRINT_TBL"
        idl_proj_name="Normal"
        
        proj_frm_str="+proj=latlong +datum=WGS84"
        proj_to_str="+proj=eqc +a=6371007.181 +b=6371007.181"
        
        offset_lon="-180.0"
        offset_lat="-90.0"
    ;;

    "Polar-North" )
        resolution=1024
        nr_rows=8
        nr_cols=8
        print_type="PRINT_TBL"
        idl_proj_name=$proj_name_str
        
        proj_frm_str="+proj=latlong +datum=WGS84"
        proj_to_str="+proj=stere +lat_0=-90 +a=6371007.181 +b=6371007.181"
        
        offset_lon="-45.0"
        offset_lat="0.0"
    ;;
    
    "Polar-South" )
        resolution=1024
        nr_rows=8
        nr_cols=8
        print_type="PRINT_TBL"
        idl_proj_name=$proj_name_str
        
        proj_frm_str="+proj=latlong +datum=WGS84"
        proj_to_str="+proj=stere +lat_0=-90 +a=6371007.181 +b=6371007.181"
        
        offset_lon="-135.0"
        offset_lat="0.0"
    ;;
    
    * )
        echo "(ERROR)[E58] Given projection type name is incorrect."
        exit 1
    ;;
esac

cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj_frm_str +to $proj_to_str"
UL_xy=$(echo "$ul_lon $ul_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
LR_xy=$(echo "$lr_lon $lr_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
xy_offset_str=$(echo "$offset_lon $offset_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')

cat > runidl << EOF
adjust_map_xy_coord, $UL_xy, $LR_xy, $xy_offset_str, $MIPS_PC_PIXEL_RESOL, $MIPS_PC_PIXEL_RESOL, adj_ul_x, adj_ul_y, adj_lr_x, adj_lr_y
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Execution halted" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[Exx] IDL script adjust_map_xy_coord.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

adj_ul_x=$(grep "ADJ_UL_X" idl_output.txt | awk '{print $3}')
adj_ul_y=$(grep "ADJ_UL_Y" idl_output.txt | awk '{print $3}')
adj_lr_x=$(grep "ADJ_LR_X" idl_output.txt | awk '{print $3}')
adj_lr_y=$(grep "ADJ_LR_Y" idl_output.txt | awk '{print $3}')

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "ADJ_UL_X: "$adj_ul_x
    echo "ADJ_UL_Y: "$adj_ul_y
    echo "ADJ_LR_X: "$adj_lr_x
    echo "ADJ_LR_Y: "$adj_lr_y
fi

cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $proj_to_str +to $proj_frm_str -f %.6f"
adj_ul_lon=$(echo "$adj_ul_x $adj_ul_y" | $cs2cs_exe_str | awk '{print $1}')
adj_ul_lat=$(echo "$adj_ul_x $adj_ul_y" | $cs2cs_exe_str | awk '{print $2}')
adj_lr_lon=$(echo "$adj_lr_x $adj_lr_y" | $cs2cs_exe_str | awk '{print $1}')
adj_lr_lat=$(echo "$adj_lr_x $adj_lr_y" | $cs2cs_exe_str | awk '{print $2}')

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "UL_LON: "$ul_lon
    echo "UL_LAT: "$ul_lat
    echo "LR_LON: "$lr_lon
    echo "LR_LAT: "$lr_lat
    
    echo "ADJ_UL_LON: "$adj_ul_lon
    echo "ADJ_UL_LAT: "$adj_ul_lat
    echo "ADJ_LR_LON: "$adj_lr_lon
    echo "ADJ_LR_LAT: "$adj_lr_lat
fi

cat > runidl << EOF
get_tileid_range, $resolution, '$idl_proj_name', $adj_ul_lon, $adj_ul_lat, $adj_lr_lon, $adj_lr_lat, $nr_rows, $nr_cols, tile_id_arr, /$print_type
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep --ignore-case "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E46] IDL script get_tileid_range.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

grep 'tile' idl_output.txt > $out_file

rm -f runidl idl_output.txt

