#!/bin/bash

# ============================================================================
# Purpose: This script is used to convert a SDS array in a HDF4 file into a 
#     raster binary file. If geolocation data in a HDF4 file is converted into 
#     a raster binary file, this script can also be used to scale the geolocation 
#     data.
# 
# Input parameters:
#     1. The L1B HDF4 file.
#     2. The SDS array name in the L1B HDF4 file that is going to be converted 
#        into a raster binary file.
#     3. If the geolocation data need to be scaled, a scale value of either 1, 
#        2 or 4 can be given. The scale value indicates by how many multiples 
#        the geolocation data resolution must be increased.
#     4. A prefix string that will be added to the output file name.
# 
# Output: The output will be a raster binary file. The file name will include 
#     the specifications of the binary file. There are four fields, delimited  
#     by the '_' character. The fields are are follows:
#     <output prefix string>_<nr of columns>_<nr of rows>_<IDL data type>.dat
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

usage_str=$(basename $0)" <hdf file> <sds name> <scale> <out prefix str>"
if [ $# -ne 4 ]; then
    echo "(ERROR) Usage: "
    echo "    "$usage_str
    
    exit 1
fi

hdf_file=$1
sds_name=$2
scale=$3
out_prefix_str=$4

case $scale in 
    "1" )
    ;;
    "2" )
    ;;
    "4")
    ;;
    * )
        echo "(ERROR) "$(basename $0)" The scale value must be either 4, 2 or 1."
        exit 1
    ;;
esac

# If the desired file already exists, don't execute the IDL code
ls "$out_prefix_str"_*.dat &> /dev/null
if [ $? -eq 0 ]; then
    exit 0
fi

cat > runidl << EOF
hdf_to_binary, '$hdf_file', '$sds_name', $scale, '$out_prefix_str'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E18] IDL script 'hdf_to_binary' could not convert file "$(basename $hdf_file)" into binary file."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

