#!/bin/bash

# Make a subset from a tile TIF image and cut it if necessary
# 
# Usage:make_cut_subset.bash composite image file UL_latitue UL_longitude LR_latitude LR_longitude subset path
# 
# Two different error values are returned by this script. If a proj file do not have falid data, an error value of 2 
# is returned. If the subsetting IDL script failed, an error value of 1 is returned.
#
# Willem.Marais@ssec.wisc.edu

# Startup message

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ -z "$MIPS_TILE_DIR" ]; then
  echo "Environment variable MIPS_TILE_DIR is not set"
  exit 1
fi

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

usage_str=$(basename $0)" <composite image file> <UL_longitude> <UL_latitutde> <LR_longitude> <LR_latitutde> <subset image path>"
if [ $# -ne 6 ]; then
    echo "(ERROR) Usage: "
    echo "    "$usage_str
    
    exit 1
fi

comp_img_path=$1
ss_UL_lon=$2
ss_UL_lat=$3
ss_LR_lon=$4
ss_LR_lat=$5
subset_img_path=$6

# BEGIN - CONFIGURE VARIABLE
input_proj4_str="+proj=latlong +datum=WGS84"
# The projection information of the tile images
# Projection script when used for ms2gt0.5
#non_polar_proj4_str="+proj=eqc +datum=WGS84"
# Projection script when used for ms2gt0.22
non_polar_proj4_str="+proj=eqc +a=6371007.181 +b=6371007.181"
# END   - CONFIGURE VARIABLES

# Construct the input file names based upon the given argument names
file_ext_str=$(echo $comp_img_path | awk -F . '{print $NF}')
comp_proj_info_path=$(dirname $comp_img_path)/$(basename $comp_img_path ".$file_ext_str")".proj"

# Get the tile UL, LR coordinates
# Check if there are any data in the proj file. If not, print an error
grep "UL_XY" $comp_proj_info_path &> /dev/null
if [ $? -ne 0 ]; then
    echo "(ERROR)[E47] The UL_XY string is not present in the '$comp_proj_info_path' file."
    exit 1
fi
comp_UL_xy=$(grep "UL_XY" $comp_proj_info_path | cut -d"=" -f 2 | awk -F "," '{print $1 ", " $2}')

grep "LR_XY" $comp_proj_info_path &> /dev/null
if [ $? -ne 0 ]; then
    echo "(ERROR)[E48] The LR_XY string is not present in the '$comp_proj_info_path' file."
    exit 1
fi
comp_LR_xy=$(grep "LR_XY" $comp_proj_info_path | cut -d"=" -f 2 | awk -F "," '{print $1 ", " $2}')

# Convert the given longitude and latitude subset coordinates into map coordinates
cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $input_proj4_str +to $non_polar_proj4_str"
subset_UL_xy=$(echo "$ss_UL_lon $ss_UL_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
subset_LR_xy=$(echo "$ss_LR_lon $ss_LR_lat" | $cs2cs_exe_str | awk '{print $1 ", " $2}')
xy_offset_str=$(echo "-180.0 -90.0" | $cs2cs_exe_str | awk '{print $1 ", " $2}')

cat > runidl << EOF
make_cut_subset, '$comp_img_path', '$subset_img_path', $comp_UL_xy, $comp_LR_xy, $subset_UL_xy, $subset_LR_xy, $xy_offset_str, $MIPS_PC_PIXEL_RESOL, $MIPS_PC_PIXEL_RESOL, 1, '$file_ext_str'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Execution halted at" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E49] IDL script make_cut_subset.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

exit $idl_rtn

