#!/bin/bash

# ============================================================================
# Purpose: This script overlay image tiles to create a composite. The script 
#     also use the sensor zenith angle data of the tile to create an composite 
#     based on the best zenith view angle per pixel. The script is used for 
#     both non-polar and polar tiles.
#     
#     The script first creates a list of all the available tile images and
#     sensor zenith angle tiles based on the given tile number, satellite name,
#     date and band combination. Then an IDL script
#     'image_szangle_composite.pro' is used to create a composite based on the
#     list of available tiles and zenith angle tiles. After a composite has
#     been created, the composite image's metadata file is updated - all the
#     list of granules that were used to create the composite is updated.
#     Together with the metadata file, a projection text file is created for
#     the composite image file. Lastly a sharpened JPEG image is created from
#     the composite image.
# 
# Input parameters:
#     Tile directory: The directory where all the tile number directories 
#         are archived.
#     Tile number: The tile number for which a composite image must be created.
#     Satellite name: The satellite name for which a composite image must be 
#         created.
#     Year date: The date for which a composite image must be created.
#     Band combination: The band combination for which a composite image must be 
#         created.
# 
# Output: This script creates a total of four files. The naming convention of 
#     all four these files are the same, they end with a different extention. 
#     The naming convention is as follows:
#     
#     <tile number>.<satellite name>.<year date>.<band combination>.<composite number>.<jpg, png, dat, proj>
#
#     The tile number, satellite name, year date and band combination are
#     parameters given to this script. The composite number indicates the
#     number of image tiles that were used to create the composite image.  The
#     JPEG image is a sharpened version of the PNG image. The dat file is the
#     metadata file of the composite image. Lastly the proj file is projection
#     text file of the composite image. 
#
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# 
# Change log:
# 10/25/2011 - The sensor zenith angle data were always written out to binary files, 
#              and the typical size of the files were 23MB. To decrease the size of 
#              the files, the files are now written in a PNG format. 
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ -z "$IDL_CMD" ]; then
  echo "Environment variable IDL_CMD is not set"
  exit 1
fi

usage_str=$(basename $0)" <tile directory> <tile number> <satellite name> <year date> <band comb> <mosaic event prefix> <event prefix> ..."
if [ $# -lt 5 ]; then
    echo "(ERROR) Usage:"
    echo "    "$usage_str
    
    exit 1
fi

tile_directory=$1
tilenum=$2
satname=$3
yeardate=$4
band_comb=$5
shift 5
mosaic_event_prefix_lst=$*

# --------------------------------------------------------------------------------------
# CONSTRUCT TEMPORARY OUTPUT FILE NAMES
# --------------------------------------------------------------------------------------
comp_base_name=$tilenum.$satname.$yeardate.$band_comb
tmp_comp_file=$comp_base_name".png_tmp"
tmp_meta_file=$comp_base_name".dat_tmp"

# --------------------------------------------------------------------------------------
# MAKE A LIST OF AVAILABLE IMAGE TILES, AND COUNT THE NUMBER OF AVAILABLE IMAGES
# --------------------------------------------------------------------------------------
tile_dir=$tile_directory/$tilenum

png_tmp_file=$(mktemp)
szen_bin_tmp_file=$(mktemp)

# Make a list of all the available png files
ls $tile_dir/$tilenum.$satname.$yeardate.*.$band_comb.png > $png_tmp_file

OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for file_str in $(cat $png_tmp_file); do
    file_basename=$(basename $file_str ".png")
    file_part_str=$(echo $file_basename | cut -d"." -f 1-4)".szenangle.bin.gzip"
    
    echo $(dirname $file_str)/$file_part_str >> $szen_bin_tmp_file
done
IFS=$OLD_IFS

png_file_lst=$(cat $png_tmp_file | awk '{print "\x27"$1"\x27"}')
png_file_lst=$(echo $png_file_lst | tr -s ' ' ',')
png_file_lst_size=$(echo $png_file_lst | tr -s ',' '\n' | wc -l)
# Make a list of all the available sensor zenith angle binary files
szangle_file_lst=$(cat $szen_bin_tmp_file | awk '{print "\x27"$1"\x27"}')
szangle_file_lst=$(echo $szangle_file_lst | tr -s ' ' ',')

# Get a name of a tile image. This will be used to by the code which 
# will create a GeoTIFF version of the image composite.
png_tile_file_path=$(head -n 1 $png_tmp_file)
png_tile_file_name=$(basename $png_tile_file_path)

rm -f $png_tmp_file $szen_bin_tmp_file

# --------------------------------------------------------------------------------------
# CONSTRUCT OUTPUT FILE NAMES
# --------------------------------------------------------------------------------------
comp_file=$tile_dir/$comp_base_name.$png_file_lst_size.png
meta_file=$tile_dir/$comp_base_name.$png_file_lst_size.dat
jpeg_file=$tile_dir/$comp_base_name.$png_file_lst_size.jpg
proj_file=$tile_dir/$comp_base_name.$png_file_lst_size.proj
#event_file=$tile_dir/$mosaic_event_prefix.$comp_base_name.$png_file_lst_size.event
tmp_jpeg_file=$comp_base_name.$png_file_lst_size.temp.jpg

# Create the IDL script and execute it
cat > runidl << EOF
img_list = [$png_file_lst]
szenangle_list = [$szangle_file_lst]
image_szangle_composite, img_list, szenangle_list, '$tmp_comp_file'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E36] IDL script for making composite image "$(basename $comp_file)" failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

# --------------------------------------------------------------------------------------
# CREATE NEW UPDATED METADATA FILE FOR COMPOSITE IMAGE
# --------------------------------------------------------------------------------------
# Generate the metadata file names for all the listed tile images files
meta_file_lst=$(echo $png_file_lst | sed 's/png/dat/g')
# Get the first metadata file and use it as a base
select_meta_file=$(echo $meta_file_lst | cut -d"," -f 1 | sed "s/'//g")
cp $select_meta_file $tmp_meta_file

mkdir metadata
for file in $(echo $meta_file_lst | tr -s ',' ' ' | sed "s/'//g"); do
    cp $file metadata/
done

# Create the composite metadata file
gran_fil_lst=$(grep GRAN_NAME metadata/$tilenum.$satname.$yeardate.*.$band_comb.dat | awk '{print $3}')
gran_fil_lst=$(echo $gran_fil_lst | tr -s ' ' ',')
sed -i "/GRAN_NAME/ c\GRAN_NAME = $gran_fil_lst" $tmp_meta_file
if [ $? -ne 0 ]; then
    echo "(ERROR)[E37] Failed to change GRAN_NAME variable in the composite data file "$(basename $meta_file)
    exit 1
fi

rm -r metadata

# --------------------------------------------------------------------------------------
# MAKE A COPY OF THE PROJECTION FILE
# --------------------------------------------------------------------------------------
select_proj_file=$tile_dir/$(basename $select_meta_file ".dat")".proj"

# --------------------------------------------------------------------------------------
# CREATE A GEOTIFF IMAGE OF THE TILE
# --------------------------------------------------------------------------------------
file_out_basename=$(basename $comp_file ".png")
file_out_dirname=$(dirname $comp_file)
geotif_file_name=$file_out_basename".tif"

geotif_file_out=$file_out_dirname"/"$geotif_file_name
tmp_geotif_file_out="tmp_"$geotif_file_name

# Get GeoTIFF file which was used to make the composite image. 
gtif_tile_file_name=$(basename $png_tile_file_name ".png")".tif"
gtif_tile_file_path=$tile_dir/$gtif_tile_file_name

if [ "$MIPS_TILE_GEOTIFF_BL" == "YES" ]; then
    geotiff_metadata_file="geotiff_metadata.txt"
    # Create a TIFF version of the new composite image
    convert -compress None $tmp_comp_file $tmp_geotif_file_out
    
    if [ ! -e $gtif_tile_file_path ]; then
        echo "(ERROR)[E97] The file "$gtif_tile_file_path" does not exist, even though MIPS_TILE_GEOTIFF_BL=YES"
        exit 1
    fi
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG] Executing the following:"
        echo "    "$MIPS_HOME/bin/listgeo $gtif_tile_file_path 
    fi
    
    # Get the GeoTIFF metadata file
    $MIPS_HOME/bin/listgeo $gtif_tile_file_path > $geotiff_metadata_file
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E98] The binary listgeo failed to extract the GeoTIFF information out the file "$gtif_tile_file_path
        exit 1
    fi
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG] Executing the following:"
        echo "    "$MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_metadata_file $tmp_geotif_file_out $geotif_file_name
    fi
    
    # Copy the metadata into the composite image 
    $MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_metadata_file $tmp_geotif_file_out $geotif_file_name
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E99] The binary geotifcp failed to create the GeoTIFF file "$(basename $geotif_file_name)
        exit 1
    fi
    
    echo "(Created GeoTIFF image "$geotif_file_name")"
    
    # Remove the temporary files if not in debug mode
    if [ "$MIPS_DEBUG" != "YES" ]; then
        rm -f $tmp_geotif_file_out $geotiff_metadata_file
    fi
fi

# --------------------------------------------------------------------------------------
# COPY ALL THE OUTPUT FILES TO OUTPUT DIRECTORY
# --------------------------------------------------------------------------------------
echo
echo "(Creating composite image file "$(basename $comp_file)")"
cp $tmp_comp_file $comp_file
cp $tmp_meta_file $meta_file
cp $select_proj_file $proj_file
if [ "$MIPS_TILE_GEOTIFF_BL" == "YES" ]; then
    cp $geotif_file_name $geotif_file_out
    
    # Remove the temporary geotiff file if not in debug mode
    if [ "$MIPS_DEBUG" != "YES" ]; then
        rm -f $geotif_file_name
    fi
fi

# --------------------------------------------------------------------------------------
# CREATE A JPEG IMAGE OF THE COMPOSITE IMAGE
# --------------------------------------------------------------------------------------
# eval convert $MIPS_IMG_SHARPEN_CNF -quality 90 $tmp_comp_file $tmp_jpeg_file
convert $MIPS_IMG_SHARPEN_CNF $tmp_comp_file -type TrueColor $tmp_jpeg_file
if [ $? -ne 0 ]; then
    echo "(ERROR)[E38] The ImageMagick command 'convert' failed."
    exit 1
fi
mv $tmp_jpeg_file $jpeg_file

# --------------------------------------------------------------------------------------
# FINALLY CREATE THE EVENT FILES FOR THE COMPOSITE IMAGE
# --------------------------------------------------------------------------------------
for mosaic_event_prefix in $mosaic_event_prefix_lst; do
    event_file=$tile_dir/$mosaic_event_prefix.$comp_base_name.$png_file_lst_size.event
    touch $event_file
done


# Remove temporary files
rm -f $tmp_comp_file $tmp_meta_file
