#!/bin/bash

# ============================================================================
# Process granule CREFL - Land surface corrected reflectance
# 
# This script is a wrapper script for the CREFL algorithm. MODIS  L1B data can
# be processed through this script. The script takes all the L1B files
# (different resolutions) as input, as well as the CREFL output file names. 
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu Date:
# 07/22/2011
# ============================================================================

# This is a wrapper script for scripts run_modis_crefl.bash

modis_usage_str="$(basename $0) modis file_1km file_hkm file_qkm ref_1km ref_hkm ref_qkm"
viirs_usage_str="$(basename $0) viirs <GMTCO or GMODO file> <SVI or/and SVM files> <LRES CREFL> <HRES CREFL>"

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

sensor_name="$1"
if [ "$sensor_name" != "modis" ] && [ "$sensor_name" != "viirs" ]; then
    echo "(ERROR) The first argument must be either 'modis' or 'viirs'."
    echo "Usage for MODIS:"
    echo "    "$modis_usage_str
    echo "Usage for VIIRS:"
    echo "    "$viirs_usage_str
    
    exit 1
fi

case $sensor_name in
    "modis" )
        if [ $# -ne 7 ]; then
            echo "(ERROR) Usage for MODIS:"
            echo "    "$modis_usage_str
            
            exit 1
        fi
        
        # MODIS L1B input files
        file_1km=$2
        file_hkm=$3
        file_qkm=$4
        
        # MODIS CREFL output files
        ref_1km=$5
        ref_hkm=$6
        ref_qkm=$7
    ;;
    
    "viirs" )
        if [ $# -lt 3 ]; then
            echo "(ERROR) Usage for VIIRS:"
            echo "    "$viirs_usage_str
            
            exit 1
        fi

        file_geo=$2
        shift 2
        file_svm_svi_lst=$*
    ;;
esac

exit_val=0

# Create corrected reflectance
if [ "$MIPS_DEBUG" == "YES" ]; then
    case $sensor_name in 
        "modis" )
            echo run_modis_crefl.bash $file_1km $file_hkm $file_qkm $ref_1km $ref_hkm $ref_qkm
            /bin/bash $MIPS_AUX/run_modis_crefl.bash $file_1km $file_hkm $file_qkm $ref_1km $ref_hkm $ref_qkm
            exit_val=$?
        ;;
        
        "viirs" )
            echo run_viirs_crefl.bash $file_geo $*
            /bin/bash $MIPS_AUX/run_viirs_crefl.bash $file_geo $file_svm_svi_lst
            exit_val=$?
        ;;
    esac
else
    case $sensor_name in 
        "modis" )
            /bin/bash $MIPS_AUX/run_modis_crefl.bash $file_1km $file_hkm $file_qkm $ref_1km $ref_hkm $ref_qkm > /dev/null
            exit_val=$?
        ;;
         
        "viirs" )
            /bin/bash $MIPS_AUX/run_viirs_crefl.bash $file_geo $file_svm_svi_lst
            exit_val=$?
        ;;
    esac
fi

# Exit with the exit value with which the either run_modis_crefl.bash exited with. 
exit $exit_val

