#!/bin/bash

# ============================================================================
# MIPS granule image BASH script
# 
# Through this script a color image is created from the given 1KM CREFL L1B
# file. The bands that must be used to create the color image from the CREFL 
# file, are passed to the script. 
# 
# Input parameters:
# satname - The satellite name. This should be one either one of the following:
#           terra, aqua, fy3a or fy3b.
# yeardate - Thear year, month and day value of the L1B file. It must be in the 
#            format of <year><month><day>, for example: 20091231.
# utctime - The UTC time of the L1B file. It must be in the format of <hour><minute>, 
#           for example: 0130.
# band_comb - The L1B band combination that must be used to create the granule 
#             color image. For MODIS it will be 01_04_03.
# ref lres - The filesystem path to the 1KM L1B file. 
#
# Output:
# A JPEG image of the given 1KM L1B file, written to the directory specified by 
# $MIPS_GRAN_IMG_DIR.
# 
# Note: This script is used for MODIS 1KM L1B data.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_GRAN_IMG_DIR" ]; then
    echo "Environment variable MIPS_GRAN_IMG_DIR is not set"
    exit 1
fi

if [ $# -ne 6 ]; then
    echo "(ERROR) Usage:"
    echo "    "$(basename $0)" <satname> <yeardate> <utctime> <band_comb> <ref lres> <resolution string>"
    
    exit 1
fi

satname=$1
yeardate=$2
utctime=$3
band_comb=$4
ref_lres=$5
resol_str=$6
band_red_nr=$(echo $band_comb | cut -d"_" -f 1)
band_grn_nr=$(echo $band_comb | cut -d"_" -f 2)
band_blu_nr=$(echo $band_comb | cut -d"_" -f 3)

# Create the granule image output directory if it does not exist.
if [ ! -e $MIPS_GRAN_IMG_DIR ]; then
    mkdir -p $MIPS_GRAN_IMG_DIR
fi

# The IDL script which creates the granule images
# Construct the output PNG file name
out_png_granimg_path=$satname."$resol_str".$yeardate.$utctime.$band_comb."hdf.png"

out_jpg_granimg_path=$MIPS_GRAN_IMG_DIR/$satname."$resol_str".$yeardate.$utctime.$band_comb."hdf.jpg"
cat > runidl << EOF
make_granule_image_crefl,'$ref_lres','CorrRefl_$band_red_nr','CorrRefl_$band_grn_nr','CorrRefl_$band_blu_nr','$out_png_granimg_path'
exit
EOF

if [ "$MIPS_DEBUG" != "YES " ]; then
    $IDL_CMD runidl &> idl_output.txt
else
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    cat idl_output.txt
fi

grep --ignore-case "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ]; then
    echo "(ERROR)[E11] Failed to create granule image of CREFL file $ref_lres"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

case $satname in 
    # Aqua and FY3B are ascending satellites, thus the first row in the raster is the bottom part of the image
    "aqua" | "fy3b" )
        eval convert $MIPS_IMG_SHARPEN_CNF -quality 90 $out_png_granimg_path $out_jpg_granimg_path
        exit_val=$?
    ;;
    
    # Terra and FY3A are descending satellites, thus the first last in the raster is the bottom part of the image
    "terra" | "fy3a" )
        eval convert $MIPS_IMG_SHARPEN_CNF -quality 90 $out_png_granimg_path $out_jpg_granimg_path
        exit_val=$?
    ;;
    
    "npp" )
        eval convert $MIPS_IMG_SHARPEN_CNF -quality 90 $out_png_granimg_path $out_jpg_granimg_path
        exit_val=$?
    ;;
esac

if [ $exit_val -ne 0 ]; then
    echo "(ERROR)[E12] Convert failed to convert image $out_png_granimg_path to $out_jpg_granimg_path"
    exit 1
fi

rm -f $out_png_granimg_path

