#!/bin/bash

# ============================================================================
# Purpose: This script creates polar tile images for MODIS data. 
#     The script use the first argument, the polar tile event file, to
#     determine how many arguments are passed to the script. The script then
#     creates the polar tiles by executing the script run_reproject_ms2gt.bash.
#     When the polar tiles were successfully created, the script
#     make_tile_composite_zangle_nr.bash is executed to create composite tile
#     images.
# 
#     If the first argument is a 'fy3a' or a 'fy3b' event file, the following 
#     arguments must be passed to the script:
#     1. The polar tile event file
#     2. CREFL 1KM file
#     3. CREFL QKM file
#     4. L1B 1KM file
#     
#     If the first argument is a 'terra' or a 'aqua' event file, the following 
#     arguments must be passed to the script:
#     1. The polar tile event file
#     2. CREFL 1KM
#     3. CREFL HKM
#     4. CREFL QKM
#     5. The L1B geolocation file
# 
# Output: 
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# 
# Change log:
# 10/25/2011 - The sensor zenith angle data were always written out to binary files, 
#              and the typical size of the files were 23MB. To decrease the size of 
#              the files, the files are now written in a PNG format. 
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

event_file=$1

# Get tile information
event_file_name=$(basename $event_file)
tile_base_dir=$(dirname $event_file)
# We want to exclude the tile number directory name
tile_base_dir=$(dirname $tile_base_dir)

tilenum=$(echo $event_file_name | cut -d"." -f 1)
satname=$(echo $event_file_name | cut -d"." -f 2)
yeardate=$(echo $event_file_name | cut -d"." -f 3)
utctime=$(echo $event_file_name | cut -d"." -f 4)

band_comb=$(echo $event_file_name | cut -d"." -f 5)

# Get metadata file name (e.g., tile442/tile442.terra.20081218.1825.dat)
meta_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.dat

# Procedure to create TIF image: Create a temporary TIF image with a fake name, and rename it to the real name. 
# This is done to make the creation of the TIF image 'atomic'.
image_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.png
# Sensor Zenith binary file
senzen_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.szenangle.bin.gzip

if [ ! -e $event_file ]; then
    # The event file has already been processed
    exit 0
fi

# if [ ! -e $event_file ]; then
#   # It could be that the event file was already processed. If the output image 
#   # exist, then there is no error.
#   if [ ! -e $image_file ]; then
#       echo "(ERROR)[Exx] The event file "$event_file_name" does not exist."
#       exit 1
#   fi
# fi

band_red_nr=$(echo $band_comb | cut -d"_" -f 1)
band_grn_nr=$(echo $band_comb | cut -d"_" -f 2)
band_blu_nr=$(echo $band_comb | cut -d"_" -f 3)
band_low_shp_nr=$(echo $band_comb | cut -d"_" -f 4)
band_hig_shp_nr=$(echo $band_comb | cut -d"_" -f 5)

# If the band sharpening value was not given, set it 
# equal to zero.
if [ "$band_low_shp_nr" == "" ]; then
    band_low_shp_nr=0
    band_hig_shp_nr=0
fi

#-------------------------------------------------------------------------------
# READ INPUT ARGUMENTS
#-------------------------------------------------------------------------------
if [ $# -ne 6 ]; then
    echo "(ERROR) Usage: process_granule_polar_tiles.bash event_file ref_lres ref_hres latitude_bin longitude_bin sensor_zenith_angle_bin"
    
    exit 1
fi

ref_skm=$2
ref_hres=$3
latitude_bin=$4
longitude_bin=$5
sensor_zenith_angle_bin=$6

echo 
echo "(Processing data for event file "$(basename $event_file)")"

#-------------------------------------------------------------------------------
# REPROJECT CREFL DATA AND CREATE PNG IMAGE TILES
#-------------------------------------------------------------------------------
# # Get metadata file name (e.g., tile442/tile442.terra.20081218.1825.dat)
# meta_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.dat

if [ ! -e $meta_file ]; then
    echo "(ERROR)[E72] Metafile '$meta_file' does not exist"
    exit 1
fi

# Generate reprojected PNG image for this tile (e.g., tile442/tile442.fy3a.2008218.1825.03_02_01_01.png)
echo
echo "(Reprojecting image for event file "$(basename $event_file)")"

# # Procedure to create TIF image: Create a temporary TIF image with a fake name, and rename it to the real name. 
# # This is done to make the creation of the TIF image 'atomic'.
# image_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.png
# # Sensor Zenith binary file
# senzen_file=$tile_base_dir/$tilenum/$tilenum.$satname.$yeardate.$utctime.szenangle.bin.gzip

/bin/bash $MIPS_AUX/run_reproject_ms2gt.bash $latitude_bin $longitude_bin $sensor_zenith_angle_bin \
    $ref_hres $ref_skm $meta_file $image_file $senzen_file

return_value=$?

# BEGIN - For ms2gt0.22 version - BEGIN
# Check if no tile was produced.
if [ $return_value -eq 2 ]; then
    exit 0
fi
# END - For ms2gt0.22 version - END

if [ $return_value -ne 0 ]; then
    exit 1
fi

#-------------------------------------------------------------------------------
# COMPOSITING THE AVAILABLE PNG IMAGES
#-------------------------------------------------------------------------------
# Create file names
comp_base_name=$tilenum.$satname.$yeardate.$band_comb
tile_base_name=$tilenum.$satname.$yeardate.$utctime.$band_comb
# Create event file names
comp_event_file=$tile_base_dir/$tilenum/"polarmosaic".$comp_base_name.event

echo $MIPS_PROC_STEPS | grep ":comps_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Compositing image for event file "$(basename $event_file)")"
    
    /bin/bash $MIPS_AUX/make_tile_composite_zangle_nr.bash $tile_base_dir $tilenum $satname $yeardate $band_comb
    if [ $? -ne 0 ]; then
        exit 1
    fi
    
    touch $comp_event_file
fi

