#!/bin/bash

# ============================================================================
# MIPS Process granule tiles BASH script
# 
# Purpose: Based on a given tile event file, process a set of CREFL L1B files 
#     and create an image tile, composite image tile and sensor zenith angle 
#     binary file. This script is used for MODIS data.
# 
# Input parameters:
#     When MODIS data is processed, the first parameter is always 
#     tile event file. If a MODIS tile event file is given, the script accepts 
#     the following parameters:
#         event_file - Tile event file
#         ref_lres - CREFL 500m L1B file
#         ref_hres - CREFL 250m L1B file
#         ref_geo - L1B geolocation file
#     If a NPP tile event file is given, the script accepts the following 
#     parameters:
#         event_file - Tile event file
#         ref_lres - CREFL 750m L1B file
#         ref_hres - CREFL 350m L1B file
#         ref_geo - L1B geolocation file
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

event_file=$1
event_file_name=$(basename $event_file)

# Get tile information
tilenum=$(echo $event_file_name | cut -d"." -f 1)
satname=$(echo $event_file_name | cut -d"." -f 2)
yeardate=$(echo $event_file_name | cut -d"." -f 3)
utctime=$(echo $event_file_name | cut -d"." -f 4)

# Get the band combination, and the band numbers for 
# the red, green and blu channels. Also get the band 
# number which will be used for band sharpening.
band_comb=$(echo $event_file_name | cut -d"." -f 5)

# Construct the metadata file name based on the given event file name
meta_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.dat
# Construct the tile image file name
image_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.png
# Construct the sensor zenith binary file name
senzen_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.szenangle.bin.gzip

if [ ! -e $event_file ]; then
    # The event file has already been processed
    exit 0
fi

# if [ ! -e $event_file ]; then
#   # It could be that the event file was already processed. If the output image 
#   # exist, then there is no error.
#   if [ ! -e $image_file ]; then
#       echo "(ERROR)[E63] The event file "$event_file_name" does not exist."
#       exit 1
#   fi
# fi

band_red_nr=$(echo $band_comb | cut -d"_" -f 1)
band_grn_nr=$(echo $band_comb | cut -d"_" -f 2)
band_blu_nr=$(echo $band_comb | cut -d"_" -f 3)
band_low_shp_nr=$(echo $band_comb | cut -d"_" -f 4)
band_hig_shp_nr=$(echo $band_comb | cut -d"_" -f 5)

# If the band sharpening value was not given, set it 
# equal to zero.
if [ "$band_low_shp_nr" == "" ]; then
    band_low_shp_nr=0
    band_hig_shp_nr=0
fi

#-------------------------------------------------------------------------------
# READ INPUT ARGUMENTS
#-------------------------------------------------------------------------------
if [ $# -ne 6 ]; then
    echo "(ERROR) Usage: "
    echo "    "$(basename $0)" event_file ref_lres ref_hres latitude_bin longitude_bin sensor_zenith_angle_bin"
    exit 1
fi

ref_lres=$2
ref_hres=$3
latitude_bin=$4
longitude_bin=$5
sensor_zenith_angle_bin=$6

echo 
echo "(Processing data for event file "$event_file_name")"

#-------------------------------------------------------------------------------
# REPROJECT CREFL DATA AND CREATE PNG IMAGE TILES
#-------------------------------------------------------------------------------
# Construct the metadata file name based on the given event file name
# meta_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.dat
if [ ! -e $meta_file ]; then
    echo "(ERROR)[E15] Metafile '$meta_file' does not exist"
    exit 1
fi

# Generate reprojected PNG image for this tile (e.g., tile442/tile442.fy3a.2008218.1825.03_02_01_01.png)
echo
echo "(Reprojecting image for event file "$(basename $event_file)")"

# # Construct the tile image file name
# image_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.$band_comb.png
# # Construct the sensor zenith binary file name
# senzen_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$utctime.szenangle.bin.gzip

# Execute the script which create an image tile together with the sensor zenith angle binary file
/bin/bash $MIPS_AUX/run_reproject_ms2gt.bash $latitude_bin $longitude_bin $sensor_zenith_angle_bin \
    $ref_hres $ref_lres $meta_file $image_file $senzen_file

return_value=$?

# BEGIN - For ms2gt0.22 version - BEGIN
# Check if no tile was produced.
if [ $return_value -eq 2 ]; then
    exit 0
fi
# END - For ms2gt0.22 version - END

# If an error occured, exit immediately.
if [ $return_value -ne 0 ]; then
    exit 1
fi

#-------------------------------------------------------------------------------
# COMPOSITING THE AVAILABLE PNG IMAGES
#-------------------------------------------------------------------------------
# Create file names
comp_base_name=$tilenum.$satname.$yeardate.$band_comb
tile_base_name=$tilenum.$satname.$yeardate.$utctime.$band_comb
# Create event file names
#kml_event_file=$MIPS_TILE_DIR/$tilenum/"kml".$comp_base_name.event
#comp_event_file=$MIPS_TILE_DIR/$tilenum/"subset".$comp_base_name.event
mosaic_event_file=$MIPS_TILE_DIR/$tilenum/"mosaic".$comp_base_name.event

echo $MIPS_PROC_STEPS | grep ":comps_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Compositing image for event file "$(basename $event_file)")"
    
    /bin/bash $MIPS_AUX/make_tile_composite_zangle_nr.bash $MIPS_TILE_DIR $tilenum $satname $yeardate $band_comb "subset" "kml"
    if [ $? -ne 0 ]; then
        exit 1
    fi
    
    # Create event files for the KML and composite processing scripts, indicating that 
    # there is an available composite file.
    #touch $kml_event_file
    #touch $comp_event_file
    touch $mosaic_event_file
fi

