#!/bin/bash

# ============================================================================
# Purpose: This script is used to create KML files from using the global 
#     composite images. Four different types of KML files are created by this 
#     script. For each composite image that were created, a KML files are 
#     created. KML files are created for the most recent composite image. There 
#     after for each unique composite event file, a global KML file is created. 
#     Composite event files are unique in terms of the source satellite, day, 
#     and band combination. The global KML file acts as an index to all the 
#     image composite KML files. The indexing is relative to the directory 
#     where the global KML file is located. The third type of KML that is created 
#     links to a global KML file with a URL set in the MIPS configuration file 
#     (the variable is MIPS_KML_URL). This KML file is called the 'current' KML 
#     file. The fourth type of KML file is called a 'today' KML file. It links to 
#     to the most recent 'current' KML files. This KML file also automatically 
#     refreshes Google Earth to show the most recent KML images.  
#     
# Input parameters:
#     1. KML files will be created for the given band combination.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

if [ $# -ne 1 ]; then
    echo "Usage:"
    echo "    "$(basename $0)" <band comb>"
    exit 1
fi

band_comb=$1

MIPS_KML_INTERM_DIR=$MIPS_KML_DIR"/intermediate"

comp_jpeg_file=""
sensor_name=""

# First remove KML event files which are older than two days. This is to prevent 
# possible stale KML event files which could disrupt the processing
#if [ "$MIPS_DEBUG" != "YES" ]; then
#    find $MIPS_TILE_DIR -name "kml.*.$band_comb.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
#fi

# Make a list of all the available KML event files
tmp_find_kml_file_lst=$(mktemp)
tmp_kml_file_lst=$(mktemp)
tmp_interm_file=$(mktemp)
tmp_kml_filt_file_lst=$(mktemp)

#-------------------------------------------------------------------------------
# FIND AVAILABLE EVENT FILES
# 
# The event files that are available could be of different days. A list is also 
# created which lists the unique days (tmp_kml_filt_file_lst).
#-------------------------------------------------------------------------------
find $MIPS_TILE_DIR -name "kml.*.$band_comb.*.event" -mmin -$MIPS_LIFET_EVENTF -type f -print > $tmp_find_kml_file_lst

/bin/bash $MIPS_HOME/scripts/auxiliary/filter_event_file_lst.bash $tmp_find_kml_file_lst "kml" $band_comb $tmp_kml_file_lst

# Find all the unique KML event files, in terms of satellite and date
#for file_str in $(cat $tmp_kml_file_lst); do
#    tmp_file_str=$(basename $file_str)
#    tmp_file_parts=$(echo $tmp_file_str | cut -d"." -f 3-)
#    echo $tmp_file_parts >> $tmp_interm_file
#done
#sort -u $tmp_interm_file > $tmp_kml_filt_file_lst

#-------------------------------------------------------------------------------
# CREATE KML FILES FOR EACH COMPOSITE TILE - MAKE_IMAGE_TILES
# 
# For each composite event file a pyramid tiled KML files will be created. 
#-------------------------------------------------------------------------------
# Get all the KML event files and process each on of the files
for kml_event_file in $(cat $tmp_kml_file_lst); do
    kml_event_file_bsn=$(basename $kml_event_file)
    tilenum=$(echo $kml_event_file_bsn | cut -d"." -f 2)
    satname=$(echo $kml_event_file_bsn | cut -d"." -f 3)
    yeardate=$(echo $kml_event_file_bsn | cut -d"." -f 4)
    band_comb=$(echo $kml_event_file_bsn | cut -d"." -f 5)
    comp_count=$(echo $kml_event_file_bsn | cut -d"." -f 6)

    # Get the most recent composite and metadata files. First test one of the data files are 
    # available
    #ls $MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.*.dat &> /dev/null
    #if [ $? -ne 0 ]; then
    #    # There is no metadata file, but there is a corresponding event file. Exit with an 
    #    # error
    #    #echo "(ERROR)[E54] There is not corresponding metadata file for the event file "$kml_event_file_bsn
    #    
    #    echo "(WARNING)[Wxx] There is not corresponding metadata file for the event file "$kml_event_file_bsn
    #    rm -f $kml_event_file
    #    continue
    #    
    #    #exit 1
    #fi
    
    #---------------------------------------------------------------------------
    # There will be more than one composite images per tile, given that mulitple 
    # granules overlap a tile. The most recent composite image is chosen.
    #---------------------------------------------------------------------------
    
    ## Get the most recent metadata file
    #comp_mdata_tmp_file=$(mktemp)
    #ls $MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.*.dat > $comp_mdata_tmp_file
    #/bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $comp_mdata_tmp_file $comp_mdata_tmp_file 5
    #comp_mdata_file=$(tail -n 1 $comp_mdata_tmp_file)
    #rm -f $comp_mdata_tmp_file
    comp_mdata_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.$comp_count.dat
    if [ ! -e $comp_mdata_file ]; then
        echo "(WARNING)[Wxx] There is not corresponding metadata file for the event file "$kml_event_file_bsn
        #rm -f $kml_event_file
		
        continue
    fi
    
    echo $kml_event_file_bsn | cut -d"." -f 3-5 >> $tmp_interm_file
    
    #---------------------------------------------------------------------------
    # For each current KML file, the latest granule that was used in making the 
    # KML files is recorded.
    #---------------------------------------------------------------------------
    # Get the granule file name which is the 'latest' file name. The time will be written to a temporary file. The 
    # temporary file will begin with a prefix, 'latesttime'. The file name will be 
    # latesttime.<satname>.<yeardate>.<band_comb>.txt
    max_utctime_file=latesttime.$satname.$yeardate.$band_comb.txt
    gran_name_lst=$(grep "GRAN_NAME" $comp_mdata_file | awk '{print $3}')
    # Reverse the string to get the last file name and reverse it again
    last_gran_name=$(echo $gran_name_lst | sed 's/./&\n/g' | tac | tr -d '\n' | cut -d "," -f 1 | sed 's/./&\n/g' | tac | tr -d '\n')
    # Get the last time value of the granule name
    case $satname in 
        "terra" | "aqua" )
            last_utctime=$(echo $last_gran_name | cut -d"." -f 3)
        ;;
        
        "fy3a" | "fy3b" )
            last_utctime=$(echo $last_gran_name | cut -d"." -f 4)
        ;;
        
        "npp" )
            last_utctime=$(echo $last_gran_name | cut -d"_" -f 4 | cut -c2-5)
        ;;
    esac
    
    if [ ! -e $max_utctime_file ]; then
        echo $last_utctime > $max_utctime_file
    else
        tmp_val=$(cat $max_utctime_file)
        if [ "$last_utctime" -gt "$tmp_val" ]; then
            echo $last_utctime > $max_utctime_file
        fi
    fi
    
    #---------------------------------------------------------------------------
    # Make the necessary preparations to execute the IDL script 
    # make_image_tiles.pro to create the KML files per composite image.
    #---------------------------------------------------------------------------
    # Construct the composite image file
    comp_jpeg_file=$(basename $comp_mdata_file ".dat")".jpg"
    comp_jpeg_file_path=$MIPS_TILE_DIR/$tilenum/$comp_jpeg_file
    if [ ! -e $comp_jpeg_file_path ]; then
        echo "(WARNING) The metafile "$(basename $comp_mdata_file)" exist, but the image file "$comp_jpeg_file" does not exist."
        continue
    fi
    
    # Get the sensor name
    sensor_name=$(grep "SENSOR" $comp_mdata_file | awk '{print $3}')
    
    # Make the KML directory if it does not exist
    mkdir -p $MIPS_KML_DIR
    
    # Create KML for this tile
    echo "(Creating KML for "$tilenum")"
    
cat > runidl << EOF
cd, '$MIPS_KML_INTERM_DIR'
make_image_tiles, '$comp_jpeg_file', '$MIPS_TILE_DIR', '$MIPS_KML_INTERM_DIR'
exit
EOF
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        cat runidl
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
        cat idl_output.txt
    else
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
    fi
    
    grep -i "Error" idl_output.txt &> /dev/null
    if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
        echo "(ERROR)[E50] IDL script make_image_tiles.pro failed"
        echo "========= IDL script ========="
        cat runidl
        echo "========= IDL output ========="
        cat idl_output.txt

        # Remove temporary files
        rm -f $tmp_kml_file_lst $tmp_interm_file $tmp_kml_filt_file_lst

        exit 1
    fi
    
    #if [ "$MIPS_DEBUG" != "YES" ]; then
    #    rm $kml_event_file
    #    rm idl_output.txt
    #fi
    
    # Remove the current KML event file, and previous versions of it
    #rm $kml_event_file
    for count in $(seq 1 $comp_count); do
        rm -f $MIPS_TILE_DIR/$tilenum/"kml."$tilenum.$satname.$yeardate.$band_comb.$count.event
    done
    
    rm idl_output.txt
done

sort -u $tmp_interm_file > $tmp_kml_filt_file_lst
#-------------------------------------------------------------------------------
# MAKE GLOBAL KML FILES PER SATELLITE, DAY, BAND COMBINATION - MAKE_GLOBAL_KML
# 
# Use this list of tile event files. Thus the following will be executed for 
# every event file per satellite, year date and band combination.
#-------------------------------------------------------------------------------
for file_str in $(cat $tmp_kml_filt_file_lst); do
    kml_event_file=$(grep $file_str $tmp_kml_file_lst | head -n 1)
    kml_event_file_bsn=$(basename $kml_event_file)
    tilenum=$(echo $kml_event_file_bsn | cut -d"." -f 2)
    satname=$(echo $kml_event_file_bsn | cut -d"." -f 3)
    yeardate=$(echo $kml_event_file_bsn | cut -d"." -f 4)
    band_comb=$(echo $kml_event_file_bsn | cut -d"." -f 5)
	comp_count=$(echo $kml_event_file_bsn | cut -d"." -f 6)
	
    #---------------------------------------------------------------------------
    # There will be more than one composite images per tile, given that mulitple 
    # granules overlap a tile. The most recent composite image is chosen.
    #---------------------------------------------------------------------------
    #comp_mdata_tmp_file=$(mktemp)
    #ls $MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.*.dat > $comp_mdata_tmp_file
    #/bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $comp_mdata_tmp_file $comp_mdata_tmp_file 5
    #comp_mdata_file=$(tail -n 1 $comp_mdata_tmp_file)
    #rm -f $comp_mdata_tmp_file
    #comp_jpeg_file=$(basename $comp_mdata_file ".dat")".jpg"
    comp_mdata_file=$MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.$comp_count.dat
    if [ ! -e $comp_mdata_file ]; then
        echo "(WARNING)[Wxx] There is not corresponding metadata file for the event file "$kml_event_file_bsn
        
        continue
    fi
    
    #---------------------------------------------------------------------------
    # Create an image for the 'current' KML file, which shows the latest granule 
    # time value that was used to make the KML files.
    #---------------------------------------------------------------------------
    # Create "Last Update Image" for KML files, which shows the last update time of the KML files
    # Get the latest time of the given satellite name, yeardate and band combination
    max_utctime_file=latesttime.$satname.$yeardate.$band_comb.txt
    last_update_time=$(cat $max_utctime_file)
    update_jpg_file=$MIPS_KML_DIR/$satname.$yeardate.update.jpg
    case "$satname" in 
        "terra" )
            nice_satname="Terra"
        ;;
        
        "aqua" )
            nice_satname="Aqua"
        ;;
        
        "fy3a" )
            nice_satname="FY-3A"
        ;;
        
        "fy3b" )
            nice_satname="FY-3B"
        ;;
        
        "npp" )
            nice_satname="NPP"
        ;;
    esac
    nice_date_str=$(date -d $yeardate +'%Y/%m/%d')
    label_str=$nice_satname" "$sensor_name" "$nice_date_str"; Last update: "$last_update_time" UTC"
    convert -pointsize 16 -fill white -background black label:"$label_str" $update_jpg_file
    
    #---------------------------------------------------------------------------
    # Create the global KML file
    #---------------------------------------------------------------------------
    # Create super overlay KML for this tile
    echo "(Making final top-level kml files)"
    
    #comp_jpeg_file=$(basename $comp_jpeg_file)
	comp_jpeg_file=$tilenum.$satname.$yeardate.$band_comb.$comp_count.jpg
cat > runidl << EOF
cd, '$MIPS_KML_INTERM_DIR'
make_global_kml, '$comp_jpeg_file', '$MIPS_KML_INTERM_DIR'
exit
EOF
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        cat runidl
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
        cat idl_output.txt
    else
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
    fi

    grep -i "Error" idl_output.txt &> /dev/null
    if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
        echo "(ERROR)[E51] IDL script make_global_kml.pro failed"
        echo "========= IDL script ========="
        cat runidl
        echo "========= IDL output ========="
        cat idl_output.txt
        
        # Remove temporary files
        rm -f $tmp_kml_file_lst $tmp_interm_file $tmp_kml_filt_file_lst
        
        exit 1
    fi
    
    #---------------------------------------------------------------------------
    # Create the 'current' and 'today' KML files
    #---------------------------------------------------------------------------
    echo "Sensor name: "$sensor_name
    sat_array=""
    case $sensor_name in 
        "MODIS" )
            sat_array="['terra', 'aqua']"
            echo 2
        ;;
        "VIIRS" )
            sat_array="['npp']"
        ;;
    esac
    
cat > runidl << EOF
cd, '$MIPS_KML_DIR'
make_user_kml, '$sensor_name', $sat_array, '$MIPS_KML_URL', '$comp_jpeg_file', '$MIPS_SITE_NAME', $MIPS_SITE_LAT, $MIPS_SITE_LON
exit
EOF
    # Make user-end kml files
    if [ "$MIPS_DEBUG" == "YES" ]; then
        cat runidl
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
        cat idl_output.txt
    else
        $IDL_CMD runidl &> idl_output.txt
        idl_rtn=$?
    fi

    grep "Error" idl_output.txt &> /dev/null
    if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
        echo "(ERROR)[E52] IDL script make_user_kml.pro failed"
        echo "========= IDL script ========="
        cat runidl
        echo "========= IDL output ========="
        cat idl_output.txt

        # Remove temporary files
        rm -f $tmp_kml_file_lst $tmp_interm_file $tmp_kml_filt_file_lst

        exit 1
    fi
done

# Remove temporary files
rm -f $tmp_kml_file_lst $tmp_interm_file $tmp_kml_filt_file_lst $tmp_find_kml_file_lst
rm -f latesttime.*.txt
