#!/bin/bash

# ============================================================================
# Purpose: This script is a wrapper script for the program ms2gt. Band 
#     sharpening is done by the script. The script performs the following 
#     tasks to create a color tile image.
#     1. First all the data in the L1B files which are going to be used 
#        to create a color tile image, are extracted from the L1B files and 
#        are written to binary files. The following binary files are created: 
#        a. Latitude and longitude.
#        b. Sensor zenith angle.
#        c. The surface corrected reflection data, which will be a total of 
#           three file.
#        d. The surface corrected reflection data that will be used in the 
#           band sharpening process. The resolution of this data will 
#           always be 250m.
#     2. Next ascending data in Terra and FY3A data are filtered out, also  
#        descending data are removed in Aqua and FY3B data. This is done to 
#        remove solar reflection data that were observed during a night and day 
#        transition.
#     3. Out of range data in the sensor zenith angle data are filtered. This 
#        is done to make sure that there are no inconsitent values in the sensor 
#        zenith angle data.
#     4. Next CREFL solar reflection data are filtered which have a high 
#        sensor zenith angle. The threshold angle with which the data are 
#        filtered, is set by the configuration environmental variable 
#        MIPS_FILT_HSENZ_ANGLE.
#     5. The ms2gt binaries ll2cr and fornav are then executed to project 
#        the necessary data from the L1B files to create the color image tile. 
#     6. Lastly the projected data together with the band sharpening data 
#        are used to create a color image. The IDL procodure script 
#        reproject_binary.pro is used to create a color image tile.
# 
# Input parameters:
#     1. ms2gt GPD configuration file. This file is created by the BASH script 
#        run_reproject_ms2gt.bash.
#     2. A L1B geolocation file.
#     3. CREFL L1B QKM file.
#     4. CREFL L1B HKM file for MODIS.
#     5. The band number which will be used for band sharpening. The sharpen band 
#        data will be extracted from the QKM L1B file.
#     6. The band number, a two digit number, which will be used as color red. 
#     7. The band number, a two digit number, which will be used as color green.
#     8. The band number, a two digit number, which will be used as color blue.
#     9. The filesystem path of where the output image tile will be written to.
#     10. The filesystem path for the projected sensor zenith angle binary output.
#
# Output files:
#     1. The binary files that were created (latitude, longitude, sensor zenith 
#         view angle, CREFL data) are not removed at the end of the script.
#     2. The projected sensor view zenith angle file.
#     3. The projected RGB composite image.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# 
# Change log:
# 10/25/2011 - The sensor zenith angle data were always written out to binary files, 
#              and the typical size of the files were 23MB. To decrease the size of 
#              the files, the files are now written in a PNG format. 
# ============================================================================

usage_str=$(basename $0)" <MSG2GT template GPD> <Latitude binary> <Longitude binary> <Sensor zenith angle binary> <CREFL L1B HRES> <CREFL L1B LRES or MRES> <Band low S> <Band high S> <Band R> <Band G> <Band B> <Output file> <Sensor Zenith Angle file output>"
if [ $# -ne 13 ]; then
    echo "(ERROR) Usage: "
    echo "    "$usage_str
    
    exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

ms2gt_template=$1
lat_lres_bin_file=$2
lon_lres_bin_file=$3
senz_bin_file=$4
ref_crefl_hres=$5
ref_crefl_sres=$6
band_ls=$7
band_hs=$8
band_r=$9
shift 4
band_g=$6
band_b=$7
fil_out=$8
szen_out=$9

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo $(basename $0)" parameters:"
    echo "ms2gt_template: "$ms2gt_template
    echo "lat_lres_bin_file: "$lat_lres_bin_file
    echo "lon_lres_bin_file: "$lon_lres_bin_file
    echo "senz_bin_file: "$senz_bin_file
    echo "ref_crefl_hres: "$ref_crefl_hres
    echo "ref_crefl_sres: "$ref_crefl_sres
    echo "band_ls: "$band_ls
    echo "band_hs: "$band_hs
    echo "band_r: "$band_r
    echo "band_g: "$band_g
    echo "band_b: "$band_b
    echo "fil_out: "$fil_out
    echo "szen_out: "$szen_out
fi

# ============================================================================
# GATHER SOME META DATA FROM THE INPUT FILE NAMES. ALSO PREPARE OUTPUT FILE 
# NAMES
# ============================================================================
base_file_name_str=$(basename $ref_crefl_hres)
satname=$(echo $base_file_name_str | cut -d"." -f 1)
yeardate=$(echo $base_file_name_str | cut -d"." -f 3)
utctime=$(echo $base_file_name_str | cut -d"." -f 4)

crfl_band_ls="loscorrrefl$band_ls"
crfl_band_hs="hiscorrrefl$band_hs"
crfl_band_r="redcorrrefl$band_r"
crfl_band_g="grncorrrefl$band_g"
crfl_band_b="blucorrrefl$band_b"
crfl_band_lst="$crfl_band_r $crfl_band_g $crfl_band_b"
crfl_band_hres_lst="$crfl_band_hs"
crfl_band_hres_file_lst=""
crfl_band_sres_lst="$crfl_band_ls"
crfl_band_sres_file_lst=""

nr_columns=$(grep "Grid Width: " $ms2gt_template | cut -d":" -f 2 | cut -c2-)
nr_rows=$(grep "Grid Height: " $ms2gt_template | cut -d":" -f 2 | cut -c2-)

# Construct the names of the projected reflection image files
proj_ls_corrS_bin_file="proj_"$crfl_band_ls"_"$nr_columns"_"$nr_rows"_2.dat"
proj_hs_corrS_bin_file="proj_"$crfl_band_hs"_"$nr_columns"_"$nr_rows"_2.dat"
proj_red_corrR_bin_file="proj_"$crfl_band_r"_"$nr_columns"_"$nr_rows"_2.dat"
proj_grn_corrG_bin_file="proj_"$crfl_band_g"_"$nr_columns"_"$nr_rows"_2.dat"
proj_blu_corrB_bin_file="proj_"$crfl_band_b"_"$nr_columns"_"$nr_rows"_2.dat"
proj_szenangle_bin_file=$szen_out".bin"
proj_szenangle_gzip_file=$szen_out
proj_file_hres_lst=""
proj_file_sres_lst=""

# ============================================================================
# CREATE BINARY FILES FROM THE HDF FILES.
# ============================================================================
# The full names of the binaries files are not given the to the script 
# hdf_to_binary.bash. A prefix string value is given to the script. The 
# script creates binary files with the following file name formats: 
#     <string prefix>_<column dimension value>_<row dimension value>_<IDL data type nr>.dat
for line_str in $crfl_band_lst; do
    corrfl_nr_str=$(echo $line_str | cut -c 12-)
    corrfl_sds_str="CorrRefl_"$corrfl_nr_str
    
    proj_file_name_str="proj_"$line_str"_"$nr_columns"_"$nr_rows"_2.dat"
    ncdump -h $ref_crefl_sres | grep $corrfl_sds_str &> /dev/null
    if [ $? -eq 0 ]; then
        /bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl_sres $corrfl_sds_str 1 $line_str
        if [ $? -ne 0 ]; then 
            exit 1
        fi
        
        crfl_band_sres_lst=$line_str" "$crfl_band_sres_lst
        
        file_name_str=$(ls "$line_str"_*.dat)
        crfl_band_sres_file_lst=$file_name_str" "$crfl_band_sres_file_lst
        proj_file_sres_lst=$file_name_str:$proj_file_name_str" "$proj_file_sres_lst
    else
        /bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl_hres $corrfl_sds_str 1 $line_str
        if [ $? -ne 0 ]; then 
            exit 1
        fi
        
        crfl_band_hres_lst=$line_str" "$crfl_band_hres_lst
        
        file_name_str=$(ls "$line_str"_*.dat)
        crfl_band_hres_file_lst=$file_name_str" "$crfl_band_hres_file_lst
        proj_file_hres_lst=$file_name_str:$proj_file_name_str" "$proj_file_hres_lst
    fi
done

# ----------------------------------------------------------------------------
# Convert the high resolution sharpening band into a binary file
# ----------------------------------------------------------------------------
corrfl_nr_str=$(echo $crfl_band_hs | cut -c 12-)
corrfl_sds_str="CorrRefl_"$corrfl_nr_str

/bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl_hres $corrfl_sds_str 1 $crfl_band_hs
if [ $? -ne 0 ]; then 
    exit 1
fi

proj_file_name_str="proj_"$crfl_band_hs"_"$nr_columns"_"$nr_rows"_2.dat"
file_name_str=$(ls "$crfl_band_hs"_*.dat)
crfl_band_hres_file_lst=$file_name_str" "$crfl_band_hres_file_lst
proj_file_hres_lst=$file_name_str:$proj_file_name_str" "$proj_file_hres_lst

# ----------------------------------------------------------------------------
# Convert the low resolution sharpening band into a binary file
# ----------------------------------------------------------------------------
corrfl_nr_str=$(echo $crfl_band_ls | cut -c 12-)
corrfl_sds_str="CorrRefl_"$corrfl_nr_str

/bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl_sres $corrfl_sds_str 1 $crfl_band_ls
if [ $? -ne 0 ]; then 
    exit 1
fi

proj_file_name_str="proj_"$crfl_band_ls"_"$nr_columns"_"$nr_rows"_2.dat"
file_name_str=$(ls "$crfl_band_ls"_*.dat)
crfl_band_sres_file_lst=$file_name_str" "$crfl_band_lres_file_lst
proj_file_sres_lst=$file_name_str:$proj_file_name_str" "$proj_file_sres_lst

crfl_band_ls_file=$(ls "$crfl_band_ls"_*.dat)
crfl_band_hs_file=$(ls "$crfl_band_hs"_*.dat)
crfl_band_r_file=$(ls "$crfl_band_r"_*.dat)
crfl_band_g_file=$(ls "$crfl_band_g"_*.dat)
crfl_band_b_file=$(ls "$crfl_band_b"_*.dat)

# ============================================================================
# FILTER BINARY DATA; REMOVE ASCENDING DATA FROM TERRA/FY3A BINARY CREFL FILES. 
# REMOVE DESCENDING DATA FROM AQUA/FY3B BINARY CREFL FILES.
# ============================================================================
# Make a list of all the input files
crfl_band_file_lst="$crfl_band_ls_file $crfl_band_hs_file $crfl_band_r_file $crfl_band_g_file $crfl_band_b_file"

for line_str in $crfl_band_file_lst; do
    echo /bin/bash $MIPS_AUX/asc_desc_filt_bin_binfile.bash $lat_lres_bin_file $MIPS_PC_GEO_FILLVAL $line_str $satname
    /bin/bash $MIPS_AUX/asc_desc_filt_bin_binfile.bash $lat_lres_bin_file $MIPS_PC_GEO_FILLVAL $line_str $satname
    
    if [ $? -ne 0 ]; then 
        exit 1
    fi
done

# ============================================================================
# FILTER SENSOR ZENITH ANGLE BINARY DATA; REPLACE OUT OF RANGE ANGLE VALUES 
# WITH 180 DEGREE VALUES.
# ============================================================================
/bin/bash $MIPS_AUX/condition_szenangle_bin.bash $senz_bin_file $MIPS_PC_SENZA_SCALEVAL
# If an error occured, exit immediately.
if [ $? -ne 0 ]; then
    exit 1
fi

# ============================================================================
# GET TRACK AND SCAN VALUES FOR LRES AND HRES RESOLUTION BANDS, AND ALSO FOR 
# THE BAND SHARPENING BAND.
# 
# The number of tracks of the sensor, is the number of pixel columns in the
# data. The scan number of a sensor, is the number of photo diodes that
# observed a mirror scan from one end of a track to another end. For 1KM
# resolution data, there is 10 photo diodes that observed a scan. For HKM and
# QKM resolution data, the number of photo diodes where 20 and 40 diodes
# respectively.  The number of rows in a L1B swath is equal to the
# multiplication of the number of scans and the scan number.
# ============================================================================
#sres_crefl_band=$(echo $crfl_band_sres_lst | cut -d" " -f 1)
#sres_crefl_file=$(ls $sres_crefl_band_*.dat)
#hres_crefl_band=$(echo $crfl_band_hres_lst | cut -d" " -f 1)
#hres_crefl_file=$(ls $hres_crefl_band_*.dat)

# Calculate the along track and scan row values of the latitude and longitude data
lres_nr_tracks=$(echo $lat_lres_bin_file | cut -d"_" -f 3)
lres_pix_rows=$(echo $lat_lres_bin_file | cut -d"_" -f 4)
lres_pix_per_scan=$MIPS_PC_ROWS_PSCAN
lres_nr_scans=$(echo "$lres_pix_rows / $lres_pix_per_scan" | bc)

srescorr_nr_cols=$(echo $crfl_band_ls_file | cut -d "_" -f 2)
sres_factor=$(echo "$srescorr_nr_cols / $lres_nr_tracks" | bc)
sres_nr_tracks=$(echo "$lres_nr_tracks * $sres_factor" | bc)
sres_pix_rows=$(echo "$lres_pix_rows * $sres_factor" | bc)
sres_pix_per_scan=$(echo "$MIPS_PC_ROWS_PSCAN * $sres_factor" | bc)
sres_nr_scans=$(echo "$sres_pix_rows / $sres_pix_per_scan" | bc)

hrescorr_nr_cols=$(echo $crfl_band_hs_file | cut -d "_" -f 2)
hres_factor=$(echo "$hrescorr_nr_cols / $lres_nr_tracks" | bc)
hres_nr_tracks=$(echo "$lres_nr_tracks * $hres_factor" | bc)
hres_pix_rows=$(echo "$lres_pix_rows * $hres_factor" | bc)
hres_pix_per_scan=$(echo "$MIPS_PC_ROWS_PSCAN * $hres_factor" | bc)
hres_nr_scans=$(echo "$hres_pix_rows / $hres_pix_per_scan" | bc)

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "lres_nr_tracks: "$lres_nr_tracks
    echo "lres_pix_rows: "$lres_pix_rows
    echo "lres_pix_per_scan: "$lres_pix_per_scan
    echo "lres_nr_scans: "$lres_nr_scans
    echo
    echo "srescorr_nr_cols: "$srescorr_nr_cols
    echo "sres_factor: "$sres_factor
    echo "sres_nr_tracks: "$sres_nr_tracks
    echo "sres_pix_rows: "$sres_pix_rows
    echo "sres_pix_per_scan: "$sres_pix_per_scan
    echo "sres_nr_scans: "$sres_nr_scans
    echo
    echo "hrescorr_nr_cols: "$hrescorr_nr_cols
    echo "hres_nr_tracks: "$hres_nr_tracks
    echo "hres_pix_rows: "$hres_pix_rows
    echo "hres_pix_per_scan: "$hres_pix_per_scan
    echo "hres_nr_scans: "$hres_nr_scans
fi

# ============================================================================
# FITLER SOLAR REFLECTION DATA WHICH HAVE HIGH SENSOR ZENITH ANGLE VALUES.
# ============================================================================
if [ ! -z $MIPS_FILT_HSENZ_ANGL ]; then
    for file_str in $crfl_band_file_lst; do
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo "(DEBUG) Executing the following:"
            echo "    "$MIPS_AUX/filter_high_sensor_zangle_bin.bash $file_str $senz_bin_file $MIPS_FILT_HSENZ_ANGL $MIPS_PC_SENZA_SCALEVAL
        fi
        
        $MIPS_AUX/filter_high_sensor_zangle_bin.bash $file_str $senz_bin_file $MIPS_FILT_HSENZ_ANGL $MIPS_PC_SENZA_SCALEVAL
    done
    
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

# ============================================================================
# PREPARE COLUMN AND ROW IMAGE DATA FILES
# ============================================================================

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# ll2cr:
#
# Create projection grid from geographic coordinates data 
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
geo_fill_value=$MIPS_PC_GEO_FILLVAL
file_prefix_str=$satname$yeardate$utctime
lres_file_prefix="lres_"$file_prefix_str
sres_file_prefix="sres_"$file_prefix_str
hres_file_prefix="hres_"$file_prefix_str

# Preform sres projection on geographical data
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "  $MIPS_HOME/bin/ll2cr -f -F $geo_fill_value -1e30 $lres_nr_tracks $lres_nr_scans $lres_pix_per_scan $lat_lres_bin_file $lon_lres_bin_file $ms2gt_template $lres_file_prefix"
fi
$MIPS_HOME/bin/ll2cr -f -F $geo_fill_value -1e30 $lres_nr_tracks $lres_nr_scans $lres_pix_per_scan \
    $lat_lres_bin_file $lon_lres_bin_file $ms2gt_template $lres_file_prefix

if [ $? -ne 0 ]; then
    echo "(ERROR)[E21] ll2cr failed to reproject the files $lat_lres_bin_file and $lon_lres_bin_file for the LRES grid"
    
    exit 1
fi

# Get the row and column files for lres data
lres_rows_img_file=$(ls $lres_file_prefix"_rows_"*.img)
lres_cols_img_file=$(ls $lres_file_prefix"_cols_"*.img)

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# interp_colsrows.bash:
# 
# The row and column data are in the same resolution as the latitude and
# longtitude data. The binary fornav requires that the row and column data to
# be in the same resolution as the input solar reflection data.
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    "/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $sres_factor sres $lres_cols_img_file $lres_rows_img_file
fi
/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $sres_factor sres $lres_cols_img_file $lres_rows_img_file
# If an error occured, exit immediately.
if [ $? -eq 1 ]; then
    exit 1
fi

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    "/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $hres_factor hres $lres_cols_img_file $lres_rows_img_file
fi
/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $hres_factor hres $lres_cols_img_file $lres_rows_img_file
# If an error occured, exit immediately.
if [ $? -eq 1 ]; then
    exit 1
fi

# Get the row and column files for sres data.
sres_rows_img_file=$(ls $sres_file_prefix"_rows_"*.img)
sres_cols_img_file=$(ls $sres_file_prefix"_cols_"*.img)

# Get the row and column files for hres data.
hres_rows_img_file=$(ls $hres_file_prefix"_rows_"*.img)
hres_cols_img_file=$(ls $hres_file_prefix"_cols_"*.img)

# ============================================================================
# DO FORWARD PROJECTION ON THE BINARY DATA
# ============================================================================

# ----------------------------------------------------------------------------
# fornav:
#
# Forward navigate (project) the solar reflectance bands input data.
# ----------------------------------------------------------------------------

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# First project the lower resolution data
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
echo "proj_file_sres_lst: "$proj_file_sres_lst
for proj_file_sres_str in $proj_file_sres_lst; do
    echo $proj_file_sres_str
    
    input_file_str=$(echo $proj_file_sres_str | cut -d":" -f 1)
    output_file_str=$(echo $proj_file_sres_str | cut -d":" -f 2)
    
    rm -f $output_file_str
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG] Executing the following:"
        echo "    $MIPS_HOME/bin/fornav 1 -t s2 -f 32767 -F 0 -D 40 $sres_nr_tracks $sres_nr_scans $sres_pix_per_scan $sres_cols_img_file $sres_rows_img_file $input_file_str $nr_columns $nr_rows $output_file_str"
    fi
    
    $MIPS_HOME/bin/fornav 1 -t s2 -f 32767 -F 0 -D 40 $sres_nr_tracks $sres_nr_scans $sres_pix_per_scan \
        $sres_cols_img_file $sres_rows_img_file \
        $input_file_str $nr_columns $nr_rows $output_file_str
    
    if [ $? -ne 0 ]; then
        echo "(ERROR)[Exx] fornav failed to project the files $output_file_str for the low resolution grid"
        
        exit 1
    fi
    
    # BEGIN - For ms2gt0.22 version - BEGIN
    # If the projected output file is empty, fornav do not create the output files. Therefore if an error 
    # did not occur and if one of the projected output files do not exist, then the projected output data is 
    # empty.
    if [ ! -e $output_file_str ]; then
        exit 2
    fi
    # END - For ms2gt0.22 version - END
done

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# Project the higher resolution data
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
echo "proj_file_hres_lst: "$proj_file_hres_lst
for proj_file_hres_str in $proj_file_hres_lst; do
    echo $proj_file_hres_str
    
    input_file_str=$(echo $proj_file_hres_str | cut -d":" -f 1)
    output_file_str=$(echo $proj_file_hres_str | cut -d":" -f 2)
    
    rm -f $output_file_str
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG] Executing the following:"
        echo "    $MIPS_HOME/bin/fornav 1 -t s2 -f 32767 -F 0 -D 40 $hres_nr_tracks $hres_nr_scans $hres_pix_per_scan $hres_cols_img_file $hres_rows_img_file $input_file_str $nr_columns $nr_rows $output_file_str"
    fi
    
    $MIPS_HOME/bin/fornav 1 -t s2 -f 32767 -F 0 -D 40 $hres_nr_tracks $hres_nr_scans $hres_pix_per_scan \
        $hres_cols_img_file $hres_rows_img_file \
        $input_file_str $nr_columns $nr_rows $output_file_str

    if [ $? -ne 0 ]; then
        echo "(ERROR)[Exx] fornav failed to project the files $output_file_str for the high resolution grid"
        
        exit 1
    fi
    
    # BEGIN - For ms2gt0.22 version - BEGIN
    # If the projected output file is empty, fornav do not create the output files. Therefore if an error 
    # did not occur and if one of the projected output files do not exist, then the projected output data is 
    # empty.
    if [ ! -e $output_file_str ]; then
        exit 2
    fi
    # END - For ms2gt0.22 version - END
done

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# Project the sensor zenith angle data
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    $MIPS_HOME/bin/fornav 1 -t s2 -D 80 $lres_nr_tracks $lres_nr_scans $lres_pix_per_scan $lres_cols_img_file $lres_rows_img_file $senz_bin_file $nr_columns $nr_rows $proj_szenangle_bin_file"
fi
# Project the sensor zenith angle data. 
$MIPS_HOME/bin/fornav 1 -t s2 -D 80 \
    $lres_nr_tracks $lres_nr_scans $lres_pix_per_scan \
    $lres_cols_img_file $lres_rows_img_file \
    $senz_bin_file \
    $nr_columns $nr_rows \
    $proj_szenangle_bin_file

if [ $? -ne 0 ]; then
    echo "(ERROR)[Exx] fornav failed to project the file $proj_szenangle_bin_file for the low resolution grid"
    
    exit 1
fi

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# Convert the projected sensor zenith angle binary file into a compressed file
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# BEGIN - For ms2gt0.22 version - BEGIN
# If the projected output file is empty, fornav do not create the output files. Therefore if an error 
# did not occur and if one of the projected output files do not exist, then the projected output data is 
# empty.
if [ ! -e $proj_szenangle_bin_file ]; then
    exit 2
fi
# END - For ms2gt0.22 version - END

cat > runidl << EOF
make_gzip_bin, '$proj_szenangle_bin_file', $nr_columns, $nr_rows, 2, '$proj_szenangle_gzip_file'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[Exx] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt

    exit 1
fi

# ============================================================================
# CREATE A TRUECOLOR IMAGES FROM THE PROJECTION BINARY FILES.
# ============================================================================
color_enhance_arr_file=""
case $satname in
    "terra" )
        color_enhance_arr_file=$MIPS_TERRA_LENCH_FILE
    ;;

    "aqua" )
        color_enhance_arr_file=$MIPS_AQUA_LENCH_FILE
    ;;

    "fy3a" )
        color_enhance_arr_file=$MIPS_FY3A_LENCH_FILE
    ;;

    "fy3b" )
        color_enhance_arr_file=$MIPS_FY3B_LENCH_FILE
    ;;
    
    "npp" )
        color_enhance_arr_file=$MIPS_NPP_LENCH_FILE
    ;;
esac

if [ ! -e "$color_enhance_arr_file" ]; then
    echo "(ERROR)[Exx] The file "$color_enhance_arr_file" does not exist."
    exit 1
fi

tmp_color_enhance_arr_file=$PWD/'color_enhance_array_file.txt'
band_comb=$band_r"_"$band_g"_"$band_b"_"$band_s
enhance_cfg_line_str=$(grep -v '#' $color_enhance_arr_file | grep $band_comb)
echo $enhance_cfg_line_str | cut -d":" -f 2 >  $tmp_color_enhance_arr_file
echo $enhance_cfg_line_str | cut -d":" -f 3 >> $tmp_color_enhance_arr_file

cat > runidl << EOF
reproject_binary, '$proj_hs_corrS_bin_file', '$proj_ls_corrS_bin_file', ['$proj_red_corrR_bin_file', '$proj_grn_corrG_bin_file', '$proj_blu_corrB_bin_file'], '$fil_out', $nr_columns, $nr_rows, 2, ENHC_FILE='$tmp_color_enhance_arr_file'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[Exx] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

rm -f $tmp_color_enhance_arr_file $proj_szenangle_bin_file

