#!/bin/bash

# ============================================================================
# Purpose: This script is a wrapper script for the program ms2gt. The script 
#     performs the following tasks to create a color tile image.
#     1. First all the data in the L1B files which are going to be used 
#        to create a color tile image, are extracted from the L1B files and 
#        are written to binary files. The following binary files are created: 
#        a. Latitude and longtitude.
#        b. Sensor zenith angle.
#        c. The surface corrected reflection data, which will be a total of 
#           three file.
#     2. Next ascending data in Terra and FY3A data are filtered out, also  
#        descending data are removed in Aqua and FY3B data. This is done to 
#        remove solar reflection data that were observed during a night and day 
#        transition.
#     3. Out of range data in the sensor zenith angle data are filtered. This 
#        is done to make sure that there are no inconsitent values in the sensor 
#        zenith angle data.
#     4. Next CREFL solar reflection data are filtered which have a high 
#        sensor zenith angle. The threshold angle with which the data are 
#        filtered, is set by the configuration environmental variable 
#        MIPS_FILT_HSENZ_ANGLE.
#     5. The ms2gt binaries ll2cr and fornav are then executed to project 
#        the necessary data from the L1B files to create the color image tile. 
#     6. Lastly the  IDL procodure script reproject_binary.pro is used to
#        create a color image tile.
# 
# Input parameters:
#     1. ms2gt GPD configuration file. This file is created by the BASH script 
#        run_reproject_ms2gt.bash.
#     2. A L1B geolocation file.
#     3. Either a CREFL L1B QKM, or HKM or 1KM file.
#     4. The band number, a two digit number, which will be used as color red. 
#     5. The band number, a two digit number, which will be used as color green.
#     6. The band number, a two digit number, which will be used as color blue.
#     7. The filesystem path of where the output image tile will be written to.
#     8. The filesystem path for the projected sensor zenith angle binary output.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# 
# Change log:
# 10/25/2011 - The sensor zenith angle data were always written out to binary files, 
#              and the typical size of the files were 23MB. To decrease the size of 
#              the files, the files are now written in a PNG format. 
# ============================================================================

usage_str=$(basename $0)" <MSG2GT template GPD> <Latitude binary> <Longitude binary> <Sensor zenith angle binary> <CREFL L1B> <Band R> <Band G> <Band B> <Output file> <Sensor Zenith Angle file output>"
if [ $# -ne 10 ]; then
    echo "(ERROR) Usage: "
    echo "    "$usage_str
    
    exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

ms2gt_template=$1
lat_1km_bin_file=$2
lon_1km_bin_file=$3
sensor_zenith_qkm_bin_file=$4
ref_crefl=$5
band_r=$6
band_g=$7
band_b=$8
fil_out=$9
shift 1
szen_out=$9

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo $(basename $0)" parameters:"
    echo "ms2gt_template: "$ms2gt_template
    echo "lat_binary: "$lat_binary
    echo "lon_binary: "$lon_binary
    echo "senz_binary: "$senz_binary
    echo "ref_crefl: "$ref_crefl
    echo "band_r: "$band_r
    echo "band_g: "$band_g
    echo "band_b: "$band_b
    echo "fil_out: "$fil_out
    echo "szen_out: "$szen_out
fi

# ============================================================================
# GATHER SOME META DATA FROM THE INPUT FILE NAMES.
# ============================================================================
base_file_name_str=$(basename $ref_crefl)
satname=$(echo $base_file_name_str | cut -d"." -f 1)
yeardate=$(echo $base_file_name_str | cut -d"." -f 3)
utctime=$(echo $base_file_name_str | cut -d"." -f 4)
file_prefix_str=$satname$yeardate$utctime

# ============================================================================
# CREATE BINARY FILES FROM THE HDF FILES.
# ============================================================================
# The full names of the binaries files are not given the to the script 
# hdf_to_binary.bash. A prefix string value is given to the script. The 
# script creates binary files with the following file name formats: 
#     <string prefix>_<column dimension value>_<row dimension value>_<IDL data type nr>.dat
/bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl "CorrRefl_$band_r" 1 "qkmcorr$band_r"
/bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl "CorrRefl_$band_g" 1 "qkmcorr$band_g"
/bin/bash $MIPS_AUX/hdf_to_binary.bash $ref_crefl "CorrRefl_$band_b" 1 "qkmcorr$band_b"

# Get the full binary file names by using the string prefixes that were 
# given the to the hdf_to_binary.bash script.
#lat_1km_bin_file=$(ls lat_1km*.dat)
#lon_1km_bin_file=$(ls lon_1km*.dat)
#sensor_zenith_qkm_bin_file=$(ls sensorzenithqkm*.dat)
qkm_corrR_bin_file=$(ls qkmcorr"$band_r"*.dat)
qkm_corrG_bin_file=$(ls qkmcorr"$band_g"*.dat)
qkm_corrB_bin_file=$(ls qkmcorr"$band_b"*.dat)

# Check if all the necessary files have been created.
file_lst="$lat_1km_bin_file $lon_1km_bin_file $sensor_zenith_qkm_bin_file $qkm_corrR_bin_file $qkm_corrG_bin_file $qkm_corrB_bin_file"
file_missing_bl="false"
for file_path in $file_lst; do
    # List all the files that are missing.
    if [ ! -e $file_path ]; then
        echo "(ERROR)[E26] File "$file_path" is missing"
        file_missing_bl="true"
    fi
done

if [ "$file_missing_bl" == "true" ]; then
    exit 1
fi

# ============================================================================
# FILTER BINARY DATA; REMOVE ASCENDING DATA FROM TERRA/FY3A BINARY CREFL FILES. 
# REMOVE DESCENDING DATA FROM AQUA/FY3B BINARY CREFL FILES.
# ============================================================================
# Get the ms2gt map projection parameter file name, and 
# check what projection is going to be used.
/bin/bash $MIPS_AUX/asc_desc_filt_bin_binfile.bash $lat_1km_bin_file $MIPS_PC_GEO_FILLVAL $qkm_corrR_bin_file $satname
if [ $? -ne 0 ]; then 
    exit 1
fi

/bin/bash $MIPS_AUX/asc_desc_filt_bin_binfile.bash $lat_1km_bin_file $MIPS_PC_GEO_FILLVAL $qkm_corrG_bin_file $satname
if [ $? -ne 0 ]; then 
    exit 1
fi

/bin/bash $MIPS_AUX/asc_desc_filt_bin_binfile.bash $lat_1km_bin_file $MIPS_PC_GEO_FILLVAL $qkm_corrB_bin_file $satname
if [ $? -ne 0 ]; then 
    exit 1
fi

# ============================================================================
# FILTER SENSOR ZENITH ANGLE BINARY DATA; REPLACE OUT OF RANGE ANGLE VALUES 
# WITH 180 DEGREE VALUES.
# ============================================================================
/bin/bash $MIPS_AUX/condition_szenangle_bin.bash $sensor_zenith_qkm_bin_file $MIPS_PC_SENZA_SCALEVAL
# If an error occured, exit immediately.
if [ $? -ne 0 ]; then
    exit 1
fi

# ============================================================================
# GET TRACK AND SCAN VALUES FOR 1KM AND QKM RESOLUTION BANDS, AND ALSO FOR 
# THE BAND SHARPENING BAND.
# 
# The number of tracks of the sensor, is the number of pixel columns in the data.
# The scan number of a sensor, is the number of photo diodes that observed 
# a mirror scan from one end of a track to another end. For 1KM resolution data, 
# there is 10 photo diodes that observed a scan. For HKM and QKM resolution data, 
# the number of photo diodes where 20 and 40 diodes respectively.
# The number of rows in a L1B swath is equal to the multiplication of the number 
# of scans and the scan number.
# ============================================================================
qkm_nr_rows=$(echo $qkm_corrR_bin_file | cut -d"_" -f 3)
km_nr_rows=$(echo $lat_1km_bin_file | cut -d"_" -f 4)
scale_factor=$(echo "$qkm_nr_rows / $km_nr_rows" | bc)
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "qkm_nr_rows: "$qkm_nr_rows
    echo "km_nr_rows: "$km_nr_rows
    echo "scale_factor: "$scale_factor
fi

km_nr_tracks=$(echo $lat_1km_bin_file | cut -d"_" -f 3)
km_pix_rows=$(echo $lat_1km_bin_file | cut -d"_" -f 4)
km_pix_per_scan=$MIPS_PC_ROWS_PSCAN
km_nr_scans=$(echo "$km_pix_rows / $km_pix_per_scan" | bc)

qkm_nr_tracks=$(echo $qkm_corrR_bin_file | cut -d"_" -f 2)
qkm_pix_rows=$(echo $qkm_corrR_bin_file | cut -d"_" -f 3)
qkm_pix_per_scan=$(echo "$MIPS_PC_ROWS_PSCAN * $scale_factor" | bc)
qkm_nr_scans=$(echo "$qkm_pix_rows / $qkm_pix_per_scan" | bc)

# ============================================================================
# FITLER SOLAR REFLECTION DATA WHICH HAVE HIGH SENSOR ZENITH ANGLE VALUES.
# ============================================================================
if [ ! -z $MIPS_FILT_HSENZ_ANGL ]; then
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "(DEBUG) Executing the following:"
        echo "    "$MIPS_AUX/filter_high_sensor_zangle_bin.bash $qkm_corrR_bin_file $qkm_corrG_bin_file $qkm_corrB_bin_file $sensor_zenith_qkm_bin_file $MIPS_FILT_HSENZ_ANGL $MIPS_PC_SENZA_SCALEVAL
    fi
    
    $MIPS_AUX/filter_high_sensor_zangle_bin.bash $qkm_corrR_bin_file $qkm_corrG_bin_file $qkm_corrB_bin_file \
        $sensor_zenith_qkm_bin_file $MIPS_FILT_HSENZ_ANGL $MIPS_PC_SENZA_SCALEVAL

    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

# ============================================================================
# PREPARE OUTPUT FILES NAMES FOR THE PROJECTED BINARY FILES.
# ============================================================================
nr_columns=$(grep "Grid Width: " $ms2gt_template | cut -d":" -f 2 | cut -c2-)
nr_rows=$(grep "Grid Height: " $ms2gt_template | cut -d":" -f 2 | cut -c2-)

# Construct the names of the projected reflection image files
proj_qkm_corrR_bin_file="proj_qkm_corr"$band_r"_"$nr_columns"_"$nr_rows"_2.dat"
proj_qkm_corrG_bin_file="proj_qkm_corr"$band_g"_"$nr_columns"_"$nr_rows"_2.dat"
proj_qkm_corrB_bin_file="proj_qkm_corr"$band_b"_"$nr_columns"_"$nr_rows"_2.dat"
proj_szenangle_bin_file=$szen_out".bin"
proj_szenangle_png_file=$szen_out

km_file_prefix="km_"$file_prefix_str
qkm_file_prefix="ukm_"$file_prefix_str
# ============================================================================
# PROJECT THE INPUT BINARY FILES WITH M2GT PROGRAM.
# ============================================================================
#
# ----------------------------------------------------------------------------
# The ms2gt program
# ----------------------------------------------------------------------------
# 
# The ms2gt program has two executable binary files, ll2cr and fornav, which
# are used to project sensor data which are similiar to the MODIS 
# sensor. For more information about ms2gt, refer to the following website:
# http://nsidc.org/data/modis/ms2gt
# 
# The ll2cr algorithm transform latitude and longtitude pairs into column and
# row pair values. Each row and column pair value refer to a specific location
# on output projected image grid. Each pair value is associated with a pixel in
# the solar reflection and the sensor zenith angle data.
# 
# MODIS has groups of row data called scans. The geographic coordinates
# of the data from one scan to another scan are not continous. The reason for
# this anomaly is influenced by the following two factors: 1) how the sensor
# observe data in terms of the photo diode layout in the sensor, 2) and view
# angle geomertry of each scan.  Given this anomaly the ll2rc binary needs the
# scan number value, the number of scans in the data and the number of tracks
# in the data.
# 
# The row and column output file names are not explicitly specified when ll2cr 
# is executed. A prefix string is given, through which the output files can 
# be identified.
# 
# The fornav algorithm use the row and column files to determine what pixels in
# the input data should be copied to the projected output grid data. Given that
# there is not a one to one mapping from the unprojected data to the output
# grid, the fornav algorithm has to do fill in the missing grid cells in the
# output file. This is done by using an elliptical weight function, where
# neighboring pixel values are used in the input data.

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# ll2cr:
#
# Create projection grid from geographic coordinates data 
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

geo_fill_value=$MIPS_PC_GEO_FILLVAL
#case $satname in 
#    "fy3a" | "fy3b" )
#        geo_fill_value=999.9
#    ;;
#
#    "terra" | "aqua" )
#        geo_fill_value=-999.0
#    ;;
#esac

# Preform qkm projection on geographical data
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "  $MIPS_HOME/bin/ll2cr -v -f -F $geo_fill_value -1e30 $km_nr_tracks $km_nr_scans $km_pix_per_scan $lat_1km_bin_file $lon_1km_bin_file $ms2gt_template $km_file_prefix"
fi
$MIPS_HOME/bin/ll2cr -v -f -F $geo_fill_value -1e30 $km_nr_tracks $km_nr_scans $km_pix_per_scan \
    $lat_1km_bin_file $lon_1km_bin_file $ms2gt_template $km_file_prefix

if [ $? -ne 0 ]; then
    echo "(ERROR)[E28] ll2cr failed to reproject the files $lat_1km_bin_file and $lon_1km_bin_file for the QKM grid"
    
    exit 1
fi

# Get the row and column files for 1km data
km_rows_img_file=$(ls $km_file_prefix"_rows_"*.img)
km_cols_img_file=$(ls $km_file_prefix"_cols_"*.img)

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# interp_colsrows.bash:
# 
# The row and column data are in the same resolution as the latitude and
# longtitude data. The binary fornav requires that the row and column data to
# be in the same resolution as the input solar reflection data.
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    "/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $scale_factor ukm $km_cols_img_file $km_rows_img_file
fi
/bin/bash $MIPS_HOME/scripts/auxiliary/interp_colsrows.bash $scale_factor ukm $km_cols_img_file $km_rows_img_file
# If an error occured, exit immediately.
if [ $? -eq 1 ]; then
    exit 1
fi

# Get the row and column files for qkm data.
qkm_rows_img_file=$(ls $qkm_file_prefix"_rows_"*.img)
qkm_cols_img_file=$(ls $qkm_file_prefix"_cols_"*.img)

# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# fornav:
#
# Forward navigate (project) the solar reflectance bands input data.
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

# Remove previously created projected reflectance data
rm -f $proj_qkm_corrR_bin_file $proj_qkm_corrG_bin_file $proj_qkm_corrB_bin_file

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    $MIPS_HOME/bin/fornav 3 -v -t s2 s2 s2 -f 32767 32767 32767 -F 0 0 0 -D 40 $qkm_nr_tracks $qkm_nr_scans $qkm_pix_per_scan $qkm_cols_img_file $qkm_rows_img_file $qkm_corrR_bin_file $qkm_corrG_bin_file $qkm_corrB_bin_file $nr_columns $nr_rows $proj_qkm_corrR_bin_file $proj_qkm_corrG_bin_file $proj_qkm_corrB_bin_file"
fi
# Project solar reflection bands.
$MIPS_HOME/bin/fornav 3 -v -t s2 s2 s2 -f 32767 32767 32767 -F 0 0 0 -D 40 \
    $qkm_nr_tracks $qkm_nr_scans $qkm_pix_per_scan \
    $qkm_cols_img_file $qkm_rows_img_file \
    $qkm_corrR_bin_file $qkm_corrG_bin_file $qkm_corrB_bin_file \
    $nr_columns $nr_rows \
    $proj_qkm_corrR_bin_file \
    $proj_qkm_corrG_bin_file \
    $proj_qkm_corrB_bin_file

if [ $? -ne 0 ]; then
    echo "(ERROR)[E29] fornav failed to project the files $proj_qkm_corrR_bin_file, $proj_qkm_corrG_bin_file, $proj_qkm_corrB_bin_file for the QKM grid"
    exit 1
fi

# BEGIN - For ms2gt0.22 version - BEGIN
# If the projected output file is empty, fornav do not create the output files. Therefore if an error 
# did not occur and if one of the projected output files do not exist, then the projected output data is 
# empty.
if [ ! -e $proj_qkm_corrR_bin_file ]; then
    exit 2
fi
# END - For ms2gt0.22 version - END

# Remove previously created projected reflectance data
rm -f $proj_szenangle_bin_file

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG] Executing the following:"
    echo "    $MIPS_HOME/bin/fornav 1 -v -t s2 -D 40 $km_nr_tracks $km_nr_scans $km_pix_per_scan $km_cols_img_file $km_rows_img_file $sensor_zenith_qkm_bin_file $nr_columns $nr_rows $proj_szenangle_bin_file"
fi
# Project solar reflection band which will be used for band sharpening. This data 
# will be in 250m resolution format.
$MIPS_HOME/bin/fornav 1 -v -t s2 -D 40 \
    $km_nr_tracks $km_nr_scans $km_pix_per_scan \
    $km_cols_img_file $km_rows_img_file \
    $sensor_zenith_qkm_bin_file \
    $nr_columns $nr_rows \
    $proj_szenangle_bin_file

if [ $? -ne 0 ]; then
    echo "(ERROR)[E68] fornav failed to project the file $proj_szenangle_bin_file for the QKM grid"
    exit 1
fi

# BEGIN - For ms2gt0.22 version - BEGIN
# If the projected output file is empty, fornav do not create the output files. Therefore if an error 
# did not occur and if one of the projected output files do not exist, then the projected output data is 
# empty.
if [ ! -e $proj_szenangle_bin_file ]; then
    exit 2
fi
# END - For ms2gt0.22 version - END

# Convert the projected sensor zenith angle binary file into a PNG file
convert -size $nr_columns"x"$nr_rows -depth 16 gray:$proj_szenangle_bin_file png:$proj_szenangle_png_file
if [ $? -ne 0 ]; then
    echo "(ERROR)[Exx] convert failed to create a PNG version of the sensor zenith angle binary data"
    exit 1
fi

# ============================================================================
# CREATE A TRUECOLOR IMAGES FROM THE PROJECTION BINARY FILES.
# ============================================================================
color_enhance_arr_file=""
case $satname in
    "terra" )
        color_enhance_arr_file=$MIPS_TERRA_LENCH_FILE
    ;;

    "aqua" )
        color_enhance_arr_file=$MIPS_AQUA_LENCH_FILE
    ;;

    "fy3a" )
        color_enhance_arr_file=$MIPS_FY3A_LENCH_FILE
    ;;

    "fy3b" )
        color_enhance_arr_file=$MIPS_FY3B_LENCH_FILE
    ;;

    "npp" )
        color_enhance_arr_file=$MIPS_NPP_LENCH_FILE
    ;;
esac

if [ ! -e "$color_enhance_arr_file" ]; then
    echo "(ERROR)[Exx] The file "$color_enhance_arr_file" does not exist."
    exit 1
fi

tmp_color_enhance_arr_file=$PWD/'color_enhance_array_file.txt'
band_comb=$band_r"_"$band_g"_"$band_b
enhance_cfg_line_str=$(grep -v '#' $color_enhance_arr_file | grep $band_comb)
echo $enhance_cfg_line_str | cut -d":" -f 2 >  $tmp_color_enhance_arr_file
echo $enhance_cfg_line_str | cut -d":" -f 3 >> $tmp_color_enhance_arr_file

cat > runidl << EOF
reproject_binary_qkm, ['$proj_qkm_corrR_bin_file', '$proj_qkm_corrG_bin_file', '$proj_qkm_corrB_bin_file'], '$fil_out', $nr_columns, $nr_rows, 2, ENHC_FILE='$tmp_color_enhance_arr_file'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E30] The following IDL script failed."
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt

    exit 1
fi

rm -f $tmp_color_enhance_arr_file $proj_szenangle_bin_file

