#!/bin/bash

# ============================================================================
# MIPS metadata processer BASH script
# 
# Each L1B files overlaps server tiles. This script determines which tiles are 
# overlapped by the given L1B files (of the same date), and creates the necessary 
# metadata and event files.
# 
# Input parameters:
# Geolocation file - A L1B geolocation file
# Sensor name - This must be MODIS .
# Satellite name - This must be either aqua, terra, fy3a or fy3b.
# Date - The date of the satellite pass, in the format of <year><month><day>. For 
#        example, 20111231.
# Time - UTC time, in the format of <hour><minutes>. For example, 0631.
# Tile output directory - The directory where in the metadata files and event file 
#                         will be written to. Each of these files will be written 
#                         in each own's tile directory.
# 
# Output:
# For all the possible tiles which are overlapped by the given L1B, a tile number 
# directory is created. In each tile directory, for each band combination both a 
# metadata and event file are created. The file numbers range from 0 to 647, and 
# tile directories are from tile000 to tile647.
# The file format for the metadata files are: 
#     <tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.dat
# The file format for the event files are:
#     <tile number>.<satellite name>.<yeardate>.<utc time>.<band combination>.event
# 
# Notes:
# 1. The file given by the environment variable $MIPS_TEXCLUDE_CONF, gives a list of 
#    tile numbers which should not be processed. The IDL script 'write_global_metadata.pro' 
#    use this file to skip the processing of specific tiles.
# 2. The different band combinations for which tile images should be created, is 
#    specified by a file given by the environmental variable $MIPS_BAND_COMB_CONF. 
#    The band combinations are passed onto the IDL script 'write_global_metadata.pro'.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo "[DEBUG]: "$(basename $0)" arguments:"
    echo "    "$*
fi

if [ $# -ne 8 ]; then
    echo "(ERROR) Usage: "
    echo "    "$(basename $0)" <geo file path> <latitude binary file> <longitude binary file> <MODIS | VIIRS> <aqua | terra | fy3a | fy3b | npp> <yyyymmdd> <hhmm> <tile dir path>"
    
    exit 1
fi

# Get arguments
geo_file=$1
lat_bin_file=$2
lon_bin_file=$3
sensorname=$4
satname=$5
yeardate=$6
utctime=$7
tiledir=$8

# Set tile resolution (degrees)
tileres="10"

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# Get the list of band combinations given by a file specified 
# by the environmental variable $MIPS_BAND_COMB_CONF. If the 
# file is empty, use MIPS_DEFAULT_BANDCOMB.
mips_band_comb_file=$(mktemp)
if [ -e $MIPS_BAND_COMB_CONF ]; then
    grep -v "#" $MIPS_BAND_COMB_CONF > $mips_band_comb_file
fi

band_comb_lst="["
if [ -s $mips_band_comb_file ]; then
    # The file is of not zero size
    OLD_IFS=$IFS
    IFS=$(echo -e '\n\b')
    for line_str in $(cat $mips_band_comb_file); do
        red_bnr=$(echo $line_str | awk '{print $1}')
        grn_bnr=$(echo $line_str | awk '{print $2}')
        blu_bnr=$(echo $line_str | awk '{print $3}')
        lsh_bnr=$(echo $line_str | awk '{print $4}')
        hsh_bnr=$(echo $line_str | awk '{print $5}')

        if [ "$lsh_bnr" == "" ]; then
            lsh_bnr=0
        fi
        
        if [ "$hsh_bnr" == "" ]; then
            hsh_bnr=0
        fi

        band_comb_lst=$band_comb_lst"[$red_bnr, $grn_bnr, $blu_bnr, $lsh_bnr, $hsh_bnr], "
    done
    IFS=$OLD_IFS
else
    red_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 1)
    grn_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 2)
    blu_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 3) 
    lsh_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 4)
    hsh_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 5)
    
    band_comb_lst=$band_comb_lst"[$red_bnr, $grn_bnr, $blu_bnr, $lsh_bnr, $hsh_bnr], "
fi
# To end the constructed array, add a zero element array.
band_comb_lst=$band_comb_lst"[0, 0, 0, 0, 0]]"

rm -f $mips_band_comb_file

if [ ! -e $MIPS_TEXCLUDE_CONF ]; then
    echo "(WARNING) The tile exclusion file does not exist. An empty file will be created."
    echo '' > $MIPS_TEXCLUDE_CONF
fi

# Get the width, length and the data type of the latitude and longitude data
bsname_lat_bin_file=$(basename $lat_bin_file .dat)
bin_file_width=$(echo $bsname_lat_bin_file | cut -d"_" -f 3)
bin_file_length=$(echo $bsname_lat_bin_file | cut -d"_" -f 4)
bin_file_type=$(echo $bsname_lat_bin_file | cut -d"_" -f 5)

# Run IDL script to find which tiles are covered by this granule
cat > runidl << EOF
write_global_metadata, '$lat_bin_file', '$lon_bin_file', $bin_file_width, $bin_file_length, $bin_file_type, $
    $MIPS_PC_GEO_FILLVAL, '$sensorname', '$satname', $yeardate, $utctime, $tileres, $band_comb_lst, '$tiledir', $
    '$MIPS_TEXCLUDE_CONF'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E13] IDL script write_global_metadata.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

# Edit the *.dat files, set the granule file name in the metadata files
for file in $(find $tiledir -name "tile*."$satname"."$yeardate"."$utctime"."*".dat" -type f -print); do
    sed -i "s^@GRAN_NAME@^$(basename $geo_file)^g" $file > /dev/null

    if [ $? -ne 0 ]; then
        echo "(ERROR)[E14] Failed to change GRAN_NAME value in file "$file
        exit 1
    fi
done

