#!/bin/bash

# ============================================================================
# Process MODIS granule CREFL - Land surface corrected reflectance
# 
# This script is a wrapper script for the CREFL algorithm. MODIS  L1B data can
# be procesed through this script. The script takes all the L1B files
# (different resolutions) as input, as well as the CREFL output file names. 
# 
# Note: If the output files already exist, the script will not process the 
#       input L1B data.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu 
# Date:
# 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ $# -ne 6 ]; then
    echo "(ERROR) Usage: "
    echo "    "$(basename $0)" <L1B 1km> <L1B hkm> <L1B qkm> <CREFL 1km> <CREFL hkm> <CREFL qkm>"
    exit 1
fi

# Get input arguments
fil_1km=$1
fil_hkm=$2
fil_qkm=$3

ref_1km=$4
ref_hkm=$5
ref_qkm=$6

# If the output files already exist, do not continue with the processing of the 
# input L1B data.
if [ -e $ref_1km ] && [ -e $ref_hkm ] && [ -e $ref_qkm ]; then
    exit 0
fi

#-------------------------------------------------------------------------------
# FIX SATURATED MODIS BAND VALUES
# 
# For more information about the MODIS band saturation, refer to the 
# 'modis_saturation_fix.pro' code.
#-------------------------------------------------------------------------------

tmp_fil_hkm=$(mktemp -p $PWD)
cp $fil_hkm $tmp_fil_hkm
tmp_fil_qkm=$(mktemp -p $PWD)
cp $fil_qkm $tmp_fil_qkm

nscans=$(ncdump -h $fil_1km | grep "number of scans =" | awk '{print $5}')
# Fix saturated bands in the MODIS L1B files
cat > runidl << EOF
modis_saturation_fix, '$tmp_fil_hkm', 'HKM', $nscans
modis_saturation_fix, '$tmp_fil_qkm', 'QKM', $nscans
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep --ignore-case "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E5] IDL script modis_saturation_fix.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

#-------------------------------------------------------------------------------
# EXECUTE CREFL ALGORITHM OVER INPUT L1B DATA
#-------------------------------------------------------------------------------
# Create soft links to standard filenames expected by crefl
ln -fs $fil_1km MOD021KM.tmp.hdf
ln -fs $tmp_fil_hkm MOD02HKM.tmp.hdf
ln -fs $tmp_fil_qkm MOD02QKM.tmp.hdf

# Set an environment variable for the CREFL algorithm
export ANCPATH=$MIPS_HOME/bin

# Execute CREFL for 1KM data
echo
echo "(Creating 1KM corrected reflectance in "$ref_1km")"
$MIPS_HOME/bin/crefl.1.7.1 modis --verbose --overwrite --1km --of=$ref_1km --bands=1,2,3,4,5,6,7 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf 
#$MIPS_HOME/bin/crefl.1.7.1 --verbose --overwrite --1km --of=$ref_1km --bands=1,2,3,4,5,6,7 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf 

if [ $? -ne 0 ]; then
    echo "(ERROR)[E6] CREFL failed to process 1KM L1B data"
    exit 1
fi

# Execute CREFL for HKM data
echo
echo "(Creating HKM corrected reflectance in "$ref_hkm")"
$MIPS_HOME/bin/crefl.1.7.1 modis --verbose --overwrite --500m --of=$ref_hkm --bands=1,2,3,4,5,6,7 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf
#$MIPS_HOME/bin/crefl.1.7.1 --verbose --overwrite --500m --of=$ref_hkm --bands=1,2,3,4,5,6,7 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf

if [ $? -ne 0 ]; then
    echo "(ERROR)[E7] CREFL failed to process HKM L1B data"
    exit 1
fi

# Execute CREFL for QKM data
echo
echo "(Creating QKM corrected reflectance in "$ref_qkm")"
$MIPS_HOME/bin/crefl.1.7.1 modis --verbose --overwrite --of=$ref_qkm --bands=1,2 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf
#$MIPS_HOME/bin/crefl.1.7.1 --verbose --overwrite --of=$ref_qkm --bands=1,2 MOD021KM.tmp.hdf MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf

if [ $? -ne 0 ]; then
    echo "(ERROR)[E8] CREFL failed to process QKM L1B data"
    exit 1
fi

# Remove temporary files
rm -f MOD02HKM.tmp.hdf MOD02QKM.tmp.hdf MOD021KM.tmp.hdf $tmp_fil_hkm $tmp_fil_qkm 
rm -f idl_output.txt idlsave.pro runidl

