#!/bin/bash

# Run polar metadata processing given a L1B geolocation file.
#
# Usage: run_polar_metadata.bash geolocation-file sensor name satname date time region tiledir

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ $# -ne 9 ]; then
    echo "(ERROR) Usage: "$(basename $0)" <geo file path> <latitude binary file> <longitude binary file> <MODIS | VIIRS> <aqua | terra | fy3a | fy3b | npp> <yyyymmdd> <hhmm> <region: north | south> <tile dir path>"
    exit 1
fi

# Get arguments
geo_file=$1
lat_bin_file=$2
lon_bin_file=$3
sensorname=$4
satname=$5
yeardate=$6
utctime=$7
region=$8
tiledir=$9

case $region in 
    "North" | "north" )
        region_nr=1
    ;;

    "South" | "south" )
        region_nr=0
    ;;

    * )
        echo "(ERROR)[E73] The given region name is invalid. It must be either 'north' or 'south'."
        exit 1
    ;;
esac

if [ ! -e $tiledir ]; then
    mkdir -p $tiledir
fi

# Set tile resolution (degrees)
tileres="10"

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# Get the list of band combinations given by a file specified 
# by the environmental variable $MIPS_BAND_COMB_CONF. If the 
# file is empty, use MIPS_DEFAULT_BANDCOMB.
mips_band_comb_file=$(mktemp)
if [ -e $MIPS_BAND_COMB_POLAR_CONF ]; then
    grep -v "#" $MIPS_BAND_COMB_POLAR_CONF > $mips_band_comb_file
fi

band_comb_lst="["
if [ -s $mips_band_comb_file ]; then
    # The file is of not zero size
    OLD_IFS=$IFS
    IFS=$(echo -e '\n\b')
    for line_str in $(cat $mips_band_comb_file); do
        red_bnr=$(echo $line_str | awk '{print $1}')
        grn_bnr=$(echo $line_str | awk '{print $2}')
        blu_bnr=$(echo $line_str | awk '{print $3}')
        lsh_bnr=$(echo $line_str | awk '{print $4}')
        hsh_bnr=$(echo $line_str | awk '{print $5}')
        
        if [ "$lsh_bnr" == "" ]; then
            lsh_bnr="0"
            hsh_bnr="0"
        fi
        
        band_comb_lst=$band_comb_lst"[$red_bnr, $grn_bnr, $blu_bnr, $lsh_bnr, $hsh_bnr], "
    done
    IFS=$OLD_IFS
else
    red_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 1)
    grn_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 2)
    blu_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 3) 
    lsh_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 4)
    hsh_bnr=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 5)
    
    band_comb_lst=$band_comb_lst"[$red_bnr, $grn_bnr, $blu_bnr, $lsh_bnr, $hsh_bnr], "
fi
# To end the constructed array, add a zero element array.
band_comb_lst=$band_comb_lst"[0, 0, 0, 0, 0]]"

rm -f $mips_band_comb_file

# Get the width, length and the data type of the latitude and longitude data
bsname_lat_bin_file=$(basename $lat_bin_file .dat)
bin_file_width=$(echo $bsname_lat_bin_file | cut -d"_" -f 3)
bin_file_length=$(echo $bsname_lat_bin_file | cut -d"_" -f 4)
bin_file_type=$(echo $bsname_lat_bin_file | cut -d"_" -f 5)

# Run IDL script to find which tiles are covered by this granule
cat > runidl << EOF
write_polar_metadata, '$lat_bin_file', '$lon_bin_file', $bin_file_width, $bin_file_length, $bin_file_type, $
    $MIPS_PC_GEO_FILLVAL, '$sensorname', '$satname', $yeardate, $utctime, $tileres, $band_comb_lst, $region_nr, $
    '$tiledir'
exit
EOF

if [ "$MIPS_DEBUG" == "YES" ]; then
    cat runidl
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
    cat idl_output.txt
else
    $IDL_CMD runidl &> idl_output.txt
    idl_rtn=$?
fi

grep -i "Error" idl_output.txt &> /dev/null
if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
    echo "(ERROR)[E74] IDL script write_polar_metadata.pro failed"
    echo "========= IDL script ========="
    cat runidl
    echo "========= IDL output ========="
    cat idl_output.txt
    
    exit 1
fi

# Edit the *.dat files, set the granule file name in the dat files
for file in $(find $tiledir -name "ptile*."$satname"."$yeardate"."$utctime"."*".dat" -type f -print); do
    sed -i "s^@GRAN_NAME@^$(basename $geo_file)^g" $file > /dev/null

    if [ $? -ne 0 ]; then
        echo "(ERROR)[E55] Failed to change GRAN_NAME value in file "$file
        exit 1
    fi
done

rm -f idl_output.txt idlsave.pro runidl

