#!/bin/bash

# ============================================================================
# Purpose: The purpose of this script is to create a color tile image for both
#     polar and non-polar data. The script relies on two scripts, called
#     'reproject_ms2gt_eqres.bash' and 'reproject_ms2gt_mix.bash', to create the
#     image tile. In the process of creating the image tiles, each band's L1B
#     data are projected onto a region and from these projected data a color
#     image is created. The two scripts perform these functions, while this
#     script prepares the necessary information for these script to create a
#     color image tile.
# 
#     First the script collect the input parameters and determine the mode of
#     execution. The script can either process the whole L1B (make a projected
#     version of the L1B data), or create a tile image which is a subset of the
#     L1B data. Secondly the grid size is calculated of the projected output
#     file. The grid size is a necessary input for the MS2GT projection
#     utility. Thirdly the MS2GT configuration files are constructed. The
#     projection parameters are set in the MS2GT configuration file. Next the
#     script determines if band sharping should be used to create the highest
#     possible resolution image. If all the bands are of the same resolution,
#     no band sharpening is performed. Based on the last step, either the
#     script 'reproject_ms2gt_eqres.bash' and 'reproject_ms2gt_mix.bash' is
#     executed.
# 
# Note: The band number that is used to band sharpening must be a band which 
#     is availble in the QKM L1B file.
# 
# Input parameters:
#     1. L1B geolocation file
#     2. CREFL L1B QKM file
#     3. CREFL L1B HKM or 1KM file
#     4. Either the tile metadata file, or either the values 'polar-north', 
#        'polar-south', 'normal'. With a tile metadata file, only a tile image 
#        is created. If either 'polar-north' or 'polar-south' is given, and the 
#        that covers a polar region, a polar image is created. Otherwise a 
#        rectangular projected image will be created. 
#     5. The output image filepath.
#     6. The projected sensor zenith angle binary file of the output file.
# 
# Output:
#     Three files are created. The first file is the projected image file. The 
#     second file is the projected sensor zenith angle binary file which 
#     corresponds with the projected image file. Together with the output image 
#     file, a text file is created which holds the projected information about 
#     the output files.
#
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011 
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

usage_str=$(basename $0)" <Latitude binary> <Longitude binary> <Sensor zenith angle binary> <CREFL L1B HRES> <CREFL L1B LRES> <Metadata file | 'polar-north' | 'polar-south' | 'normal'> <File out> <Sensor Zenith Angle file output>"
if [ $# -ne 8 ]; then
    echo "Usage:"
    echo "  "$usage_str
    exit 1
fi
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# Get the input arguments
if [ $6 == "polar-south" ] || [ $6 == "polar-north" ] || [ $6 == "normal" ]; then
    # If argument number 6 is either equal to 'polar-north', 'polar-south' or 'normal', the whole L1B image 
    # will be projected. The projected output image size must be calculated.
    lat_binary=$1
    lon_binary=$2
    senz_binary=$3
    ref_crefl_hres=$4
    ref_crefl_skm=$5
    # Get the default band combination values.
    band_r=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 1)
    band_g=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 2)
    band_b=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 3)
    band_ls=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 4)
    band_hs=$(echo $MIPS_DEFAULT_BANDCOMB | cut -d"_" -f 5)
    proj_type=$6
    fil_out=$7
    szen_out=$8
    
    if [ $proj_type == "normal" ]; then
        bsname_lat_binary=$(basename $lat_binary ".dat")
        lat_arr_width=$(echo $bsname_lat_binary | cut -d"_" -f 3)
        lat_arr_length=$(echo $bsname_lat_binary | cut -d"_" -f 4)
        lat_arr_type=$(echo $bsname_lat_binary | cut -d"_" -f 5)
        
        # If an equal rectangular projection is going to be used, get the 
        # upper left and lower right coordinates of the L1B image.
        out_txt_file=$(mktemp)
        echo "latlon_bin_geo_ul_lr, '$lat_binary', '$lon_binary', $lat_arr_width, $lat_arr_length, $lat_arr_type, '$out_txt_file'" > runidl
        echo "exit" >> runidl

        if [ "$MIPS_DEBUG" == "YES" ]; then
            cat runidl
            $IDL_CMD runidl &> idl_output.txt
            idl_rtn=$?
            cat idl_output.txt
        else
            $IDL_CMD runidl &> idl_output.txt
            idl_rtn=$?
        fi
        
        grep -i "Error" idl_output.txt &> /dev/null
        if [ $? -eq 0 ] || [ $idl_rtn -ne 0 ]; then
            echo "(ERROR)[E53] Could not extract latitude and longtitude bounds from geolocation file"
            cat idl_output.txt
            exit 1
        fi
        
        oul=$(cat $out_txt_file | cut -d" " -f 1)
        olr=$(cat $out_txt_file | cut -d" " -f 2)
        
        rm -f $out_txt_file idl_output.txt runidl
    else
        # If a stero graphic projection is going to be used, get the corner coordinates 
        # of a polar projection which is centered at the polar. The width of the image 
        # will be 4096 km x 4096 km.
        region_str=$(echo $proj_type | cut -d"-" -f 2)
        out_txt_file=$(mktemp)
        
        /bin/bash $MIPS_HOME/scripts/auxiliary/get_polar_bounds.bash 4 4 $region_str $out_txt_file
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # Get the upper left and lower right coordinates of the output polar projection file.
        oul_lon=$(grep UL_LON $out_txt_file | cut -d"=" -f 2)
        oul_lat=$(grep UL_LAT $out_txt_file | cut -d"=" -f 2)
        oul="$oul_lon,$oul_lat"
        
        olr_lon=$(grep LR_LON $out_txt_file | cut -d"=" -f 2)
        olr_lat=$(grep LR_LAT $out_txt_file | cut -d"=" -f 2)
        olr="$olr_lon,$olr_lat"
    fi
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo $(basename $0)" parameters:"
        echo "    lat_bin:"$lat_binary
        echo "    lon_bin:"$lon_binary
        echo "    senz_bin:"$senz_binary
        echo "    ref_crefl_hres: "$ref_crefl_hres
        echo "    ref_crefl_skm: "$ref_crefl_skm
        echo "    band_r: "$band_r
        echo "    band_g: "$band_g
        echo "    band_b: "$band_b
        echo "    band_ls: "$band_ls
        echo "    band_hs: "$band_hs
        echo "    proj_type: "$proj_type
        echo "    fil_out: "$fil_out
        echo "    szen_out: "$szen_out
        echo "    oul: "$oul
        echo "    olr: "$olr
    fi
else
    # Argument number four is a metadata file, therefore a part of the L1B data will be 
    # projected onto a tile. The band combination that must be used to create the 
    # color image of the tile is extracted from the given metadata file. The corner 
    # coordinates are also extracted from the given metadata file.
    lat_binary=$1
    lon_binary=$2
    senz_binary=$3
    ref_crefl_hres=$4
    ref_crefl_skm=$5
    metadata_fil=$6
    fil_out=$7
    szen_out=$8
    
    if [ ! -e $metadata_fil ]; then
        echo "(ERROR)[E69] The metafile "$metadata_fil" does not exist."
        exit 1
    fi
    
    # Get the default band combination information
    band_comb_cfg=$(grep "BAND_COMB" $metadata_fil | cut -d"=" -f 2)
    band_r=$(echo $band_comb_cfg | cut -d"_" -f 1)
    band_g=$(echo $band_comb_cfg | cut -d"_" -f 2)
    band_b=$(echo $band_comb_cfg | cut -d"_" -f 3)
    band_ls=$(echo $band_comb_cfg | cut -d"_" -f 4)
    band_hs=$(echo $band_comb_cfg | cut -d"_" -f 5)
    
    # Get the upper left and lower right corner coordinates
    oul=$(grep "OUL" $metadata_fil | cut -d"=" -f 2)
    olr=$(grep "OLR" $metadata_fil | cut -d"=" -f 2)
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo $(basename $0)" parameters:"
        echo "    lat_bin:"$lat_binary
        echo "    lon_bin:"$lon_binary
        echo "    senz_bin:"$senz_binary
        echo "    ref_crefl_hres: "$ref_crefl_hres
        echo "    ref_crefl_skm: "$ref_crefl_skm
        echo "    metadata_fil: "$metadata_fil
        echo "    fil_out: "$fil_out
        echo "    szen_out: "$szen_out
    fi
fi

# ============================================================================
# CALCULATE GRID SIZE AND PARAMETERS
#
# Either a subpart of the L1B data will be projected on a tile, or the whole
# L1B image will be projected. The corner coordinates will be transformed into
# map coordinates. The width and the length of the projected image are
# calculated through the projected corner coordinates. This width and the 
# length are together called the grid size, and is calculated through the 
# script calc_grid_size.bash. 
# 
# The projected corner coordinates do not necesarily align over a projected 
# grid - the grid's starting refrence point might differ from the corner 
# coordinate's starting reference point. The script calc_grid_size.bash 
# re-adjust the projected corner coordinates in order for the corner coordinates 
# to fit onto the projection grid. 
# ============================================================================
# Get the projection type
polar_proj_bl="False"
if [ -z $metadata_fil ]; then
    # If no metadata file was provided, set the internal parameters based on the 
    # type of projection that was requested.
    case $proj_type in 
        "polar-north" )
            polar_proj_bl="True"
            proj_type_str="Polar-North"
        ;;

        "polar-south" )
            polar_proj_bl="True"
            proj_type_str="Polar-South"
        ;;

        "normal" )
            proj_type_str="Normal"
        ;;

        * )
            echo "(ERROR)[E70] The projection type is incorrect. It must be either 'polar-north', 'polar-south' or 'normal'."
            exit 1
        ;;
    esac
else
    # If a metadata file was given to the script, set the variables based on 
    # the metadata file.
    grep POLAR_REGION $metadata_fil &> /dev/null
    if [ $? -ne 0 ]; then
        # Normal projection
        proj_type_str="Normal"
    else
        polar_proj_bl="True"
        # Polar projection
        # Find the polar region projection 
        polar_reg_proj=$(grep POLAR_REGION $metadata_fil | cut -d"=" -f 2 | awk '{print $1}')
        proj_type_str="Polar-"$polar_reg_proj
    fi
fi

grid_info_txt=grid_info.txt
$MIPS_HOME/scripts/auxiliary/calc_grid_size.bash "$oul" "$olr" "$proj_type_str" $MIPS_PC_PIXEL_RESOL $grid_info_txt
if [ $? -ne 0 ]; then
    exit 1
fi

# ============================================================================
# SET AND GET THE PROJECTION INFORMATION
# ============================================================================
case $proj_type_str in 
    "Polar-North" )
        # Set the refrence latitude coordinate
        polar_ref_lat_coord="90.0"
        # Set the second refrence latitude coordinate
        polar_ref_sec_coord="90.0"
    ;;
    
    "Polar-South" )
        # Set the refrence latitude coordinate
        polar_ref_lat_coord="-90.0"
        # Set the second refrence latitude coordinate
        polar_ref_sec_coord="-90.0"
    ;;
esac

# Get the adjusted corner coordinates (latitude and longtitude), for the 
# upper left and lower right corners.
adj_oul_lon=$(grep UL_LON $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
adj_oul_lat=$(grep UL_LAT $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
adj_olr_lon=$(grep LR_LON $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
adj_olr_lat=$(grep LR_LAT $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
# Get the number of columns and rows of the projection grid.
nr_columns=$(grep COLS $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
nr_rows=$(grep ROWS $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
# Get the adjusted projected corner coordinates.
oul_x=$(grep ADJ_UL_X $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
oul_y=$(grep ADJ_UL_Y $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
olr_x=$(grep ADJ_LR_X $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
olr_y=$(grep ADJ_LR_Y $grid_info_txt | cut -d"=" -f 2 | awk '{print $1}')
oul_xy="$oul_x,$oul_y"
olr_xy="$olr_x,$olr_y"

# ============================================================================
# CREATE MS2GT CONFIG FILE
# ============================================================================
# Copy the template config files (gpd, mpp) to current directory and fill in the values
cat $MIPS_HOME/templates/normal_ms2gt_templ.gpd > template.gpd
cat $MIPS_HOME/templates/normal_ms2gt_templ.mpp > template.mpp
if [ ! -e template.gpd ] || [ ! -e template.mpp ]; then
    echo "(ERROR)[E33] Copies of the MS2GT template files could not be made"
    exit 1
fi

# Set the grid parameter configuration file
sed -i "s/GRID_WIDTH/$nr_columns/g" template.gpd
sed -i "s/GRID_HEIGHT/$nr_rows/g" template.gpd

# Set the resolution of the pixels in meters.
sed -i "s/RES/$MIPS_PC_PIXEL_RESOL/g"  template.gpd
sed -i "s/MMP_FILE_NAME/template.mpp/g" template.gpd

# Set the map parameter configuration file
if [ $polar_proj_bl == "False" ]; then
    sed -i "s/PROJ_NAME/CYLINDRICAL EQUIDISTANT/g" template.mpp
    sed -i "s/MAP_REF_LAT/0.0/g" template.mpp
    sed -i "s/MAP_REF_LON/0.0/g" template.mpp
    # Parameters for ms2gt0.5
    #sed -i "s/MAP_EQUA_RAD/6378137.0/g" template.mpp
    #sed -i "s/MAP_POLAR_RAD/6356752.314245/g" template.mpp
    # Parameters for ms2gt0.22
    sed -i "s/MAP_EQUA_RAD/6371007.181/g" template.mpp
    sed -i "s/MAP_POLAR_RAD/6371007.181/g" template.mpp
else
    sed -i "s/PROJ_NAME/POLAR STEREOGRAPHIC/g" template.mpp
    sed -i "s/MAP_REF_LAT/$polar_ref_lat_coord/g" template.mpp
    sed -i "s/MAP_REF_LON/0.0/g" template.mpp
    sed -i "s/MAP_EQUA_RAD/6371007.181/g" template.mpp
    sed -i "s/MAP_POLAR_RAD/6371007.181/g" template.mpp

    echo "Map Second Reference Latitude: $polar_ref_sec_coord" >> template.mpp
fi
sed -i "s/MAP_ORG_LAT/$adj_oul_lat/g" template.mpp
sed -i "s/MAP_ORG_LON/$adj_oul_lon/g" template.mpp

# ============================================================================
# DETERMINE IF BAND SHARPENING MUST BE USED
# 
# If all the bands are not of the same resolution and one of the bands are 
# of a higher resolution, that band can be used to increase the resolution of 
# the other bands. This is called the sharpening band. The sharpening band is 
# explicitly defined. If all the bands are of the same resolution, band 
# sharping will not be preformed. The script band_comb_type.bash determines 
# if band sharpening must be done. 
# ============================================================================

# If the band numbers are single digit numbers, convert the numbers into double 
# digit values.
band_r_str_len=${#band_r}
if [ $band_r_str_len -eq 1 ]; then
    band_r="0$band_r"
fi

band_g_str_len=${#band_g}
if [ $band_g_str_len -eq 1 ]; then
    band_g="0$band_g"
fi

band_b_str_len=${#band_b}
if [ $band_b_str_len -eq 1 ]; then
    band_b="0$band_b"
fi

band_ls_str_len=${#band_ls}
if [ $band_ls_str_len -eq 1 ]; then
    band_ls="0$band_ls"
fi

band_hs_str_len=${#band_hs}
if [ $band_hs_str_len -eq 1 ]; then
    band_hs="0$band_hs"
fi

band_comb_type_fil="band_comb_type_file.txt"
/bin/bash $MIPS_HOME/scripts/auxiliary/band_comb_type.bash $band_r $band_g $band_b $band_ls $band_hs $band_comb_type_fil
if [ $? -ne 0 ]; then
    echo "(ERROR)[E71] The BASH script band_comb_type.bash failed"
    exit 1
fi
band_comb_type=$(cat $band_comb_type_fil)

# ============================================================================
# EXECUTE MS2GT WRAPPER SCRIPTS
# 
# The necessary configuration files were constructed for the MS2GT program. The 
# scripts reproject_ms2gt_eqres.bash and reproject_ms2gt_mix.bash projects 
# the L1B data accroding to the specifications set in this script.
# ============================================================================

tmp_dir=$(pwd)
tmp_fil_out=$(mktemp -p $tmp_dir)
tmp_szen_out=$(mktemp -p $tmp_dir)
case $band_comb_type in 
    "HRES" )
        # All the bands are of the same resolution.
        /bin/bash $MIPS_HOME/scripts/auxiliary/reproject_ms2gt_eqres.bash template.gpd $lat_binary $lon_binary $senz_binary $ref_crefl_hres \
            $band_r $band_g $band_b $tmp_fil_out $tmp_szen_out
    ;;

    "MRES" | "LRES" )
        # All the bands are of the same resolution.
        /bin/bash $MIPS_HOME/scripts/auxiliary/reproject_ms2gt_eqres.bash template.gpd $lat_binary $lon_binary $senz_binary $ref_crefl_skm \
            $band_r $band_g $band_b $tmp_fil_out $tmp_szen_out
    ;;
    
    "MIX" )
        #if [ "$band_s" == "" ]; then
        #    echo "(ERROR)[E31] Please specify the band number that must be used in the band sharpening process."
        #    exit 1
        #fi
        
        #char_val=$(echo $band_s | cut -c1)
        #if [ "$char_val" != "0" ]; then
        #    band_s="0"$band_s
        #fi
        
        # All the bands are not of the same resolution.
        /bin/bash $MIPS_HOME/scripts/auxiliary/reproject_ms2gt_mix.bash template.gpd $lat_binary $lon_binary $senz_binary $ref_crefl_hres \
            $ref_crefl_skm $band_ls $band_hs $band_r $band_g $band_b $tmp_fil_out $tmp_szen_out
    ;;

    * )
        echo "(ERROR)[E71] The BASH script band_comb_type.bash failed"
        exit 1
    ;;
esac

return_value=$?

# BEGIN - For ms2gt0.22 version - BEGIN
# Check if no tile was produced.
if [ $return_value -eq 2 ]; then
    rm -f *.proj *.pro $grid_info_txt template.gpd template.mpp 
    rm -f $out_txt_file $tmp_fil_out $tmp_szen_out
    rm -f runidl flsave.pro idlsave.pro $band_comb_type_fil
    
    exit 2
fi
# END - For ms2gt0.22 version - END

# If an error occured, exit immediately.
if [ $return_value -ne 0 ]; then
    exit 1
fi

# ============================================================================
# CREATE PROJECTION TEXT FILE
# 
# The projection text file is create in junction with the projected image file. 
# The text file has the following information:
# UL_XY=Upper left projected longitude, Upper left projected latitude.
# LR_XY=Lower right projected longitude, Lower right projected latitude.
# PIXELS_SIZE=The pixel size in meters, the width and length are equal.
# PROJ_NAME=This will be either equal to EQRECT, NPOLARST or SPOLARST.
# ROWS=The number of rows in the output projected image.
# COLS=The number of columns in the output projected image.
# ============================================================================

# Create projection info file
proj_file=$(dirname $fil_out)/$(basename $fil_out ".png")".proj"
echo "UL_XY="$oul_xy > $proj_file
echo "LR_XY="$olr_xy >> $proj_file
echo "PIXEL_SIZE=$MIPS_PC_PIXEL_RESOL" >> $proj_file
case $proj_type_str in 
    "Normal" )
        echo "PROJ_NAME=EQRECT" >> $proj_file
    ;;
    
    "Polar-North" )
        echo "PROJ_NAME=NPOLARST" >> $proj_file
    ;;
    
    "Polar-South" )
        echo "PROJ_NAME=SPOLARST" >> $proj_file 
    ;;
esac
echo "ROWS="$nr_rows >> $proj_file
echo "COLS="$nr_columns >> $proj_file

# ============================================================================
# COPY SENSOR ZENITH ANGLE PROJECTED FILE
# ============================================================================
# NOTE: The subset script and the polar subset script must depend on the image
# file output, and NOT on the sensor zenith angle binary output. 
# Copy the sensor zenith binary file.
# If the sensor zenith binary file already exist, do not copy it.
if [ ! -e $szen_out ]; then
    cp $tmp_szen_out $szen_out
fi

# ============================================================================
# CREATE A GEOTIFF IMAGE OF THE TILE
# ============================================================================
if [ "$MIPS_TILE_GEOTIFF_BL" == "YES" ]; then
    # We know that the given output image is a PNG image. Create the output file 
    # name based on the PNG image file name.
    file_out_basename=$(basename $fil_out ".png")
    file_out_dirname=$(dirname $fil_out)
    geotif_file_name=$file_out_basename".tif"
    
    geotif_file_out=$file_out_dirname"/"$geotif_file_name
    tmp_geotif_file_out="tmp_"$geotif_file_name
    
    geotiff_templ_file="geotiff_template.txt"
    if [ "$polar_proj_bl" == "True" ]; then
        source_geotiff_templ_file=$MIPS_HOME/templates/geotiff_metadata_polar.txt
    else
        source_geotiff_templ_file=$MIPS_HOME/templates/geotiff_metadata.txt
    fi
    
    cat $source_geotiff_templ_file > $geotiff_templ_file
    if [ ! -e "$geotiff_templ_file" ]; then
        echo "(ERROR)[E90] The "$source_geotiff_templ_file" could not be dublicated."
        exit 1
    fi
    
    # Set the value in the geotiff template file
    sed -i "s/@UL_X@/$oul_x/g" $geotiff_templ_file
    sed -i "s/@UL_Y@/$oul_y/g" $geotiff_templ_file
    sed -i "s/@RESOL@/$MIPS_PC_PIXEL_RESOL/g" $geotiff_templ_file
    if [ "$polar_proj_bl" == "True" ]; then
        sed -i "s/@POLAR_LAT_VAL@/$polar_ref_lat_coord/g" $geotiff_templ_file
    fi
    
    # Create a TIFF version of the image
    convert -compress None $tmp_fil_out $tmp_geotif_file_out
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E91] The command 'convert' failed to convert the PNG image "$(basename $tmp_fil_out)" into a TIFF image"
        exit 1
    fi
    
    # Create a copy of the geotiff image
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG] Executing the following:"
        echo "    "$MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_templ_file $tmp_geotif_file_out $geotif_file_out
    fi

    $MIPS_HOME/bin/geotifcp -c lzw -g $geotiff_templ_file $tmp_geotif_file_out $geotif_file_out
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E92] The binary geotifcp failed to create the GeoTIFF file "$(basename $geotif_file_out)
        exit 1
    fi
    
    echo "(Created GeoTIFF image "$geotif_file_name")"
    echo "[DEBUG] Date: "$(date)" Process number: "$$

    rm -f $tmp_geotif_file_out $geotiff_templ_file
fi

# ============================================================================
# COPY THE OUTPUT IMAGE
# 
# The output projected image can be large, therefore it can take a while for
# the file to be copied. A script which might wait for a tile image, might try
# to use the file before the file has been fully copied. To prevent such a
# possible conflict, the script copies the image under a pseudo file name under
# the same directory where the file will reside. The pseudo file is then
# renamed at the destination directory to the output file name.
# ============================================================================
# Copy the image file (Atomically as possible)
cp $tmp_fil_out $fil_out".tmp"
mv $fil_out".tmp" $fil_out
rm $tmp_fil_out

if [ ! -e $fil_out ]; then
    echo "(ERROR)[E32] Failed to create a copy of tile image "$(basename $fil_out)
    exit 1
fi

echo "(Tile image file "$fil_out" has been created)"
echo "[DEBUG] Date: "$(date)" Process number: "$$

# ==========================================================
# DELETE TEMPORARY FILES
# ==========================================================
if [ "$MIPS_DEBUG" != "YES" ]; then
    # Don't remove *.dat files
    rm -f *.proj *.pro $grid_info_txt template.gpd template.mpp 
    rm -f $out_txt_file $tmp_fil_out $tmp_szen_out
    rm -f runidl flsave.pro idlsave.pro $band_comb_type_fil
fi

# Remove files which were created by ms2gt
rm -f *.img 

