#!/bin/bash

# ============================================================================
# MIPS MODIS granule processor BASH script
# 
# This script is the main script through which MODIS L1B data are processed. 
# The script creates tile images and polar tile images from the given L1B 
# data.
# 
# There are several steps in this script that are followed to create image tiles 
# and polar image tiles. The steps are:
# 
# 1. Do land surface reflection correction on the L1B to improve image quality
# 2. Create an unproject image of the L1B 1KM file (a granule image)
# 3. Generate normal and polar metadata files, which are prerequisites for 
#    producing normal and polar image tiles
# 4. Produce normal image tiles
# 5. Produce polar image tiles
# 6. Produce composite image tiles for both normal and polar tiles, using available 
#    data
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi
# The MIPS_AUX environmental variable is used throughout this scripts. This is 
# used rather than refering to the full path of the auxiliary scripts directory.
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Check number of arguments
if [ $# -ne 7 ]; then
    echo "(ERROR) Usage:"
    echo "    "$(basename $0)" file_1km file_hkm file_qkm file_geo satname yeardate utctime"
    echo "where"
    echo "file_1km = MODIS L1B 1000 meter HDF file"
    echo "file_hkm = MODIS L1B 500 meter HDF file"
    echo "file_qkm = MODIS L1B 250 meter HDF file"
    echo "file_geo = MODIS L1B geolocation HDF file"
    echo "satname = Satellite name (terra or aqua)"
    echo "yeardate = Date in format yyyymmdd where yyyy is year, mm is month and dd is day of month (e.g., 20081218)"
    echo "utctime = UTC time in format hhmm at start of granule (e.g. 1835)"
    
    exit 1
fi

# Get arguments
file_1km=$1
file_hkm=$2
file_qkm=$3
file_geo=$4
satname=$5
yeardate=$6
utctime=$7

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

#-------------------------------------------------------------------------------
# CHECK IF THERE ARE ANY PROCESSING REQUESTS
# 
# The processings steps listed in the top part of this script, can be controlled 
# by the environmental variable MIPS_PROC_STEPS. If the variable is present, the 
# processings steps defined in the variable will be executed, and the other steps 
# will be ignored. If the variable is not present, all the processing steps will 
# be executed.
# 
# The names, used in the variable, for each processing step are as follows (refer 
# to the document list in the top part of this file for a description of each 
# step):
# 1. crefl
# 2. gran_img
# 3. metadata
# 4. polarmetadata
# 5. tiles_img
# 6. polartiles_img
# 7. comps_img
# 
# Each processing step defined MUST be delimited by a ':' character. 
#-------------------------------------------------------------------------------
if [ -z "$MIPS_PROC_STEPS" ]; then
    export MIPS_PROC_STEPS=":crefl:gran_img:metadata:polarmetadata:tiles_img:polartiles_img:comps_img:"
fi

#-------------------------------------------------------------------------------
# CHECK VADILITY OF INPUT HDF5 L1B FILES
# 
# Before the processing steps are executed, the script first analyse the files to 
# see if the files are 
# 1. readable, 
# 2. are not internally corrupt, 
# 3. have valid parameters, and 
# 4. have enough day mode scans (minimum five). 
#-------------------------------------------------------------------------------
# Check if arguments exist
if [ ! -r $file_1km ]; then echo "(ERROR)[E0] File cannot be read: "$file_1km; exit 1; fi
if [ ! -r $file_hkm ]; then echo "(ERROR)[E0] File cannot be read: "$file_hkm; exit 1; fi
if [ ! -r $file_qkm ]; then echo "(ERROR)[E0] File cannot be read: "$file_qkm; exit 1; fi
if [ ! -r $file_geo ]; then echo "(ERROR)[E0] File cannot be read: "$file_geo; exit 1; fi

# Check if HDF files are valid
result=$($MIPS_HOME/bin/ncdump -h $file_1km | grep "short EV_1KM_Emissive")
if [ $? -ne 0 ]; then echo "(ERROR)[E9] File is not valid MODIS L1B 1000 meter HDF: "$file_1km; exit 1; fi
result=$($MIPS_HOME/bin/ncdump -h $file_hkm | grep "short EV_500_RefSB")
if [ $? -ne 0 ]; then echo "(ERROR)[E9] File is not valid MODIS L1B 500 meter HDF: "$file_hkm; exit 1; fi
result=$($MIPS_HOME/bin/ncdump -h $file_qkm | grep "short EV_250_RefSB")
if [ $? -ne 0 ]; then echo "(ERROR)[E9] File is not valid MODIS L1B 250 meter HDF: "$file_qkm; exit 1; fi
result=$($MIPS_HOME/bin/ncdump -h $file_geo | grep "byte Land/SeaMask")
if [ $? -ne 0 ]; then echo "(ERROR)[E9] File is not valid MODIS L1B geolocation HDF: "$file_geo; exit 1; fi

# Check if satellite name, date, time are valid
if [ $satname != "terra" ] && [ $satname != "aqua" ]; then echo "(ERROR)[E10] Satellite name is not valid: "$satname; exit 1; fi
if [ $yeardate -lt 20000101 ] || [ $yeardate -gt 21000101 ]; then echo "(ERROR)[E10] Date is not valid: "$yeardate; exit 1; fi
if [ $utctime -lt 0000 ] || [ $utctime -gt 2359 ]; then echo "(ERROR)[E10] UTC time is not valid: "$utctime; exit 1; fi

# Check if this is a daytime granule
echo
echo "(Checking for daytime granule)"
nscans=$($MIPS_HOME/bin/ncdump -h $file_1km | grep "Number of Day mode scans" | awk '{print $7}')
echo "Number of day mode scans = "$nscans
if [ $nscans -lt 5 ]; then
    echo "(WARNING)[W0] Granule has less than 5 day mode scans: processing will exit"
    exit 0
fi

#-------------------------------------------------------------------------------
# DO CORRECTED REFLECTANCE PROCESSING - CREFL
#-------------------------------------------------------------------------------
# Create the new file names for the corrected reflectance files
ref_1km=$satname."1km".$yeardate.$utctime.crefl.hdf
ref_hkm=$satname."hkm".$yeardate.$utctime.crefl.hdf
ref_qkm=$satname."qkm".$yeardate.$utctime.crefl.hdf

echo $MIPS_PROC_STEPS | grep ":crefl:" &> /dev/null
if [ $? -eq 0 ]; then
    echo 
    echo "(Creating CREFL data)"
    
    /bin/bash $MIPS_AUX/process_granule_crefl.bash modis $file_1km $file_hkm $file_qkm \
                                                         $ref_1km $ref_hkm $ref_qkm
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# CREATE GRANULE IMAGE
#-------------------------------------------------------------------------------
# Create the granule image output directory if it does not exist
mkdir -p $MIPS_GRAN_IMG_DIR

echo $MIPS_PROC_STEPS | grep ":gran_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo 
    echo "(Creating CREFL image)"
    
    # Create a granule image from the corrected reflectance 1KM data, using the 
    # true color band combination.
    /bin/bash $MIPS_AUX/process_granule_img.bash $satname $yeardate $utctime "01_04_03" $ref_1km "1km"
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# EXTRACT LATITUDE, LONGITUDE AND SOLARZENITH ANGLE DATA FROM GEOLOCATION FILE
#-------------------------------------------------------------------------------
/bin/bash $MIPS_AUX/hdf4_modis_geodata_to_bin.bash $file_geo "lat_km" "lon_km" "sensorzenithkm"
# Get the file names for the latitude, longitude and sensor zenith data
lat_km_bin_file=$(ls "lat_km"*".dat")
if [ ! -e $lat_km_file_bin ]; then
    echo "(ERROR)[Exx] The latitude binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

lon_km_bin_file=$(ls "lon_km"*".dat")
if [ ! -e $lon_km_file_bin ]; then
    echo "(ERROR)[Exx] The longitude binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

senz_bin_file=$(ls "sensorzenithkm"*".dat")
if [ ! -e $lon_km_file_bin ]; then
    echo "(ERROR)[Exx] The sensor zenith angle binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

#-------------------------------------------------------------------------------
# TILE METADATA PROCESSING
#-------------------------------------------------------------------------------
# Create the tile output directory if it does not exist.
mkdir -p $MIPS_TILE_DIR

echo $MIPS_PROC_STEPS | grep ":metadata:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Finding tiles covered by this granule)"
    
    # For each metadata file that is created, an event file is created which is used in 
    # the processing steps here after. The event files are used to keep track of what 
    # image tiles must be created.
    /bin/bash $MIPS_AUX/run_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "MODIS" $satname $yeardate $utctime $MIPS_TILE_DIR
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# POLAR TILE METADATA PROCESSING
#-------------------------------------------------------------------------------
# Create the polar tile output directories if these do not exist. There must be 
# both a 'north' and 'south' sub-directories.
mkdir -p $MIPS_POLR_TILE_DIR/north $MIPS_POLR_TILE_DIR/south

echo $MIPS_PROC_STEPS | grep ":polarmetadata:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Finding Northen polar tiles covered by this granule)"
    
    # For each polar metadata file that is created, a polar event file is created which 
    # is used to keep track of what polar image tiles should be created.
    /bin/bash $MIPS_AUX/run_polar_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "MODIS" $satname $yeardate $utctime \
        "North" $MIPS_POLR_TILE_DIR/north
    
    echo
    echo "(Finding Southern polar tiles covered by this granule)"
    
    # For each polar metadata file that is created, a polar event file is created which 
    # is used to keep track of what polar image tiles should be created.
    /bin/bash $MIPS_AUX/run_polar_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "MODIS" $satname $yeardate $utctime \
        "South" $MIPS_POLR_TILE_DIR/south
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# CREATE IMAGE TILES
#-------------------------------------------------------------------------------
echo $MIPS_PROC_STEPS | grep ":tiles_img:" &> /dev/null
if [ $? -eq 0 ]; then
    # First remove tile event files which are older than two days. This is to prevent 
    # possible stale tile event files which could disrupt the processing
    metadata_info=$satname.$yeardate.$utctime
    
    #if [ "$MIPS_DEBUG" != "YES" ]; then
    #    find $MIPS_TILE_DIR -name "tile*.$metadata_info.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
    #fi
    
    echo 
    echo "(Creating tiles)"
	
    # For each event file created by the script run_metadata.bash, the following script is 
    # executed to create the image tiles.
    for event_file in $(find $MIPS_TILE_DIR -name "tile*.$metadata_info.*.event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_tiles.bash $event_file $ref_hkm $ref_qkm $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a tile image has been successfully created based on the tile event file, 
        # remove the tile event file.
        rm $event_file
    done
fi

#-------------------------------------------------------------------------------
# CREATE POLAR IMAGE TILES
#-------------------------------------------------------------------------------
echo $MIPS_PROC_STEPS | grep ":polartiles_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Creating Northen polar tiles)"
    
    # For each event file created by the script run_polar_metadata.bash, the following script is 
    # executed to create the polar image tiles.
    for event_file in $(find $MIPS_POLR_TILE_DIR/north -name "ptile*."$satname"."$yeardate"."$utctime"."*".event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_polar_tiles.bash $event_file $ref_hkm $ref_qkm $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a polar tile image has been successfully created based on the polar tile event file, 
        # remove the tile event file.
        rm $event_file
    done
    
    echo
    echo "(Creating Southern polar tiles)"

    # For each event file created by the script run_polar_metadata.bash, the following script is 
    # executed to create the polar image tiles.
    for event_file in $(find $MIPS_POLR_TILE_DIR/south -name "ptile*."$satname"."$yeardate"."$utctime"."*".event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_polar_tiles.bash $event_file $ref_hkm $ref_qkm $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a polar tile image has been successfully created based on the polar tile event file, 
        # remove the tile event file.
        rm $event_file
    done
fi

#-------------------------------------------------------------------------------
# CLEANUP
#-------------------------------------------------------------------------------
# Remove excess files created by the scripts that had been executed. Do not remove 
# the files when the script is in debugging mode.
if [ "$MIPS_DEBUG" != "YES" ]; then
    /bin/rm -f idlsave.pro flsave.pro runidl geotiff_metadata.txt idl_output.txt
    # Remove *.dat files that were created by the reproject_ms2gt_*.bash scripts. 
    /bin/rm -f *.dat $ref_1km $ref_hkm $ref_qkm
fi

echo 
echo "(Ended: "$(date)")"

