#!/bin/bash

# ============================================================================
# MIPS VIIRS granule processor BASH script
# 
# TODO: Rewrite!!!!!!
# This script is the main script through which MODIS L1B data are processed. 
# The script creates tile images and polar tile images from the given L1B 
# data.
# 
# There are several steps in this script that are followed to create image tiles 
# and polar image tiles. The steps are:
# 
# 1. Do land surface reflection correction on the L1B to improve image quality
# 2. Create an unproject image of the L1B 1KM file (a granule image)
# 3. Generate normal and polar metadata files, which are prerequisites for 
#    producing normal and polar image tiles
# 4. Produce normal image tiles
# 5. Produce polar image tiles
# 6. Produce composite image tiles for both normal and polar tiles, using available 
#    data
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi
# The MIPS_AUX environmental variable is used throughout this scripts. This is 
# used rather than refering to the full path of the auxiliary scripts directory.
MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Check number of arguments
if [ $# -lt 7 ]; then
    echo "(ERROR) Usage:"
    echo "    "$(basename $0)" <SVM and/or SVI files> <GMTCO or GMODO file> satname yeardate utctime"
    echo "where"
    echo "TODO"
    echo "satname = Satellite name (npp)"
    echo "yeardate = Date in format yyyymmdd where yyyy is year, mm is month and dd is day of month (e.g., 20081218)"
    echo "utctime = UTC time in format hhmm at start of granule (e.g. 1835)"
    
    exit 1
fi

# Get arguments. The last four arguments are the geolocation file, satellite name, yeardate and utc time
nr_args=$#
nr_svi_svm_args=$(echo "$nr_args - 4" | bc)
file_svm_svi_lst=""
seq_val_arr=$(eval echo {1..$nr_svi_svm_args})
for seq_val in $seq_val_arr; do
    file_svm_svi_lst="$1 "$file_svm_svi_lst  
    shift 1
done
file_geo=$1
satname=$2
yeardate=$3
utctime=$4

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

#if [ "$MIPS_DEBUG" == "YES" ]; then
#    echo "file_svm_svi_lst: "$file_svm_svi_lst
#    echo "file_geo: "$file_geo
#    echo "satname: "$satname
#    echo "yeardate: "$yeardate
#    echo "utctime: "$utctime
#fi

#-------------------------------------------------------------------------------
# CHECK IF THERE ARE ANY PROCESSING REQUESTS
# 
# The processings steps listed in the top part of this script, can be controlled 
# by the environmental variable MIPS_PROC_STEPS. If the variable is present, the 
# processings steps defined in the variable will be executed, and the other steps 
# will be ignored. If the variable is not present, all the processing steps will 
# be executed.
# 
# The names, used in the variable, for each processing step are as follows (refer 
# to the document list in the top part of this file for a description of each 
# step):
# 1. crefl
# 2. gran_img
# 3. metadata
# 4. polarmetadata
# 5. tiles_img
# 6. polartiles_img
# 7. comps_img
# 
# Each processing step defined MUST be delimited by a ':' character. 
#-------------------------------------------------------------------------------
if [ -z "$MIPS_PROC_STEPS" ]; then
    export MIPS_PROC_STEPS=":crefl:gran_img:metadata:polarmetadata:tiles_img:polartiles_img:comps_img:"
fi

#-------------------------------------------------------------------------------
# CHECK VADILITY OF INPUT HDF5 L1B FILES
# 
# Before the processing steps are executed, the script first analyse the files to 
# see if the files are 
# 1. readable, 
# 2. are not internally corrupt, 
# 3. have valid parameters, and 
# 4. have enough day mode scans (minimum five). 
#-------------------------------------------------------------------------------
# Check if all the given files to exists, and if the files are valid
last_sv_code_str=""
last_sv_file_path=""
for file_str in $file_svm_svi_lst; do
    if [ ! -r $file_str  ]; then 
        echo "(ERROR)[Exx] File cannot be read: "$file_str
        exit 1 
    fi
    
    result=$($MIPS_HOME/bin/h5dump -H $file_str | grep "Reflectance")
    if [ $? -ne 0 ]; then
        echo "(ERROR)[Exx] File is not valid VIIRS L1B HDF: "$file_str
        exit 1
    fi
    
    # Keep track of the last SVM/SVI number in the file list. This will be used to check if 
    # data is day time or night time data.
    last_sv_letter_str=$(basename $file_str | cut -c3)
    last_sv_number_str=$(basename $file_str | cut -c4-5 | bc)
    last_sv_code_str=$last_sv_letter_str$last_sv_number_str
    last_sv_file_path=$file_str
done

if [ ! -r $file_geo ]; then
    echo "(ERROR)[Exx] File cannot be read: "$file_str
    exit 1 
fi

# Check if this is a daytime granule
echo
echo "(Checking for daytime granule)"

last_sv_code_str=$(echo $last_sv_code_str | tr "[:lower:]" "[:upper:]")
hdf5_attr_path_str="/Data_Products/VIIRS-"$last_sv_code_str"-SDR/VIIRS-"$last_sv_code_str"-SDR_Gran_0/N_Day_Night_Flag"
day_night_flag=$($MIPS_HOME/bin/h5dump -a $hdf5_attr_path_str $last_sv_file_path | grep "(0,0):" | awk '{print $2}' | sed 's/[\"]//g')
if [ "$day_night_flag" == "Night" ]; then
    echo "(WARNING)[W0] Granule is a night time granule. Data will not be processed."
    exit 0
fi

# Check if satellite name, date, time are valid
if [ $satname != "npp" ]; then echo "(ERROR)[E10] Satellite name is not valid: "$satname; exit 1; fi
if [ $yeardate -lt 20000101 ] || [ $yeardate -gt 21000101 ]; then echo "(ERROR)[E10] Date is not valid: "$yeardate; exit 1; fi
if [ $utctime -lt 0000 ] || [ $utctime -gt 2359 ]; then echo "(ERROR)[E10] UTC time is not valid: "$utctime; exit 1; fi

#-------------------------------------------------------------------------------
# DO CORRECTED REFLECTANCE PROCESSING - CREFL
#-------------------------------------------------------------------------------
# Create the new file names for the corrected reflectance files
ref_lres=$satname."lres".$yeardate.$utctime.crefl.hdf
ref_hres=$satname."hres".$yeardate.$utctime.crefl.hdf

echo $MIPS_PROC_STEPS | grep ":crefl:" &> /dev/null
if [ $? -eq 0 ]; then
    echo 
    echo "(Creating CREFL data)"
    
    /bin/bash $MIPS_AUX/process_granule_crefl.bash viirs $file_geo $file_svm_svi_lst $ref_lres $ref_hres
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# CREATE GRANULE IMAGE
#-------------------------------------------------------------------------------
# Create the granule image output directory if it does not exist
mkdir -p $MIPS_GRAN_IMG_DIR

echo $MIPS_PROC_STEPS | grep ":gran_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo 
    echo "(Creating CREFL image)"
    
    # Create a granule image from the corrected reflectance HKM data, using the 
    # true color band combination.
    /bin/bash $MIPS_AUX/process_granule_img.bash $satname $yeardate $utctime "05_04_03" $ref_lres "750m"
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# EXTRACT LATITUDE, LONGITUDE AND SOLARZENITH ANGLE DATA FROM GEOLOCATION FILE
#-------------------------------------------------------------------------------
/bin/bash $MIPS_AUX/hdf5_viirs_geodata_to_bin.bash $file_geo "lat_km" "lon_km" "sensorzenithkm"
# Get the file names for the latitude, longitude and sensor zenith data
lat_km_bin_file=$(ls "lat_km"*".dat")
if [ ! -e $lat_km_file_bin ]; then
    echo "(ERROR)[Exx] The latitude binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

lon_km_bin_file=$(ls "lon_km"*".dat")
if [ ! -e $lon_km_file_bin ]; then
    echo "(ERROR)[Exx] The longitude binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

senz_bin_file=$(ls "sensorzenithkm"*".dat")
if [ ! -e $lon_km_file_bin ]; then
    echo "(ERROR)[Exx] The sensor zenith angle binary file of the geolocation file "$(basename $file_geo)" does not exist."
    exit 1
fi

#-------------------------------------------------------------------------------
# TILE METADATA PROCESSING
#-------------------------------------------------------------------------------
# Create the tile output directory if it does not exist.
mkdir -p $MIPS_TILE_DIR

echo $MIPS_PROC_STEPS | grep ":metadata:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Finding tiles covered by this granule)"
    
    # For each metadata file that is created, an event file is created which is used in 
    # the processing steps here after. The event files are used to keep track of what 
    # image tiles must be created.
    /bin/bash $MIPS_AUX/run_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "VIIRS" $satname $yeardate $utctime $MIPS_TILE_DIR
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# POLAR TILE METADATA PROCESSING
#-------------------------------------------------------------------------------
# Create the polar tile output directories if these do not exist. There must be 
# both a 'north' and 'south' sub-directories.
mkdir -p $MIPS_POLR_TILE_DIR/north $MIPS_POLR_TILE_DIR/south

echo $MIPS_PROC_STEPS | grep ":polarmetadata:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Finding Northen polar tiles covered by this granule)"
    
    # For each polar metadata file that is created, a polar event file is created which 
    # is used to keep track of what polar image tiles should be created.
    /bin/bash $MIPS_AUX/run_polar_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "VIIRS" $satname $yeardate $utctime \
        "North" $MIPS_POLR_TILE_DIR/north
    
    echo
    echo "(Finding Southern polar tiles covered by this granule)"
    
    # For each polar metadata file that is created, a polar event file is created which 
    # is used to keep track of what polar image tiles should be created.
    /bin/bash $MIPS_AUX/run_polar_metadata.bash $file_geo $lat_km_bin_file $lon_km_bin_file "VIIRS" $satname $yeardate $utctime \
        "South" $MIPS_POLR_TILE_DIR/south
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
fi

#-------------------------------------------------------------------------------
# CREATE IMAGE TILES
#-------------------------------------------------------------------------------
echo $MIPS_PROC_STEPS | grep ":tiles_img:" &> /dev/null
if [ $? -eq 0 ]; then
    # First remove tile event files which are older than two days. This is to prevent 
    # possible stale tile event files which could disrupt the processing
    metadata_info=$satname.$yeardate.$utctime
    
    if [ "$MIPS_DEBUG" != "YES" ]; then
        find $MIPS_TILE_DIR -name "tile*.$metadata_info.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
    fi
    
    echo 
    echo "(Creating tiles)"

    # For each event file created by the script run_metadata.bash, the following script is 
    # executed to create the image tiles.
    for event_file in $(find $MIPS_TILE_DIR -name "tile*.$metadata_info.*.event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_tiles.bash $event_file $ref_lres $ref_hres $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a tile image has been successfully created based on the tile event file, 
        # remove the tile event file.
        rm $event_file
    done
fi

#-------------------------------------------------------------------------------
# CREATE POLAR IMAGE TILES
#-------------------------------------------------------------------------------
echo $MIPS_PROC_STEPS | grep ":polartiles_img:" &> /dev/null
if [ $? -eq 0 ]; then
    echo
    echo "(Creating Northen polar tiles)"
    
    # For each event file created by the script run_polar_metadata.bash, the following script is 
    # executed to create the polar image tiles.
    for event_file in $(find $MIPS_POLR_TILE_DIR/north -name "ptile*."$satname"."$yeardate"."$utctime"."*".event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_polar_tiles.bash $event_file $ref_lres $ref_hres $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a polar tile image has been successfully created based on the polar tile event file, 
        # remove the tile event file.
        rm $event_file
    done
    
    echo
    echo "(Creating Southern polar tiles)"

    # For each event file created by the script run_polar_metadata.bash, the following script is 
    # executed to create the polar image tiles.
    for event_file in $(find $MIPS_POLR_TILE_DIR/south -name "ptile*."$satname"."$yeardate"."$utctime"."*".event" -type f -print); do
        # TODO: Pass binary latitude, longitude and solar zenith angle data
        /bin/bash $MIPS_AUX/process_granule_polar_tiles.bash $event_file $ref_lres $ref_hres $lat_km_bin_file $lon_km_bin_file $senz_bin_file
        
        # If an error occured, exit immediately.
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        # If a polar tile image has been successfully created based on the polar tile event file, 
        # remove the tile event file.
        rm $event_file
    done
fi

#-------------------------------------------------------------------------------
# CLEANUP
#-------------------------------------------------------------------------------
# Remove excess files created by the scripts that had been executed. Do not remove 
# the files when the script is in debugging mode.
if [ "$MIPS_DEBUG" != "YES" ]; then
    /bin/rm -f idlsave.pro flsave.pro runidl geotiff_metadata.txt idl_output.txt
    # Remove *.dat files that were created by the reproject_ms2gt_*.bash scripts. 
    /bin/rm -f *.dat $ref_lres $ref_hres
fi

echo 
echo "(Ended: "$(date)")"

