#!/bin/bash

# ============================================================================
# MIPS KML processor BASH script
# 
# This is the main script through which the KML processing is started. The 
# script search for all the KML composite tile event files created by the 
# the script process_granule_tiles.bash, and create the necessary KML files 
# for each of these files. 
# 
# Note that the script does not take any input parameters.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

#-------------------------------------------------------------------------------
# Check that all the necessary environmental variables are present.
#-------------------------------------------------------------------------------
# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

if [ -z "$IDL_CMD" ]; then
  echo "Environment variable IDL_CMD is not set"
  exit 1
fi

if [ -z "$MIPS_TILE_DIR" ]; then
  echo "Environment variable MIPS_TILE_DIR is not set"
  exit 1
fi

if [ -z "$MIPS_KML_DIR" ]; then
  echo "Environment variable MIPS_KML_DIR is not set"
  exit 1
fi

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

# Create the intermediate directory where the KML data will be archived while 
# the data are beign processed. When all the KML files have been produced, it 
# will be copied to the final KML product directory.
MIPS_KML_INTERM_DIR=$MIPS_KML_DIR"/intermediate"
if [ ! -e $MIPS_KML_INTERM_DIR ]; then
    mkdir -p $MIPS_KML_INTERM_DIR
fi

# For each band combination that are defined in the configuration file 
# MIPS_BAND_COMB_CONF, a separate set of KML files will be created. 
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for line_str in $(grep -v "#" $MIPS_BAND_COMB_CONF); do
    band_r=$(echo $line_str | awk '{print $1}')
    band_g=$(echo $line_str | awk '{print $2}')
    band_b=$(echo $line_str | awk '{print $3}')
    band_ls=$(echo $line_str | awk '{print $4}')
    band_hs=$(echo $line_str | awk '{print $5}')
    
    band_r_str_len=${#band_r}
    if [ $band_r_str_len -eq 1 ]; then
        band_r="0$band_r"
    fi

    band_g_str_len=${#band_g}
    if [ $band_g_str_len -eq 1 ]; then
        band_g="0$band_g"
    fi

    band_b_str_len=${#band_b}
    if [ $band_b_str_len -eq 1 ]; then
        band_b="0$band_b"
    fi

    band_ls_str_len=${#band_ls}
    if [ $band_ls_str_len -eq 1 ]; then
        band_ls="0$band_ls"
    fi

    band_hs_str_len=${#band_hs}
    if [ $band_hs_str_len -eq 1 ]; then
        band_hs="0$band_hs"
    fi
        
    if [ "$band_ls" == "" ]; then
        band_comb=$band_r"_"$band_g"_"$band_b"_00_00"
    else
        band_comb=$band_r"_"$band_g"_"$band_b"_"$band_ls"_"$band_hs
    fi
    
    # Produce the KML files for the specific band combination.
    /bin/bash $MIPS_HOME/scripts/auxiliary/process_kml_bandcomb.bash $band_comb
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
done
IFS=$OLD_IFS

echo 
echo "(Syncing intermediate data to output directory)"
rsync -aurv $MIPS_KML_INTERM_DIR/* $MIPS_KML_DIR/

rm -f flsave.pro idlsave.pro idl_output.txt runidl

echo 
echo "(Ended: "$(date)")"

