#!/bin/bash

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Check for MIPS_CONF environment variable
if [ -z "$MIPS_CONF" ]; then
    echo "Environment variable MIPS_CONF is not set. Source the mips_env.bash file."
    exit 1
fi

MIPS_AUX=$MIPS_HOME/scripts/auxiliary

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

mkdir -p $MIPS_MOSAIC_DIR

# First remove composite event files which are older than two days. This is to prevent 
# possible stale composite event files which could disrupt the processing
#if [ "$MIPS_DEBUG" != "YES" ]; then
#    echo "(DEBUG) Remove stale mosaic*.event files"
#    find $MIPS_TILE_DIR -name "mosaic*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
#fi

# Get a list of all the available mosaic tiff files which are older than day.
mosaic_event_file_lst=$(mktemp)
find $MIPS_TILE_DIR -name "mosaic*.event" -type f -mmin +1440  -mmin -$MIPS_LIFET_EVENTF -exec ls {} \; > $mosaic_event_file_lst

# Find the unique dates and band combinations of all the event files listed
date_file_lst=$(mktemp)
uniq_date_file_lst=$(mktemp)
for file_str in $(cat $mosaic_event_file_lst); do
    tmp_sat_str=$(basename $file_str | cut -d"." -f 3)
    tmp_date_str=$(basename $file_str | cut -d"." -f 4)
    tmp_band_comb_str=$(basename $file_str | cut -d"." -f 5)
    echo $tmp_sat_str"."$tmp_date_str"."$tmp_band_comb_str >> $date_file_lst
done
sort $date_file_lst | uniq > $uniq_date_file_lst

nr_lines_uniq_date_file_lst=$(wc -l $uniq_date_file_lst | awk '{print $1}')
if [ "$nr_lines_uniq_date_file_lst" == "0" ]; then
    echo
    echo "(DEBUG) No mosaic*.event files were found that can be used to make mosaic images."
fi

current_date_str=$(date +"%Y%m%d")
# Check if the mosaic file has already been created
# For each config, check if all the tiles are available
uniq_mosaic_file_lst=$(mktemp)
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for cfg_line_str in $(grep -v '#' $MIPS_MOSAIC_CONF); do
    mosaic_name_str=$(echo $cfg_line_str | cut -d";" -f 1)
    mosaic_ul_lr_str=$(echo $cfg_line_str | cut -d";" -f 2)
    mosaic_resol_str=$(echo $cfg_line_str | cut -d";" -f 3 | tr "[:lower:]" "[:upper:]")
    
    # Determine the resolution of the output image
    case $mosaic_resol_str in 
        "LRES" )
            mosc_resol_str=$(echo "$MIPS_PC_PIXEL_RESOL * 4" | bc)
        ;;
        
        "MRES" )
            mosc_resol_str=$(echo "$MIPS_PC_PIXEL_RESOL * 2" | bc)
        ;;
        
        "HRES" )
            mosc_resol_str=$MIPS_PC_PIXEL_RESOL
        ;;
    esac
    
    echo
    echo "(Processing "$mosaic_name_str" - $mosaic_ul_lr_str - $mosaic_resol_str)"
    
    mosc_ul_lon_str=$(echo $mosaic_ul_lr_str | cut -d" " -f 1)
    mosc_ul_lat_str=$(echo $mosaic_ul_lr_str | cut -d" " -f 2)
    mosc_lr_lon_str=$(echo $mosaic_ul_lr_str | cut -d" " -f 3)
    mosc_lr_lat_str=$(echo $mosaic_ul_lr_str | cut -d" " -f 4)
    
    tile_idr_out=tile_id_range_output.txt
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_tileid_range.bash 'Normal' \
        $mosc_ul_lon_str $mosc_ul_lat_str $mosc_lr_lon_str $mosc_lr_lat_str $tile_idr_out
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
    
    nr_tiles_count=$(wc -l $tile_idr_out | awk '{print $1}')
    
    for date_bandcomb_str in $(cat $uniq_date_file_lst); do
        mosc_sat_str=$(echo $date_bandcomb_str | cut -d"." -f 1)
        mosc_date_str=$(echo $date_bandcomb_str | cut -d"." -f 2)
        mosc_bandcomb_str=$(echo $date_bandcomb_str | cut -d"." -f 3)
        
        echo
        echo "    ($mosc_date_str - $mosc_bandcomb_str - $mosc_sat_str)"
        
        # Check if the output file already exist
        jpeg_outdir_str=$MIPS_MOSAIC_DIR/$mosaic_name_str/jpeg
        tiff_outdir_str=$MIPS_MOSAIC_DIR/$mosaic_name_str/tiff
        mkdir -p $jpeg_outdir_str $tiff_outdir_str
        
        outfile_name_tif_str=$mosaic_name_str"."$date_bandcomb_str"."$mosc_resol_str"m.tif"
        outfile_tif_str=$tiff_outdir_str/$outfile_name_tif_str
        outfile_name_jpg_str=$mosaic_name_str"."$date_bandcomb_str"."$mosc_resol_str"m.jpg"
        outfile_jpg_str=$jpeg_outdir_str/$outfile_name_jpg_str
        
        if [ -e $outfile_jpg_str ]; then
            if [ "$MIPS_DEBUG" == "NO" ]; then
                echo
                echo "(DEBUG) The output mosaic file "$outfile_jpg_str" already exist."
            fi
            #echo "    (INFO) The output mosaic file "$outfile_jpg_str" already exist."
            
            continue
        fi
        
        grep $date_bandcomb_str $mosaic_event_file_lst > $uniq_mosaic_file_lst
        
        # Check if all the tiles are in present in the uniq_mosaic_file_lst
        date_tiles_count=0
        for tile_nr_str in $(cat $tile_idr_out); do
            grep $tile_nr_str $uniq_mosaic_file_lst &> /dev/null
            if [ $? -eq 0 ]; then
                date_tiles_count=$((date_tiles_count + 1))
            fi
        done
        
        # Check to see if all the tiles are available
        #uniq_mosaic_file_count=$(wc -l $uniq_mosaic_file_lst | awk '{print $1}')
        if [ "$date_tiles_count" != "$nr_tiles_count" ]; then
            date_diff_int=$(echo "$current_date_str - $mosc_date_str" | bc)
            # If the tiles have been updated 3 days ago, there will be no new images. Therefore continue to make 
            # the mosaic image
            if [ $date_diff_int -le 2 ]; then
                if [ "$MIPS_DEBUG" == "YES" ]; then
                    echo
                    echo "(DEBUG) The mosaic file "$outfile_name_jpg_str" will not be created. Not all the required tiles are present."
                    echo "(DEBUG)     The number of tiles detected: "$date_tiles_count
                    echo "(DEBUG)     The number of tiles that are needed: "$nr_tiles_count
                fi
                
                continue
            fi
            
        fi
        
        # Create the mosaic image
        /bin/bash $MIPS_AUX/make_image_mosaic.bash $MIPS_TILE_DIR $mosc_ul_lon_str,$mosc_ul_lat_str:$mosc_lr_lon_str,$mosc_lr_lat_str \
            $mosc_date_str $mosc_sat_str $mosc_bandcomb_str $mosc_resol_str $outfile_name_tif_str
        
        if [ $? -ne 0 ]; then
            exit 1
        fi
        
        convert -quality 95 $outfile_name_tif_str $outfile_jpg_str
        if [ ! -e $outfile_jpg_str ]; then
            echo
            echo "(DEBUG) The 'convert' command failed to create the file "$outfile_jpg_str
        fi
        
        cp $outfile_name_tif_str $outfile_tif_str
    done
done
IFS=$OLD_IFS

#for mosaic_event_file in $(cat $mosaic_event_file_lst); do
#    rm -f $mosaic_event_file
#done

rm -f $outfile_name_tif_str
rm -f $mosaic_event_file_lst $uniq_mosaic_file_lst 
rm -f $date_file_lst $uniq_date_file_lst $tile_idr_out

echo 
echo "(Ended: "$(date)")"

