#!/bin/bash

# ============================================================================
# MIPS polar mosaic image processor BASH script
#
# Create polar mosaic images from all the available polar composite images. 
# Polar mosaic images are created for both the North pole and the South pole. 
# Each polar region mosaic image is create seperately. 
#
# Note that this script does not depend on any input arguments. The script 
# search through the polar tile directories for composite event files, and the 
# event files are used to determine what polar mosaic images should be created.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Check for MIPS_CONF environment variable
if [ -z "$MIPS_CONF" ]; then
    echo "Environment variable MIPS_CONF is not set. Source the mips_env.bash file."
    exit 1
fi

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

# First remove composite event files which are older than two days. This is to prevent 
# possible stale composite event files which could disrupt the processing
#if [ "$MIPS_DEBUG" != "YES" ]; then
#    find $MIPS_POLR_TILE_DIR -name "polarmosaic.ptile*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
#fi

if [ ! -e $MIPS_MOSAIC_DIR ]; then
    mkdir -p $MIPS_MOSAIC_DIR
fi

# ==========================================================================================
# North Pole
# ==========================================================================================
echo
echo "(Processing Northen polar data)"
cache_file=$(mktemp)
# Get a list of all the available composite TIFF files for the Northen polar region
comp_file_lst=$(mktemp)
find $MIPS_POLR_TILE_DIR/north -type f -name "polarmosaic.ptile*.event" -mmin -$MIPS_LIFET_EVENTF -exec ls {} \; > $comp_file_lst

for event_file in $(cat $comp_file_lst); do
    echo "(Processing event file "$(basename $event_file)")"
    event_file_base=$(basename $event_file)
    
    # Extract metadata information from the event file name.
    sat_str=$(echo $event_file_base | cut -d"." -f 3)
    date_str=$(echo $event_file_base | cut -d"." -f 4)
    band_comb_str=$(echo $event_file_base | cut -d"." -f 5)
    
    # The script which creates the polar mosaic images, 'make_image_mosaic.bash', search 
    # by itself for all the available polar composite event files. 'make_image_mosaic.bash' 
    # is executed for per region, date, satellite and band combination. Therefore this script must 
    # keep track what event files have already been used for processing. A cache file is used 
    # to keep track of what satellite, band combination and satellite's event files have already 
    # been used.
    cache_id_str=$sat_str"."$date_str"."$band_comb_str
    grep $cache_id_str $cache_file &> /dev/null
    if [ $? -eq 0 ]; then
        # This satellite, with this band combination and with this date has already been 
        # processed. Therefore delete the event file and skip the processing of this event file. 
        #if [ "$MIPS_DEBUG" != "YES" ]; then 
        #    rm -f $event_file
        #fi
        
        # This satellite, with this band combination and with this date has already been 
        # processed. Therefore skip the processing of this event file.
        continue
    fi
    echo $cache_id_str >> $cache_file
    
    echo "    (Creating Northen polar mosaic for "$cache_id_str")"
    
    jpeg_out_dir=$MIPS_MOSAIC_DIR/Arctic/jpeg
    tiff_out_dir=$MIPS_MOSAIC_DIR/Arctic/tiff
    mkdir -p $jpeg_out_dir $tiff_out_dir
    
    # Construct the output file name of the North polar mosaic image.
    output_file=$tiff_out_dir/"Arctic."$sat_str"."$date_str"."$band_comb_str"."$MIPS_POLAR_MOSIAC_RESOL"m.tif"
    /bin/bash $MIPS_HOME/scripts/auxiliary/make_image_mosaic.bash $MIPS_POLR_TILE_DIR/north polar-north $date_str $sat_str \
        $band_comb_str $MIPS_POLAR_MOSIAC_RESOL $output_file
    
    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi
    
    # Create a JPEG version of the TIFF image. 
    out_jpeg_file=$jpeg_out_dir/"Arctic."$sat_str"."$date_str"."$band_comb_str"."$MIPS_POLAR_MOSIAC_RESOL"m.jpg"
    eval convert -quality 90 $output_file $out_jpeg_file
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E65] The command 'convert' failed to create the JPEG image "$out_jpeg_file
        exit 1
    fi
    
    # Create also a 1KM JPEG image.
    if [ "$MIPS_POLAR_MOSIAC_1KM" == "YES" ]; then
        scale_val=$(echo "scale=4; ($MIPS_POLAR_MOSIAC_RESOL / 1000.0) * 100.0" | bc)
        out_jpeg_1km_file=$jpeg_out_dir/"Arctic."$sat_str"."$date_str"."$band_comb_str".1000m.jpg"
        eval convert -quality 90 -scale $scale_val% $output_file $out_jpeg_1km_file
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E93] The command 'convert' failed to create the JPEG image "$out_jpeg_1km_file
            exit 1
        fi
    fi
    
    # Create also a 4KM JPEG image.
    if [ "$MIPS_POLAR_MOSIAC_4KM" == "YES" ]; then
        scale_val=$(echo "scale=4; ($MIPS_POLAR_MOSIAC_RESOL / 4000.0) * 100.0" | bc)
        out_jpeg_4km_file=$jpeg_out_dir/"Arctic."$sat_str"."$date_str"."$band_comb_str".4000m.jpg"
        eval convert -quality 90 -scale $scale_val% $output_file $out_jpeg_4km_file
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E94] The command 'convert' failed to create the JPEG image "$out_jpeg_4km_file
            exit 1
        fi
    fi
       
    # When debugging mode is disabled and the polar mosaic image was successfully created, remove 
    # the event file.
    #if [ "$MIPS_DEBUG" == "NO" ]; then 
    #   rm -f $event_file
    #fi
done

# Delete the temporary files
rm -f $comp_file_lst $cache_file

# ==========================================================================================
# South Pole
# ==========================================================================================
echo
echo "(Processing Southern polar data)"
cache_file=$(mktemp)
# Get a list of all the available composite TIFF files for the Southern polar region
comp_file_lst=$(mktemp)
find $MIPS_POLR_TILE_DIR/south -type f -name "polarmosaic.ptile*.event" -mmin -$MIPS_LIFET_EVENTF -exec ls {} \; > $comp_file_lst

for event_file in $(cat $comp_file_lst); do
    echo "(Processing event file "$(basename $event_file)")"
    event_file_base=$(basename $event_file)
    
    # Extract metadata information from the event file name.
    sat_str=$(echo $event_file_base | cut -d"." -f 3)
    date_str=$(echo $event_file_base | cut -d"." -f 4)
    band_comb_str=$(echo $event_file_base | cut -d"." -f 5)
    
    # The script which creates the polar mosaic images, 'make_image_mosaic.bash', search 
    # by itself for all the available polar composite event files. 'make_image_mosaic.bash' 
    # is executed for per region, date, satellite and band combination. Therefore this script must 
    # keep track what event files have already been used for processing. A cache file is used 
    # to keep track of what satellite, band combination and satellite's event files have already 
    # been used.
    cache_id_str=$sat_str"."$date_str"."$band_comb_str
    grep $cache_id_str $cache_file &> /dev/null
    if [ $? -eq 0 ]; then
        # This satellite, with this band combination and with this date has already been 
        # processed. Therefore delete the event file and skip the processing of this event file. 
        #if [ "$MIPS_DEBUG" == "NO" ]; then 
        #    rm -f $event_file
        #fi
        
        # This satellite, with this band combination and with this date has already been 
        # processed. Therefore skip the processing of this event file.
        
        rm -f $event_file
        
        continue
    fi
    echo $cache_id_str >> $cache_file

    echo "    (Creating Southern polar mosaic for "$cache_id_str")"
    
    jpeg_out_dir=$MIPS_MOSAIC_DIR/Antarctic/jpeg
    tiff_out_dir=$MIPS_MOSAIC_DIR/Antarctic/tiff
    mkdir -p $jpeg_out_dir $tiff_out_dir
    
    # Construct the output file name of the North polar mosaic image.
    output_file=$tiff_out_dir/"Antarctic."$sat_str"."$date_str"."$band_comb_str"."$MIPS_POLAR_MOSIAC_RESOL"m.tif"
    /bin/bash $MIPS_HOME/scripts/auxiliary/make_image_mosaic.bash $MIPS_POLR_TILE_DIR/south polar-south $date_str $sat_str \
        $band_comb_str $MIPS_POLAR_MOSIAC_RESOL $output_file

    # If an error occured, exit immediately.
    if [ $? -ne 0 ]; then
        exit 1
    fi

    # Create a JPEG version of the TIFF image. 
    out_jpeg_file=$jpeg_out_dir/"Antarctic."$sat_str"."$date_str"."$band_comb_str"."$MIPS_POLAR_MOSIAC_RESOL"m.jpg"
    eval convert -quality 90 $output_file $out_jpeg_file
    if [ $? -ne 0 ]; then
        echo "(ERROR)[E75] The command 'convert' failed to create the JPEG image "$out_jpeg_file
        exit 1
    fi
    
    # Create also a 1KM JPEG image.
    if [ "$MIPS_POLAR_MOSIAC_1KM" == "YES" ]; then
        scale_val=$(echo "scale=4; ($MIPS_POLAR_MOSIAC_RESOL / 1000.0) * 100.0" | bc)
        out_jpeg_1km_file=$jpeg_out_dir/"Antarctic."$sat_str"."$date_str"."$band_comb_str".1000m.jpg"
        eval convert -quality 90 -scale $scale_val% $output_file $out_jpeg_1km_file
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E95] The command 'convert' failed to create the JPEG image "$out_jpeg_1km_file
            exit 1
        fi
    fi
    
    # Create also a 4KM JPEG image.
    if [ "$MIPS_POLAR_MOSIAC_4KM" == "YES" ]; then
        scale_val=$(echo "scale=4; ($MIPS_POLAR_MOSIAC_RESOL / 4000.0) * 100.0" | bc)
        out_jpeg_4km_file=$jpeg_out_dir/"Antarctic."$sat_str"."$date_str"."$band_comb_str".4000m.jpg"
        eval convert -quality 90 -scale $scale_val% $output_file $out_jpeg_4km_file
        if [ $? -ne 0 ]; then
            echo "(ERROR)[E96] The command 'convert' failed to create the JPEG image "$out_jpeg_4km_file
            exit 1
        fi
    fi
    
    # When debugging mode is disabled and the polar mosaic image was successfully created, remove 
    # the event file.
    #if [ "$MIPS_DEBUG" != "YES" ]; then 
    #    rm -f $event_file
    #fi
    rm -f $event_file
done

# Delete the temporary files
rm -f $comp_file_lst $cache_file

echo 
echo "(Ended: "$(date)")"

