#!/bin/bash
#
# ============================================================================
# MIPS subset image processor BASH script
# 
# This script creates subsets images based on the subset configuration files. 
# The script search for all available composite image tiles, and create 
# a subset image for each configured subset images. The steps that are followed 
# to create the subset images, are as follows:
# 1. For each subset configuration, configuration parameters are loaded
# 2. All the composite image tiles which overlap the geographical region of 
#    the loaded subset configuration, is located
# 3. The subset image is created
# 4. The metadata of the subset image is created
# 5. A geotiff version of the subset image is created
# 6. A JPEG version of the subset image is created
# 
# If more than 40% of the subset image is black pixels (missing data), the 
# the subset image will be in an intermediate directory. When less then 40% 
# of the image is black pixels, the subset image will be copied in the final 
# subset output directory.
# 
# Note that the script does not take any input parameters.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Check for MIPS_CONF environment variable
if [ -z "$MIPS_CONF" ]; then
    echo "Environment variable MIPS_CONF is not set. Source the dbge_env.bash file."
    exit 1
fi

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

# First remove composite event files which are older than two days. This is to prevent 
# possible stale composite event files which could disrupt the processing
#if [ "$MIPS_DEBUG" != "YES" ]; then
#    find $MIPS_TILE_DIR -name "subset.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
#fi

# Get a list of all the available composite tiff files.
comp_file_lst=$(mktemp)
uniq_comp_file_lst=$(mktemp)
find $MIPS_TILE_DIR -name "subset.tile*.event" -type f -mmin -$MIPS_LIFET_EVENTF -exec ls {} \; > $comp_file_lst

# Find all the unique date, band combinations and satellite names
tmp_file_str=$(mktemp)
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for line_str in $(cat $comp_file_lst); do
    file_bsname_str=$(basename $line_str)
    sat_name_str=$(echo $file_bsname_str | cut -d"." -f 3)
    yeardate_str=$(echo $file_bsname_str | cut -d"." -f 4)
    bandcomb_str=$(echo $file_bsname_str | cut -d"." -f 5)
    
    echo $sat_name_str"."$yeardate_str"."$bandcomb_str >> $tmp_file_str
done
IFS=$OLD_IFS

sort -u $tmp_file_str | uniq > $uniq_comp_file_lst 

OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for line_str in $(cat $uniq_comp_file_lst); do
    sat_name_str=$(echo $line_str | cut -d"." -f 1)
    yeardate_str=$(echo $line_str | cut -d"." -f 2)
    bandcomb_str=$(echo $line_str | cut -d"." -f 3)
    
    /bin/bash $MIPS_HOME/scripts/auxiliary/make_cut_subset_date_sat.bash $yeardate_str $sat_name_str $bandcomb_str incomplete
    
    if [ $? -ne 0 ]; then
        exit 1
    fi
done
IFS=$OLD_IFS

rm -f $tmp_file_str $uniq_comp_file_lst $comp_file_lst

echo 
echo "(Ended: "$(date)")"

