#!/bin/bash
#
# ============================================================================
# MIPS subset image processor BASH script
# 
# This script creates subsets images based on the subset configuration files. 
# The script search for all available composite image tiles, and create 
# a subset image for each configured subset images. The steps that are followed 
# to create the subset images, are as follows:
# 1. For each subset configuration, configuration parameters are loaded
# 2. All the composite image tiles which overlap the geographical region of 
#    the loaded subset configuration, is located
# 3. The subset image is created
# 4. The metadata of the subset image is created
# 5. A geotiff version of the subset image is created
# 6. A JPEG version of the subset image is created
# 
# If more than 40% of the subset image is black pixels (missing data), the 
# the subset image will be in an intermediate directory. When less then 40% 
# of the image is black pixels, the subset image will be copied in the final 
# subset output directory.
# 
# Note that the script does not take any input parameters.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Check for MIPS_CONF environment variable
if [ -z "$MIPS_CONF" ]; then
    echo "Environment variable MIPS_CONF is not set. Source the dbge_env.bash file."
    exit 1
fi

# See if the subsets.conf file exists
if [ ! -e $MIPS_SUBSET_CONF ]; then
    echo "The configuration file \"$MIPS_SUBSET_CONF\" is not present."
    echo "Processing cannot continue without this file."
    exit 1
fi

# Load the sensor processing constants
if [ -z $MIPS_SENSOR_CONF ]; then
    echo "(ERROR)[Exx] The processing constants configuration file "$MIPS_SENSOR_CONF" does not exist"
    exit 1
fi
source $MIPS_SENSOR_CONF

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

# First remove composite event files which are older than two days. This is to prevent 
# possible stale composite event files which could disrupt the processing
if [ "$MIPS_DEBUG" != "YES" ]; then
    find $MIPS_TILE_DIR -name "subset.*.event" -type f -mmin +$MIPS_LIFET_EVENTF -exec /bin/rm {} \;
fi

# Get a list of all the available composite tiff files.
comp_file_lst=$(mktemp)
find $MIPS_TILE_DIR -type f -name "subset.tile*.event" -exec ls {} \; > $comp_file_lst
#-------------------------------------------------------------------------------
# LOOP THROUGH ALL THE DEFINED SUBSETS
#-------------------------------------------------------------------------------
if [ ! -e $MIPS_SUBSET_CONF ]; then
    echo "(ERROR)[E45] The subset configuration file "$MIPS_SUBSET_CONF" is missing"
    exit 1
fi

if [ ! -e $MIPS_RO_SUBSET_CONF ]; then
    echo "(ERROR)[E87] The subset configuration file "$MIPS_SUBSET_CONF" is missing"
    exit 1
fi

subset_comb_file="subset_combine_file.txt"
cat $MIPS_RO_SUBSET_CONF > $subset_comb_file
cat $MIPS_SUBSET_CONF   >> $subset_comb_file

# The configuration files has spaces in the lines, thus manipulate the BASH separator
OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for conf_line_str in $(cat $subset_comb_file | grep -v "#"); do
    # Get the subset configuration parameters
    subset_name_str=$(echo $conf_line_str | cut -d";" -f 1)
    coord_bounds_str=$(echo $conf_line_str | cut -d";" -f 2)
    output_dir=$MIPS_SUBSET_DIR
    
    echo
    echo "(Creating subset $subset_name_str)"
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "subset_name_str: $subset_name_str"
        echo "coord_bounds_str: $coord_bounds_str"
        echo "output_dir: $output_dir"
    fi
    
    intermediate_dir=$output_dir"/intermediate"
    # Make sure that the intermediate subset directory exists
    if [ ! -e $intermediate_dir ]; then
        mkdir -p $intermediate_dir
    fi
    
#-------------------------------------------------------------------------------
# GET THE TILE NUMBERS WHICH OVERLAPS THE CONFIGURED SUBSET GEOGRAPHICAL AREA
#-------------------------------------------------------------------------------
    ss_ul_lon=$(echo $coord_bounds_str | awk '{print $1}')
    ss_ul_lat=$(echo $coord_bounds_str | awk '{print $2}')
    ss_lr_lon=$(echo $coord_bounds_str | awk '{print $3}')
    ss_lr_lat=$(echo $coord_bounds_str | awk '{print $4}')
    
    tile_idr_out=tile_id_range_output.txt
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_tileid_range.bash 'Normal' \
        $ss_ul_lon $ss_ul_lat $ss_lr_lon $ss_lr_lat $tile_idr_out
    
    # If an error occured, exit immediately.
    if [ $? -eq 1 ]; then
        exit 1
    fi
    
    # If the output of the script get_tileid_range.bash is empty, there are then 
    # no tile numbers which cover the subset geographical area.
    tilename_lst=$(sort $tile_idr_out)
    if [ -z "$tilename_lst" ]; then
        echo "(WARNING)[W2] The subset configuration of '$subset_name_str' does not cover any tiles"
        continue
    fi
    
#-------------------------------------------------------------------------------
# LOOP THROUGH ALL TILES NUMBERS WHICH OVERLAPS THE DEFINED SUBSET
#-------------------------------------------------------------------------------
    for tilename_str in $tilename_lst; do
        echo "    (Processing "$tilename_str")"
        
#-------------------------------------------------------------------------------
# LOOP THROUGH ALL THE AVAILABLE COMPOSITE FILES WHICH GIVEN TILE NUMBER
#-------------------------------------------------------------------------------
        # Make subset images
        for comp_file_path in $(grep $tilename_str $comp_file_lst); do
            # Get the tile composite image event file
            comp_file_name=$(basename $comp_file_path)
            tilenum=$(echo $comp_file_name | cut -d"." -f 2)
            satname=$(echo $comp_file_name | cut -d"." -f 3)
            yeardate=$(echo $comp_file_name | cut -d"." -f 4)
            band_comb=$(echo $comp_file_name | cut -d"." -f 5)

            # Get the most recent composite file. The most recent composite file has a number which indicates 
            # how many satellite passes overlaps the tile number. Therefore the most recent image composite, is 
            # where there are the most satellite passes over the tile number
            comp_mdata_tmp_file=$(mktemp)
            ls $MIPS_TILE_DIR/$tilenum/$tilenum.$satname.$yeardate.$band_comb.*.dat > $comp_mdata_tmp_file
            # The image metadata file has the following fields:
            #     <tile number>.<satellite name>.<yeardata>.<band combination>.<amount over passes>.dat
            # The fith field indicates how many satellite passes overlaps the tile number.
            /bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $comp_mdata_tmp_file $comp_mdata_tmp_file 5
            # The last file has the most satellite passes, therefore it is the most recent composite image 
            comp_mdata_file=$(tail -n 1 $comp_mdata_tmp_file)
            
            if [ -z "$comp_mdata_file" ]; then
                echo "        (WARNING)[Wxx] There are no image files for event file "$comp_file_name". The event file will be removed."
                rm -f $comp_file_name
                continue
            fi
            
            rm -f $comp_mdata_tmp_file
            
            # From the metadata file number, create the composite image tile file file name
            comp_tileim_file=$(dirname $comp_mdata_file)/$(basename $comp_mdata_file ".dat")".png"
            
            echo "        (Processing file $comp_file_name)"
            
            # Make the subset image file name, and create the file paths to the 
            # final output and the intermediate output.
            subset_output_name=$subset_name_str.$satname.$yeardate.$band_comb.tif
            subset_final_output_path=$output_dir/$subset_output_name
            subset_inter_output_path=$intermediate_dir/$subset_output_name
            # Make the subset metadata file name for the subset image, and create the 
            # file paths to the final output and the intermediate output.
            subset_mdata_name=$subset_name_str.$satname.$yeardate.$band_comb.dat
            subset_final_mdata_path=$output_dir/$subset_mdata_name
            subset_inter_mdata_path=$intermediate_dir/$subset_mdata_name
            
#-------------------------------------------------------------------------------
# CREATE SUBSET IMAGE
#-------------------------------------------------------------------------------
            # Get the previous version of the subset image. The previous subset image 
            # will be updated.
            if [ -e $subset_inter_output_path ]; then
                cp $subset_inter_output_path $subset_output_name
            else
                # If there is already a subset image in the current directory, delete it. It 
                # should not be there.
                rm -f $subset_output_name
            fi
            /bin/bash $MIPS_HOME/scripts/auxiliary/make_cut_subset.bash $comp_tileim_file \
                                                                        $ss_ul_lon $ss_ul_lat \
                                                                        $ss_lr_lon $ss_lr_lat \
                                                                        $subset_output_name
            
            # If an error occured, exit immediately.
            if [ $? -ne 0 ]; then
                rm -f $subset_output_name
                exit 1
            fi
            
#-------------------------------------------------------------------------------
# CREATE METADATA FILE FOR SUBSET IMAGE
# 
# All the granules that were used to create the composite images, are copied 
# into the metadata file of the subset images. Therefore the subset image 
# metadata file has a complete list of all the L1B granules that were used 
# to create the subset image.
#-------------------------------------------------------------------------------
            # If the idl_output.txt file do not exist, the make_cut_subset.bash 
            # script failed.
            if [ ! -e idl_output.txt ]; then
                echo "(ERROR)[E84] The BASH script failed to produce a text file 'idl_output.txt'."
                exit 1
            fi
            
            # Get the corner coordinates for the subset image
            img_ss_ul_x=$(grep UL_X idl_output.txt | awk '{print $2}')
            img_ss_ul_y=$(grep UL_Y idl_output.txt | awk '{print $2}')
            img_ss_lr_x=$(grep LR_X idl_output.txt | awk '{print $2}')
            img_ss_lr_y=$(grep LR_Y idl_output.txt | awk '{print $2}')

            if [ ! -e $subset_mdata_name ]; then
                # If a metadata file does not exist, create a new one.
                echo "SUBSET_NAME = "$subset_name_str > $subset_inter_mdata_path
                
                grep "SAT" $comp_mdata_file          >> $subset_inter_mdata_path
                grep "SENSOR" $comp_mdata_file       >> $subset_inter_mdata_path
                grep "DATE" $comp_mdata_file         >> $subset_inter_mdata_path
                grep "GRAN_NAME" $comp_mdata_file    >> $subset_inter_mdata_path
                grep "BAND_COMB" $comp_mdata_file    >> $subset_inter_mdata_path
                
                echo "UL_X = " $img_ss_ul_x          >> $subset_inter_mdata_path
                echo "UL_Y = " $img_ss_ul_y          >> $subset_inter_mdata_path
                echo "LR_X = " $img_ss_lr_x          >> $subset_inter_mdata_path
                echo "LR_Y = " $img_ss_lr_y          >> $subset_inter_mdata_path
            else
                # If a metadata file already exist, add the new L1B granule names to 
                # the existing metadata file.
                gran_name_sub_str=$(grep "GRAN_NAME" $subset_inter_mdata_path)
                comp_gran_name_str=$(grep "GRAN_NAME" $comp_mdata_file | cut -d"=" -f 2 | awk '{print $1}')
                subs_gran_name_str=$(grep "GRAN_NAME" $subset_inter_mdata_path | cut -d"=" -f 2 | awk '{print $1}')
                gran_name_str=$subs_gran_name_str","$comp_gran_name_str
                
                sed -i "s/$gran_name_sub_str/$gran_name_str/g" $subset_inter_mdata_path
            fi
            
#-------------------------------------------------------------------------------
# CREATE GEOTIFF IMAGE
#-------------------------------------------------------------------------------
            cp $subset_output_name $subset_inter_output_path
            # If more than 40% of the image is full of black pixels, do not 
            # copy it to the final output directory. Otherwise, the subset images will 
            # reside in the intermediate directory.
            pblack_pixels=$(grep "Percentage black pixels:" idl_output.txt | awk '{print $4}')
            if [ "$MIPS_DEBUG" == "YES" ]; then
                echo "[DEBUG]: Percentage of black pixels in subset image: $pblack_pixels"
            fi
            
            if [ "$pblack_pixels" -gt 40 ]; then
                continue
            fi
            
            enh_subset_output_name="enhnc_"$subset_output_name
            eval convert -compress None $MIPS_IMG_SHARPEN_STR $subset_output_name $enh_subset_output_name
            
            geotiff_subset_output_name="geotiff_"$subset_output_name
            # Get a template of the geotiff metadata file from the MIPS installation.
            cat $MIPS_HOME/templates/geotiff_metadata.txt > geotiff_metadata.txt 
            # Add the upper left map coordinates to the duplicate geotiff metadata file.
            sed -i "s/@UL_X@/$img_ss_ul_x/g" geotiff_metadata.txt
            sed -i "s/@UL_Y@/$img_ss_ul_y/g" geotiff_metadata.txt
            sed -i "s/@RESOL@/$MIPS_PC_PIXEL_RESOL/g" geotiff_metadata.txt
            # Copy the new geotiff metadata into the existing TIFF image, thus a 
            # GeoTIFF image is created.
            $MIPS_HOME/bin/geotifcp -c lzw -g geotiff_metadata.txt $enh_subset_output_name $subset_final_output_path
            if [ $? -ne 0 ]; then
                echo "(ERROR)[E85] The binary geotifcp failed to create the GeoTIFF file "$(basename $subset_inter_output_path)
                exit 1
            fi
            
#-------------------------------------------------------------------------------
# CREATE JPEG IMAGE AND COPY TO FINAL OUTPUT DIRECTORY
#-------------------------------------------------------------------------------
            # If code executes from this point onwards, it assumed that the subset image 
            # can be copied to the final output directory.
            if [ -e $subset_output_path ]; then
                # Create the JPEG subset image from the subset TIFF image. First 
                # construct the file name and filesystem path of the JPEG subset image. 
                jpg_subset_output_path=$output_dir/$subset_name_str.$satname.$yeardate.$band_comb.jpg
                
                convert -quality 95 $subset_final_output_path $jpg_subset_output_path &> /dev/null
                if [ $? -ne 0 ]; then
                    echo "(ERROR)[E86] The ImageMagick command 'convert' failed to convert the TIFF file "$(basename $subset_inter_output_path)" into a JPEG file."
                    exit 1
                fi
                
                ## Copy GeoTIFF file
                #cp $geotiff_subset_output_name $subset_final_output_path
                # Copy metadata
                cp $subset_inter_mdata_path $subset_final_mdata_path
            fi
        done
    done
done

IFS=$OLD_IFS

# Remove all the composite image event files if debug mode is disabled.
if [ "$MIPS_DEBUG" != "YES" ]; then 
    for file_path in $(cat $comp_file_lst); do
        rm -f $file_path
    done
fi

# Remove temporary files if debug mode is disabled.
if [ "$MIPS_DEBUG" != "YES" ]; then
    rm -f flsave.pro geotiff_metadata.txt idl_output.txt runidl $subset_comb_file $tile_idr_out $geotiff_subset_output_name
    rm -f $enh_subset_output_name $subset_output_name
fi

echo 
echo "(Ended: "$(date)")"

