#!/bin/bash
#
# ============================================================================
# MIPS subset image processor BASH script
# 
# This script creates subsets images based on the subset configuration files. 
# The script search for all available composite image tiles, and create 
# a subset image for each configured subset images. The steps that are followed 
# to create the subset images, are as follows:
# 1. For each subset configuration, configuration parameters are loaded
# 2. All the composite image tiles which overlap the geographical region of 
#    the loaded subset configuration, is located
# 3. The subset image is created
# 4. The metadata of the subset image is created
# 5. A geotiff version of the subset image is created
# 6. A JPEG version of the subset image is created
# 
# If more than 40% of the subset image is black pixels (missing data), the 
# the subset image will be in an intermediate directory. When less then 40% 
# of the image is black pixels, the subset image will be copied in the final 
# subset output directory.
# 
# Note that the script does not take any input parameters.
# 
# Authors: Liam.Gumley@ssec.wisc.edu, Willem.Marais@ssec.wisc.edu
# Date: 07/22/2011
# ============================================================================

# Check for MIPS_HOME environment variable
if [ -z "$MIPS_HOME" ]; then
  echo "Environment variable MIPS_HOME is not set"
  exit 1
fi

# Echo command line
echo
echo "(Echoing command line)"
echo $(basename $0) $@

echo 
echo "(Started: "$(date)")"

if [ $# -ne 2 ]; then
    echo $(basename $0)" <subset config file> <output directory>"
    exit 1
fi
subset_cfg_file=$1
output_dir=$2

# Set the core dump file size limit. This is useful, because in an event where a
# program gives a segmentation fault, the core file can be used to debug the
# program. 
ulimit -c unlimited

if [ "$MIPS_DEBUG" == "YES" ]; then
    echo
    echo "[DEBUG]: Subset config file ============= START"
    cat $subset_cfg_file
    echo "[DEBUG]: Subset config file ============= STOP"
    echo
fi

OLD_IFS=$IFS
IFS=$(echo -e '\n\b')
for cfg_line_str in $(cat $subset_cfg_file | grep -v '#'); do
    subset_name_str=$(echo $cfg_line_str  | cut -d";" -f 1)
    coord_bounds_str=$(echo $cfg_line_str | cut -d";" -f 2)
    subset_sat_str=$(echo $cfg_line_str   | cut -d";" -f 3)
    subset_date_str=$(echo $cfg_line_str  | cut -d";" -f 4)
    subset_time_str=$(echo $cfg_line_str  | cut -d";" -f 5)
    subset_bandc_str=$(echo $cfg_line_str | cut -d";" -f 6)
    coord_ctr_str=$(echo $cfg_line_str    | cut -d";" -f 7)
    
    echo "(Creating subset $subset_name_str)"
    
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo
        echo "subset_name_str: $subset_name_str"
        echo "coord_bounds_str: $coord_bounds_str"
        echo "subset_sat_str: $subset_sat_str"
        echo "subset_date_str: $subset_date_str"
        echo "subset_time_str: $subset_time_str"
        echo "subset_bandc_str: $subset_bandc_str"
        echo "coord_ctr_str: $coord_ctr_str"
        echo
    fi
    
    #-------------------------------------------------------------------------------
    # GET THE TILE NUMBERS WHICH OVERLAPS THE CONFIGURED SUBSET GEOGRAPHICAL AREA
    #-------------------------------------------------------------------------------
    ss_ul_lon=$(echo $coord_bounds_str | awk '{print $1}')
    ss_ul_lat=$(echo $coord_bounds_str | awk '{print $2}')
    ss_lr_lon=$(echo $coord_bounds_str | awk '{print $3}')
    ss_lr_lat=$(echo $coord_bounds_str | awk '{print $4}')
    
    tile_idr_out=tile_id_range_output.txt
    /bin/bash $MIPS_HOME/scripts/auxiliary/get_tileid_range.bash 'Normal' \
        $ss_ul_lon $ss_ul_lat $ss_lr_lon $ss_lr_lat $tile_idr_out
    
    # If an error occured, exit immediately.
    if [ $? -eq 1 ]; then
        exit 1
    fi
    
    # If the output of the script get_tileid_range.bash is empty, there are then 
    # no tile numbers which cover the subset geographical area.
    tilename_lst=$(sort $tile_idr_out)
    if [ -z "$tilename_lst" ]; then
        echo "(WARNING)[W2] The subset configuration of '$subset_name_str' does not cover any tiles"
        continue
    fi
    
    #---------------------------------------------------------------------------------
    # IF THE SATELLITE NAME IS "NOSATELLITE", FIND THE NEAREST SATELLITE IMAGE IN 
    # TERMS OF TIME
    #---------------------------------------------------------------------------------
    # if [ "$subset_sat_str" == "nosatellite" ]; then
    #     smpl_tile_str=$(head -n 1 $tile_idr_out)
    #     smpl_tile_dir=$MIPS_TILE_DIR/$smpl_tile_str
    #     # Get a list of all the tiles in the tile directory (non composite tiles)
    #     tile_list_file_tmp=$(mktemp)
    #     datetime_list_file_tmp=$(mktemp)
    #     tmp_datetime_list_file_tmp=$(mktemp)
    #     
    #     ls $smpl_tile_dir/$smpl_tile_str.*.*.*.$subset_bandc_str.png > $tile_list_file_tmp
    #     for tmp_str in $(cat $tile_list_file_tmp); do
    #         basename_tmp_str=$(basename $tmp_str)
    #         tmp_date_str=$(echo $basename_tmp_str | cut -d"." -f 3)
    #         tmp_time_str=$(echo $basename_tmp_str | cut -d"." -f 4)
    # 
    #         echo $tmp_date_str$tmp_time_str >> $datetime_list_file_tmp
    #     done
    #     # Add the observation datetime to the list
    #     echo $subset_date_str$subset_time_str >> $datetime_list_file_tmp
    #     sort $datetime_list_file_tmp > $tmp_datetime_list_file_tmp
    #     cp $tmp_datetime_list_file_tmp $datetime_list_file_tmp
    #     
    #     # Find at which line number is the observation datetime in the list
    #     tmp_str=$(grep -n $subset_date_str$subset_time_str $datetime_list_file_tmp)
    #     if [ -z "$tmp_str" ]; then
    #         echo "(WARNING)[W3xx] Could not process "$subset_name_str", could not find a nearest image"
    #         continue
    #     fi
    #     obs_line_nr=$(echo $tmp_str | cut -d":" -f 1)
    #     
    #     # If the observation line number is equal to one, there is no nearest image
    #     if [ "$obs_line_nr" == "1" ]; then
    #         echo "(WARNING)[W4xx] Could not process "$subset_name_str", could not find a nearest image"
    #         continue
    #     fi
    #     
    #     nrst_line_nr=$(echo "$obs_line_nr - 1" | bc)
    #     nrst_datetime_str=$(head -n $nrst_line_nr $datetime_list_file_tmp | tail -n 1)
    #     nrst_date_str=$(echo $nrst_datetime_str | cut -c1-8)
    #     nrst_time_str=$(echo $nrst_datetime_str | cut -c9-12)
    # 
    #     # Get the image file name that has the nrst_date_str and nrst_time_str values
    #     nrst_tile_file_str=$(grep $nrst_date_str"."$nrst_time_str $tile_list_file_tmp)
    #     if [ -z "$nrst_tile_file_str" ]; then
    #         echo "(WARNING)[W5xx] Could not process "$subset_name_str", could not find a nearest image"
    #         continue
    #     fi
    #     basename_nrst_tile_file_str=$(basename $nrst_tile_file_str)
    #     nrst_tile_sat_str=$(echo $basename_nrst_tile_file_str | cut -d"." -f 2)
    #     
    #     # Set the new satellite, date and time values
    #     subset_sat_str=$nrst_tile_sat_str
    #     subset_date_str=$nrst_date_str
    #     subset_time_str=$nrst_time_str
    #     
    #     rm -f $tile_list_file_tmp $datetime_list_file_tmp $tmp_datetime_list_file_tmp
    #     
    #     if [ "$MIPS_DEBUG" == "YES" ]; then
    #         echo
    #         echo "(New subset definition)"
    #         echo "subset_name_str: $subset_name_str"
    #         echo "coord_bounds_str: $coord_bounds_str"
    #         echo "subset_sat_str: $subset_sat_str"
    #         echo "subset_date_str: $subset_date_str"
    #         echo "subset_time_str: $subset_time_str"
    #         echo "subset_bandc_str: $subset_bandc_str"
    #         echo
    #     fi
    # fi
	
	if [ "$subset_sat_str" == "nosatellite" ]; then
	    coord_ctr_lon=$(echo $coord_ctr_str | awk '{print $1}')
	    coord_ctr_lat=$(echo $coord_ctr_str | awk '{print $2}')
	    
		nosat_filt_file="nosat_filt_file.txt"
		python $MIPS_HOME/python/get_list_sat_observations.py $subset_date_str"+"$subset_time_str $coord_ctr_lat $coord_ctr_lon $nosat_filt_file
		if [ -e "$nosat_filt_file" ]; then
			subset_sat_str=$(grep "sat_name:" $nosat_filt_file | cut -d":" -f 2)
			subset_date_str=$(grep "datetime:" $nosat_filt_file | cut -d":" -f 2)
		else
		    echo "(ERROR)[Exx] Could not find a 'nearest' satellite for subset configuration '$subset_name_str'"
		    continue
		fi
        
        rm -f $nosat_filt_file
		
        case "$subset_sat_str" in 
            "aqua" | "terra" )
                subset_bandc_str="01_04_03_01_01"
            ;;
            
            "npp" )
                subset_bandc_str="05_04_03_05_12"
            ;;
        esac 
        
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo
            echo "(New subset definition)"
            echo "subset_name_str: $subset_name_str"
            echo "coord_bounds_str: $coord_bounds_str"
            echo "subset_sat_str: $subset_sat_str"
            echo "subset_date_str: $subset_date_str"
            echo "subset_time_str: $subset_time_str"
            echo "subset_bandc_str: $subset_bandc_str"
            echo "coord_ctr_str: $coord_ctr_str"
            echo
        fi
	fi
    
    #---------------------------------------------------------------------------------
    # GET A LIST OF ALL THE TILES THAT ARE AVAILABLE WITH THE SPECIFIC SATELLITE NAME, 
    # DATE, AND BAND COMBINATION
    #---------------------------------------------------------------------------------
    tmp_tile_img_lst=$(mktemp)
    find $MIPS_TILE_DIR -type f -name tile*.$subset_sat_str"."$subset_date_str"."$subset_bandc_str.*.png -exec ls {} \; > $tmp_tile_img_lst
    
    filt_tile_img_lst=$(mktemp)
    # First check if all the tiles images are available
    count_nr_tiles=0
    for tilename_str in $tilename_lst; do
        echo "    (Processing "$tilename_str")"
        
        tmp_filt_tile_img_lst=$(mktemp)
        grep $tilename_str $tmp_tile_img_lst > $tmp_filt_tile_img_lst
        
        sort_tile_img_lst=$(mktemp)
        # The image metadata file has the following fields:
        #     <tile number>.<satellite name>.<yeardata>.<band combination>.<amount over passes>.dat
        # The fith field indicates how many satellite passes overlaps the tile number.
        /bin/bash $MIPS_HOME/scripts/auxiliary/sort_comp_img_filelst.bash $tmp_filt_tile_img_lst $sort_tile_img_lst 5
               
        # The last file has the most satellite passes, therefore it is the most recent composite image 
        tail -n 1 $sort_tile_img_lst >> $filt_tile_img_lst
        count_nr_tiles=$(echo "$count_nr_tiles + 1" | bc)

        rm -f $tmp_filt_tile_img_lst
    done
    
    rm -f $sort_tile_img_lst

    nr_of_lines=$(wc -l $filt_tile_img_lst | awk '{print $1}')
    
    if [ $count_nr_tiles -ne $nr_of_lines ]; then
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo "        [DEBUG]: For subset "$subset_name_str" not all the tile composite images are available."
        fi
        
        rm -f $filt_tile_img_lst $tmp_tile_img_lst
        continue
    fi 
    
    for filt_tile_img_str in $(cat $filt_tile_img_lst); do
        echo "        (Processing file "$(basename $filt_tile_img_str)")"
        subset_output_name=""
        
        tile_img_dirname_str=$(dirname $filt_tile_img_str)
        tile_img_filename_str=$(basename $filt_tile_img_str ".png")
        metadata_file=$tile_img_dirname_str/$tile_img_filename_str".dat"
        
        gran_name_lst=$(grep "GRAN_NAME" $metadata_file | awk '{print $3}')
        sensor_name_str=$(grep "SENSOR" $metadata_file | awk '{print $3}')
        last_gran_name=$(echo $gran_name_lst | sed 's/./&\n/g' | tac | tr -d '\n' | cut -d "," -f 1 | sed 's/./&\n/g' | tac | tr -d '\n')
        if [ "$sensor_name_str" == "VIIRS" ]; then
            last_utctime=$subset_date_str"+"$(echo $last_gran_name | cut -d"_" -f 4 | cut -c2-5)
        else # MODIS
            last_utctime=$subset_date_str"+"$(echo $last_gran_name | cut -d"." -f 3)
        fi
        
        subset_output_name_jpg=$subset_name_str"."$subset_sat_str"."$subset_date_str"."$subset_bandc_str".jpg"
        subset_output_name_png=$subset_name_str"."$subset_sat_str"."$subset_date_str"."$subset_bandc_str".png"
        subset_output_name_dat=$subset_name_str"."$subset_sat_str"."$subset_date_str"."$subset_bandc_str".dat"
        subset_output_name_inf=$subset_name_str"."$subset_sat_str"."$subset_date_str"."$subset_bandc_str".inf"
        
        echo $last_utctime >> $output_dir/$subset_output_name_dat
        
        /bin/bash $MIPS_HOME/scripts/auxiliary/make_cut_subset.bash $filt_tile_img_str \
                                                                    $ss_ul_lon $ss_ul_lat \
                                                                    $ss_lr_lon $ss_lr_lat \
                                                                    $subset_output_name_png
        
    done
    
    pblack_pixels=$(grep "Percentage black pixels:" idl_output.txt | awk '{print $4}' | bc)
    if [ "$MIPS_DEBUG" == "YES" ]; then
        echo "[DEBUG]: Percentage of black pixels in subset image: $pblack_pixels"
    fi
    
    if [ "$pblack_pixels" -gt 5 ]; then
        if [ "$MIPS_DEBUG" == "YES" ]; then
            echo "[DEBUG]: The image "$subset_output_name_tif" will be discarded"
        fi
        
        rm -f $subset_output_name_png $output_dir/$subset_output_name_dat
        
        continue
    fi
    
    width_height_str=$(identify -format '%[width]x%[height]' $subset_output_name_png)
    if [ $? -ne 0 ]; then
        echo "(ERROR)[Exx] Could not determine the width of the height of the subset image"
        exit 1
    fi 
    ss_width_str=$(echo $width_height_str | cut -d"x" -f 1)
    ss_height_str=$(echo $width_height_str | cut -d"x" -f 2)
    
    ss_mdl_uw_str=$(echo "($ss_width_str / 2) - 1" | bc)
    ss_mdl_uh_str=$(echo "($ss_height_str / 2) - 1" | bc)
    ss_mdl_lw_str=$(echo "($ss_width_str / 2) + 1" | bc)
    ss_mdl_lh_str=$(echo "($ss_height_str / 2) + 1" | bc)
    
    draw_point_str="rectangle $ss_mdl_uw_str,$ss_mdl_uh_str $ss_mdl_lw_str,$ss_mdl_lh_str"
    convert $subset_output_name_png -fill 'rgb(59,255,246)' -draw "$draw_point_str" $subset_output_name_png 
    
    eval convert -quality 95 $MIPS_IMG_SHARPEN_STR $subset_output_name_png $output_dir/$subset_output_name_jpg
    
    # Create the and information file which holds the following information:
    # 1. The satellite name of the image 
    # 2. The centre latitude and longitude value of the burned in pixel
    # 3. The time and date of the satellite pass
    
    # First get the middle coordinate X and Y values and project those coordinates
    md_x_str=$(grep MD_X idl_output.txt | cut -d":" -f 2 | awk '{print $1}')
    md_y_str=$(grep MD_Y idl_output.txt | cut -d":" -f 2 | awk '{print $1}')
    
    input_proj4_str="+proj=latlong +datum=WGS84"
    non_polar_proj4_str="+proj=eqc +a=6371007.181 +b=6371007.181"
    cs2cs_exe_str="$MIPS_HOME/bin/cs2cs $non_polar_proj4_str +to $input_proj4_str -f %.6f"
    
    IFS=$OLD_IFS
    subset_mdl_UL_lon=$(echo "$md_x_str $md_y_str" | $cs2cs_exe_str | awk '{print $1}')
    subset_mdl_UL_lat=$(echo "$md_x_str $md_y_str" | $cs2cs_exe_str | awk '{print $2}')
    IFS=$(echo -e '\n\b')
    
    # Get the satellite name
    inf_sat_name_str=$(echo $subset_output_name_jpg | cut -d"." -f 2)
    
    # Get the latest utc time and date of this image
    inf_utc_date_str=$(tail -n 1 $output_dir/$subset_output_name_dat)
    # Convert the date string into the following format: YYYY-MM-DD+HH:MM:SS
    inf_year_str=$(echo $inf_utc_date_str | cut -c1-4)
    inf_month_str=$(echo $inf_utc_date_str | cut -c5-6)
    inf_day_str=$(echo $inf_utc_date_str | cut -c7-8)
    inf_hour_str=$(echo $inf_utc_date_str | cut -d"+" -f 2 | cut -c1-2)
    inf_minutes_str=$(echo $inf_utc_date_str | cut -d"+" -f 2 | cut -c3-4)
    inf_new_date_str=$inf_year_str"-"$inf_month_str"-"$inf_day_str"+"$inf_hour_str":"$inf_minutes_str":00"

    echo "SAT_NAME="$inf_sat_name_str >  $output_dir/$subset_output_name_inf
    echo "SAT_DATE="$inf_new_date_str >> $output_dir/$subset_output_name_inf
    echo "SAT_MDL_LON="$subset_mdl_UL_lon >> $output_dir/$subset_output_name_inf
    echo "SAT_MDL_LAT="$subset_mdl_UL_lat >> $output_dir/$subset_output_name_inf 
    
    rm -f $filt_tile_img_lst $tmp_tile_img_lst $subset_output_name_png
done
IFS=$OLD_IFS

rm -f $tile_idr_out flsave.pro idl_output.txt idlsave.pro runidl

echo 
echo "(Ended: "$(date)")"
