#!/bin/bash

if [ -z $MIPS_HOME ]; then
    echo "Environment variable MIPS_HOME is not set"
    exit 1
fi

if [ $# -ne 2 ]; then
    echo "(ERROR) Usage:"
    echo "    "$(basename $0)" <test case directory> <:kml:subsets:polarmosiac:globalmosaic:none:>"
    
    exit 1
fi

TEST_CASE_DIR=$1
process_flags=$2

TEST_OUT_DIR=$TEST_CASE_DIR/output
TEST_DATA_DIR=$TEST_CASE_DIR/input

# Clean output directory
rm -rf $TEST_OUT_DIR/*

# Check to see if the test case directories do exist
if [ ! -e $TEST_OUT_DIR ]; then
    echo "(ERROR) The directory "$TEST_OUT_DIR" does not exist."
    exit 1
fi

# Check to see if the test case directories do exist
if [ ! -e $TEST_DATA_DIR ]; then
    echo "(ERROR) The directory "$TEST_DATA_DIR" does not exist."
    exit 1
fi

# There should be two L1B files in the $TEST_DATA_DIR. Find the two files
l1b_svm02_file_path=$(ls $TEST_DATA_DIR/SVM02*.h5)
l1b_svm03_file_path=$(ls $TEST_DATA_DIR/SVM03*.h5)
l1b_svm04_file_path=$(ls $TEST_DATA_DIR/SVM04*.h5)
l1b_svm05_file_path=$(ls $TEST_DATA_DIR/SVM05*.h5)
l1b_svm07_file_path=$(ls $TEST_DATA_DIR/SVM07*.h5)
l1b_svi01_file_path=$(ls $TEST_DATA_DIR/SVI01*.h5)
l1b_svi02_file_path=$(ls $TEST_DATA_DIR/SVI02*.h5)
l1b_gmtco_file_path=$(ls $TEST_DATA_DIR/GMTCO*.h5)

if [ ! -e "$l1b_svm02_file_path" ]; then
    echo "(ERROR) Either there is no SVM02 L1B file in the directory "$TEST_DATA_DIR", or there is more than one."
    exit 1
fi

if [ ! -e "$l1b_svm03_file_path" ]; then
    echo "(ERROR) Either there is no SVM03 L1B file in the directory "$TEST_DATA_DIR", or there is more than one."
    exit 1
fi

if [ ! -e "$l1b_svm04_file_path" ]; then
    echo "(ERROR) Either there is no SVM04 L1B file in the directory "$TEST_DATA_DIR", or there is more than one."
    exit 1
fi

if [ ! -e "$l1b_svm05_file_path" ]; then
    echo "(ERROR) Either there is no SVM05 VIIRS L1B file in the directory "$TEST_DATA_DIR", or there is more than one."
    exit 1
fi

if [ ! -e "$l1b_gmtco_file_path" ]; then
echo "(ERROR) Either there is no GMTCO VIIRS L1B file in the directory "$TEST_DATA_DIR", or there is more than one."
exit 1
fi

# Import the VIIRS configuration file
source $MIPS_HOME/scripts/mips_viirs_env.bash

# Redefine the environmental variables
export MIPS_GRAN_IMG_DIR=$TEST_OUT_DIR/granimg
export MIPS_TILE_DIR=$TEST_OUT_DIR/tiles
export MIPS_POLR_TILE_DIR=$TEST_OUT_DIR/polartiles
export MIPS_SUBSET_DIR=$TEST_OUT_DIR/subsets
export MIPS_MOSAIC_DIR=$TEST_OUT_DIR/mosaic
export MIPS_KML_DIR=$TEST_OUT_DIR/kml
export MIPS_TEST_LOG_DIR=$TEST_OUT_DIR
export MIPS_TEST_SCRATCH_DIR=$TEST_OUT_DIR/scratch
export MIPS_DEBUG="YES"

# Create the product directories
mkdir -p $MIPS_GRAN_IMG_DIR $MIPS_TILE_DIR $MIPS_POLR_TILE_DIR
mkdir -p $MIPS_SUBSET_DIR $MIPS_KML_DIR $MIPS_TEST_SCRATCH_DIR $MIPS_MOSAIC_DIR

# Define log file paths
LOG_PROCESS_GRANULE=$MIPS_TEST_LOG_DIR/process_granule_tiles_viirs_wrap.log
LOG_PROCESS_KML=$MIPS_TEST_LOG_DIR/process_kml.log
LOG_PROCESS_SUBSETS=$MIPS_TEST_LOG_DIR/process_subsets.log
LOG_PROCESS_POLAR_MOSAIC=$MIPS_TEST_LOG_DIR/process_polar_mosaic.log
LOG_PROCESS_GLOBAL_MOSAIC=$MIPS_TEST_LOG_DIR/process_global_mosaic.log

cd $MIPS_TEST_SCRATCH_DIR

# Process the VIIRS L1B granule 
/bin/bash $MIPS_HOME/scripts/process_granule_tiles_viirs_wrap.bash $l1b_svm02_file_path \
    $l1b_svm03_file_path \
    $l1b_svm04_file_path \
    $l1b_svm05_file_path \
    $l1b_svm07_file_path \
    $l1b_svi01_file_path \
    $l1b_svi02_file_path $l1b_gmtco_file_path &> $LOG_PROCESS_GRANULE

grep ERROR $LOG_PROCESS_GRANULE &> /dev/null
if [ $? -eq 0 ]; then
    echo "An error occured in the script process_granule_tiles_viirs_wrap.bash. Refer to the log file "$LOG_PROCESS_GRANULE" for more details."
    
    cd -
    
    exit 1
fi

# Process the KML files
echo $process_flags | grep ":kml:" &> /dev/null
if [ $? -eq 0 ]; then 
    /bin/bash $MIPS_HOME/scripts/process_kml.bash &> $LOG_PROCESS_KML
    grep ERROR $LOG_PROCESS_KML &> /dev/null
    if [ $? -eq 0 ]; then
        echo "An error occured in the script process_kml.bash. Refer to the log file "$LOG_PROCESS_KML" for more details."
        
        cd -

        exit 1
    fi
fi

# Process the subsets
echo $process_flags | grep ":subsets:" &> /dev/null
if [ $? -eq 0 ]; then 
    /bin/bash $MIPS_HOME/scripts/process_subsets.bash &> $LOG_PROCESS_SUBSETS
    grep ERROR $LOG_PROCESS_SUBSETS &> /dev/null
    if [ $? -eq 0 ]; then
        echo "An error occured in the script process_subsets.bash. Refer to the log file "$LOG_PROCESS_SUBSETS" for more details."
         
        cd -

        exit 1
    fi
fi

# Process polar mosaic
echo $process_flags | grep ":polarmosiac:" &> /dev/null
if [ $? -eq 0 ]; then 
    /bin/bash $MIPS_HOME/scripts/process_polar_mosaic.bash &> $LOG_PROCESS_POLAR_MOSAIC
    grep ERROR $LOG_PROCESS_POLAR_MOSAIC &> /dev/null
    if [ $? -eq 0 ]; then
        echo "An error occured in the script process_polar_mosaic.bash. Refer to the log file "$LOG_PROCESS_POLAR_MOSAIC" for more details."
          
        cd -

        exit 1
    fi
fi

# Process global mosaic
echo $process_flags | grep ":globalmosaic:" &> /dev/null
if [ $? -eq 0 ]; then 
    /bin/bash $MIPS_HOME/scripts/process_mosaic.bash &> $LOG_PROCESS_GLOBAL_MOSAIC
    grep ERROR $LOG_PROCESS_GLOBAL_MOSAIC &> /dev/null
    if [ $? -eq 0 ]; then
        echo "An error occured in the script process_global_mosaic.bash. Refer to the log file "$LOG_PROCESS_GLOBAL_MOSAIC" for more details."
          
        cd -

        exit 1
    fi
fi

cd -

